'use strict'

vevo.directive 'ngPlaylistSelectModal', ['$rootScope', '$timeout', 'Playlists', 'API', 'ComScore', 'App'
, ($rootScope, $timeout, Playlists, API, ComScore, App) ->

    restrict: 'A'
    scope: true

    controller: ($scope) ->

        $scope.init = =>
            $timeout -> $scope.handleInit()

        $scope.$on 'TvInput.ngPlaylistSelectModal.navigate', (e, data) =>
            $scope.handleNavigate(data)


        $scope.$on 'TvInput.ngPlaylistSelectModal.enterKey', =>
            $scope.handleEnterKey()

    link: (scope, el, attrs) ->

        _playlists = []
        _createPlaylistNavEl = el.find('#nav-create-playlist')
        _selectPlaylistNavEl = el.find('#nav-select-playlist')
        _curFocusEl = ''
        _distance = 0
        _wrapperEl = el.find('#playlist-select-modal-view > .wrapper')

        scope.handleInit = =>
            # auto select first playlist if there is one

            $rootScope.$broadcast 'ngPlaylistSelectModal.ngVideoModal.hideVideoElements'

            userPlaylists = Playlists.getUserPlaylists()

            arr = []

            for playlist in userPlaylists
                obj =
                    id: playlist.playlist_id
                    title: playlist.title
                arr.push obj

            _playlists = arr

            scope.playlists = _playlists

        scope.handleNavigate = (data) =>
            key = data.key
            if key is 'up'
                _navigateUp()
            else if key is 'down'
                _navigateDown()

        scope.handleEnterKey = =>
            itemEl = _getCurFocusEl()
            event = itemEl.attr 'data-enter'
            if event is 'showCreatePlaylistModal'
                $timeout -> $rootScope.subModal = 'playlistCreate'
            else if event is 'selectPlaylistToAddVideo'
                playlistId = _curFocusEl.attr 'data-playlist-id'
                playlistName = _curFocusEl.attr 'data-playlist-title'

                if App.VideoModal.isAddCurPlayingVideoToPlaylist
                    videoData = App.VideoModal.data
                else
                    videoData = App.VideoModal.videoList[App.VideoModal.curFocusOnSliderIndex]

                params =
                    playlistId: playlistId
                    isrcs: videoData.isrc
                    image_url: videoData.image_url

                Playlists.addToUserPlaylist params, =>
                    $rootScope.$broadcast 'ngPlaylistSelectModal.ngConfirmMessage.addedToPlaylist',
                        playlistName: playlistName
                        title: videoData.title
                    $timeout -> $rootScope.subModal = ''
                    ComScore.trackCustomEvent
                        watch_page_events: 'add_to_playlist'



        _navigateDown = =>

            itemEl = _getCurFocusEl()

            if itemEl.hasClass 'create-playlist-item'
                itemEl.removeClass 'cur-focus'
                nextItemEl = _selectPlaylistNavEl.find('.nav-item:first')
                nextItemEl.addClass 'cur-focus'
                _curFocusEl = nextItemEl

            else if itemEl.hasClass 'playlist-item'
                nextItemEl = itemEl.next('.nav-item')
                return if not nextItemEl.size()

                itemEl.removeClass 'cur-focus'
                nextItemEl.addClass 'cur-focus'
                _curFocusEl = nextItemEl

                _distance -= 64
                _wrapperEl.css
                    top: _distance

        _navigateUp = =>

            itemEl = _getCurFocusEl()

            if itemEl.hasClass 'playlist-item'

                prevItemEl = itemEl.prev('.nav-item')

                if prevItemEl.size()
                    itemEl.removeClass 'cur-focus'
                    prevItemEl.addClass 'cur-focus'
                    _curFocusEl = prevItemEl

                    _distance += 64
                    _wrapperEl.css
                        top: _distance
                else
                    itemEl.removeClass 'cur-focus'
                    prevItemEl = _createPlaylistNavEl.find('.nav-item:first')
                    prevItemEl.addClass 'cur-focus'
                    _curFocusEl = prevItemEl



        _getCurFocusEl = =>
            if _curFocusEl then return _curFocusEl
            _curFocusEl = el.find('.cur-focus:first')
            _curFocusEl




]

