'use strict'

vevo.directive 'ngPlaylists', ['$rootScope', '$timeout', 'Playlists', 'API', 'Storage', 'UI', 'Nav', ($rootScope, $timeout, Playlists, API, Storage, UI, Nav) ->

    restrict: 'A'
    scope: true

    controller: ($scope) ->

        $scope.init = =>
            $timeout -> $scope.handleInit()

        $scope.$on 'TvInput.ngPlaylists.userLoggedOut', =>
            $scope.handleUserLoggedOut()

        $scope.$on 'ngActivateModal.ngPlaylists.userLoggedIn', =>
            $scope.handleUserLoggedIn()

        $scope.$on 'Nav.ngPlaylists.navigateDown', =>
            $scope.handleNavigateDown()

        $scope.$on 'Nav.ngPlaylists.navigateUp', =>
            $scope.handleNavigateUp()

        $scope.$on 'Playlists.ngPlaylists.reloadViewAfterCreatingNewPublicPlaylistForUser', =>
            $scope.handleReloadViewForUser()

        $scope.$on 'TvInput.ngPlaylists.initVideoModalFromPlaylistForUser', (e, data) =>
            $scope.handleInitVideoModalFromPlaylistForUser(data)

        $scope.$on 'TvInput.ngPlaylists.initVideoModalFromPlaylistForPublic', =>
            $scope.handleInitVideoModalFromPlaylistForPublic()

        $scope.$on 'Nav.ngPlaylists.hideDeleteOption', =>
            $scope.handleHideDeleteOption()

        $scope.$on 'Nav.ngPlaylists.showDeleteOption', =>
            $scope.handleShowDeleteOption()

        $scope.$on 'TvInput.ngPlaylists.showCreateModal', =>
            $scope.handleShowCreateModal()

        $scope.$on 'TvInput.ngPlaylists.showDeleteModal', =>
            $scope.handleShowDeleteModal()

        $scope.$on 'ngPlaylistCreateModal.ngPlaylists.shiftFocusToTopNav', =>
            $scope.handleShiftFocusToTopNav()

        $scope.$on 'ngPlaylistDeleteModal.ngPlaylists.shiftFocusToTopNav', =>
            $scope.handleShiftFocusToTopNav()

        $scope.$on 'TvInput.ngPlaylists.shiftFocusToTopNav', =>
            $scope.handleShiftFocusToTopNav()

        $scope.$on 'ngPlaylistCreateModal.ngPlaylists.refocusOnSelected', =>
            $scope.handleRefocusOnSelected()

        $scope.$on 'ngPlaylistDeleteModal.ngPlaylists.deletedPlaylist', =>
            $scope.handleDeletedPlaylist()

        $scope.$on 'ngPlaylistCreateModal.ngPlaylists.reloadViewForUser', =>
            $scope.handleReloadViewForUser()

        $scope.$on '$destroy', =>


    link: (scope, el, attrs) ->

        viewEl = el.find('#playlists-view')
        footerViewEl = el.find('.footer-view:first')

        scope.handleInit = =>

            if $rootScope.storage.username
                # logged in
                scope.showUserPlaylists()
            else
                # not logged in
                scope.showPublicPlaylist()


        scope.handleUserLoggedIn = =>
            Playlists.fetchUserPlaylists =>
                scope.showUserPlaylists()

        scope.handleUserLoggedOut = =>
            scope.showPublicPlaylist()

        scope.showPublicPlaylist = =>
            playlistArr = []
            publicPlaylist = Playlists.getPublicPlaylist()
            playlistArr.push publicPlaylist
            $timeout -> scope.playlistArr = playlistArr

        scope.showUserPlaylists = =>
            userPlaylists = Playlists.getUserPlaylists()
            $timeout -> scope.playlistArr = userPlaylists

        scope.handleNavigateDown = =>
            top = Number(viewEl.attr('data-top')) - 347
            viewEl.attr('data-top', top)
            viewEl.animate
                top: top

        scope.handleNavigateUp = =>
            top = Number(viewEl.attr('data-top')) + 347
            viewEl.attr('data-top', top)
            viewEl.animate
                top: top

        scope.handleReloadViewForUser = =>
            playlistArr = Playlists.getUserPlaylists()
            $timeout -> scope.playlistArr = playlistArr

        scope.handleInitVideoModalFromPlaylistForUser = (data) =>

            playlistId = data.playlistId

            API.playlists.getSinglePlaylist {playlistId: playlistId}, (data) =>

                videosArr = data.videos
                return if not videosArr.length

                Playlists.setCurPlaylistVideos videosArr

                # show video modal after gathering all needed info
                $timeout -> $rootScope.modal = 'video'

        scope.handleInitVideoModalFromPlaylistForPublic = =>
            storageData = Storage.get()
            videosArr = storageData.publicPlaylist.videos
            return if not videosArr.length
            Playlists.setCurPlaylistVideos videosArr
            $timeout -> $rootScope.modal = 'video'

        scope.handleHideDeleteOption = =>
            Playlists.setIsFocusOnPlaylist false
            $timeout -> scope.displayDeleteOption = false

        scope.handleShowDeleteOption = =>
            Playlists.setIsFocusOnPlaylist true
            $timeout -> scope.displayDeleteOption = true

        scope.handleShowCreateModal = =>
            $rootScope.$apply -> $rootScope.subModal = 'playlistCreate'

        scope.handleShowDeleteModal = =>
            itemEl = el.find('.nav-current-item-focus:first')
            Playlists.playlistIdToDelete = itemEl.attr 'data-playlist-id'
            Playlists.playlistNameToDelete = itemEl.attr 'data-playlist-title'
            $rootScope.$apply -> $rootScope.subModal = 'playlistDelete'

        scope.handleShiftFocusToTopNav = =>

            el.find('.nav-current-item-focus:first').removeClass('nav-current-item-focus')

            # focus on the top nav
            focusItemEl = angular.element('#nav-top .nav-current-item-focus:first')
            if not focusItemEl.size()
                focusItemEl = angular.element('#nav-top .nav-current-item-selected:first')

            focusItemEl.addClass 'nav-current-item-focus'
            Nav.curNavFocusItemEl = focusItemEl
            Nav.curNavZone = 'nav-top'
            viewEl.attr('data-top', 0)
            viewEl.animate
                top: 0
            scope.handleHideDeleteOption()
            
        scope.handleRefocusOnSelected = =>
            focusItemEl = el.find('.nav-current-item-focus:first')
            Nav.curNavFocusItemEl = focusItemEl
            Nav.curNavZone = focusItemEl.parent().attr 'id'

        scope.handleDeletedPlaylist = =>
            arr = scope.playlistArr
            newPlaylistArr = []

            playlistId = Playlists.playlistIdToDelete

            for playlist in arr
                if playlist.playlist_id isnt playlistId
                    newPlaylistArr.push playlist

            $timeout -> scope.playlistArr = newPlaylistArr
            scope.handleHideDeleteOption()




]

