'use strict'

vevo.directive 'ngSearch', ['$rootScope', '$timeout', 'UI', 'Nav', 'API', 'ComScore', ($rootScope, $timeout, UI, Nav, API, ComScore) ->

    restrict: 'A'
    scope: true

    controller: ($scope) ->

        $scope.init = =>
            $timeout -> $scope.handleInit()


        $scope.$on 'TvInput.ngSearch.focusInSearchBox', => $scope.handleFocusInSearchBox()

        $scope.$on 'app.updateNavZone', => $scope.handleUpdateNavZone()

        $scope.$on 'Nav.ngSearch.updateSectionFocus', (e, data) => $scope.handleUpdateSectionFocus(data)

        $scope.$on 'TvInput.ngSearch.handleReturnKeyOnSearchView', => $scope.handleReturnKeyOnSearchView()

        $scope.$on 'Nav.ngSearch.handleReturnKeyOnSearchView', => $scope.handleReturnKeyOnSearchView()

        $scope.$on 'ngKeypad.ngSearch.updateKeypadStr', (e, data) => $scope.handleUpdateKeypadStr(data)

        $scope.$on 'ngPlaylistCreateModal.ngSearch.refocusOnSelected', => $scope.handleRefocusOnSelected()

        $scope.$on 'TvInput.ngSearch.comScoreTrackSearch', (e, data) => $scope.handleComScoreTrackSearch(data)

        $scope.$on '$destroy', => $scope.handleDestroy()


    link: (scope, el, attrs) ->

        scope.removeElementToAllowMoreKeypad = false

        scope.initHide = true
        scope.artistArr = []
        scope.videoArr = []
        scope.search = ''
        scope.isHideSearchField = false

        _totalSearchedArtist = 0
        _totalSearchedVideo = 0
        _keypadStr = ''


        topNavViewEl = $('#top-nav-view')
        topNavEl = $('#nav-top')
        searchCursorEl = $('#search-cursor')
        _cursorInterval = ''

        searchMode = true
        viewEl = el.find('#search-view')

        scope.handleInit = =>

            $timeout -> scope.handleFocusInSearchBox()


        scope.handleFocusInSearchBox = =>
            if scope.removeElementToAllowMoreKeypad
                $timeout -> scope.removeElementToAllowMoreKeypad = false


            $timeout ->
                el.find('.keypad-box').animate
                    opacity: 1
                scope.isHideSearchField = false


            UI.animateOut topNavViewEl
            searchMode = true

            Nav.curNavZone = 'nav-keypad'
            $timeout -> $rootScope.$broadcast 'app.updateNavZone'

            Nav.curNavFocusItemEl.removeClass 'nav-current-item-focus'

            _cursorInterval = setInterval =>
                searchCursorEl.animate {opacity: 0}, ->
                    searchCursorEl.animate
                        opacity: 1
            , 1000

            # this is for reopening the keypad to make sure Keypad service gets initialized again
            scope.$broadcast 'ngSearch.ngKeypad.reopen'

                    

        scope.handleFocusOutSearchBox = =>
            $timeout -> scope.removeElementToAllowMoreKeypad = true

            UI.animateIn topNavViewEl

            Nav.curNavZone = 'nav-top'
            $timeout ->
                $rootScope.$broadcast 'app.updateNavZone'
                scope.isHideSearchField = true

            Nav.curNavFocusItemEl.removeClass 'nav-current-item-focus'
            topNavViewEl.find('.nav-item').removeClass('nav-current-item-focus').removeClass('nav-current-item-selected')
            newCurNavFocusEl = topNavViewEl.find('.search-nav-item')
            newCurNavFocusEl.addClass 'nav-current-item-focus'
            newCurNavFocusEl.addClass 'nav-current-item-selected'
            Nav.curNavFocusItemEl = newCurNavFocusEl

            $rootScope.$broadcast 'ngSearch.ngKeypad.animateOut'

            clearInterval _cursorInterval


        scope.handleUpdateNavZone = =>
            scope.$apply => scope.curNavZone = Nav.curNavZone




        scope.handleUpdateSectionFocus = (data) =>
            scope.$apply => scope.sectionFocus = data.list

        scope.handleReturnKeyOnSearchView = =>
            if searchMode
                searchMode = false
            else
                viewEl.attr('data-top', 0)
                viewEl.animate
                    top: 0

            scope.handleFocusOutSearchBox()

        scope.handleUpdateKeypadStr = (data) =>
            action = data.action
            isMaxStr = scope.search.length >= 19
            return if isMaxStr and action in ['add', 'space']
            switch action
                when 'add'
                    char = data.char
                    _keypadStr += char
                when 'space'
                    _keypadStr += ' '
                when 'delete'
                    _keypadStr = _keypadStr.substring(0, _keypadStr.length - 1)
                when 'clear'
                    _keypadStr = ''
            scope.$apply => scope.search = _keypadStr

        scope.handleRefocusOnSelected = =>
            focusItemEl = el.find('.nav-current-item-focus:first')
            Nav.curNavFocusItemEl = focusItemEl
            Nav.curNavZone = focusItemEl.parent().attr 'id'

        scope.handleComScoreTrackSearch = (data) =>
            search = scope.search
            return if not search

            ComScore.trackSearch
                ns_search_term: search
                search_value: data.name
                search_num_artist: _totalSearchedArtist
                search_num_video: _totalSearchedVideo


        scope.$watch 'search', (newVal, oldVal) =>


            if newVal.length < 3
                newVal = ''

            API.search.getSearchResult {query: newVal}, (data) =>

                artistArr = data.Artists
                videoArr = data.Videos
                _totalSearchedArtist = artistArr.length
                _totalSearchedVideo = videoArr.length

                # show only 9 max search results
                artistArr = artistArr.slice(0, 9)
                videoArr = videoArr.slice(0, 9)

                searchRowEl = el.find('.search-row')

                artistRowEl = $('#nav-search-artist')
                videoRowEl = $('#nav-search-video')


                if not artistArr.length and not videoArr.length
                    # no search result for artist and video
                    $rootScope.topNavLeaveDownSearch = ''

                    scope.artistArr = artistArr
                    scope.videoArr = videoArr

                else if artistArr.length and videoArr.length

                    
                    $rootScope.topNavLeaveDownSearch = 'nav-search-artist'


                    scope.artistRowIndex = '0'
                    scope.videoRowIndex = '1'
                    scope.artistLeaveDown = 'nav-search-video'
                    scope.videoLeaveUp = 'nav-search-artist'
                    scope.artistArr = artistArr
                    scope.videoArr = videoArr

                else if artistArr.length
                    $rootScope.topNavLeaveDownSearch = 'nav-search-artist'

                    scope.artistRowIndex = '0'
                    scope.videoRowIndex = ''
                    scope.artistLeaveDown = 'nav-search-video'
                    scope.artistArr = artistArr
                    scope.videoArr = videoArr

                else if videoArr.length


                    $rootScope.topNavLeaveDownSearch = 'nav-search-video'

                    scope.artistRowIndex = ''
                    scope.videoRowIndex = 0
                    scope.artistLeaveDown = ''
                    scope.videoLeaveUp = 'nav-top'
                    scope.artistArr = artistArr
                    scope.videoArr = videoArr

                scope.initHide = false


        scope.handleDestroy = =>
            # clear setInterval
            clearInterval _cursorInterval

            search = scope.search
            return if not search
            ComScore.trackSearch {search_failure: search}


]

