'use strict'

vevo.directive 'ngVevoTv', ['$rootScope', '$timeout', 'Player', 'UI', 'App', 'Nav', 'Playlists', 'Util', 'ComScore', 'API'
, ($rootScope, $timeout, Player, UI, App, Nav, Playlists, Util, ComScore, API) ->

    restrict: 'A'
    scope: true

    controller: ($scope) ->

        $scope.init = =>
            $timeout -> $scope.handleInit()


        $scope.$on 'TvInput.ngVevoTv.hideVevoTvElements', => $scope.handleHideVevoTvElements()

        $scope.$on 'TvInput.ngVevoTv.showVevoTvElements', => $scope.handleShowVevoTvElements()

        $scope.$on 'TvInput.ngVevoTv.resetFullScreenTimer', => $scope.handleResetFullScreenTimer()

        $scope.$on 'app.updateNavZone', => $scope.handleUpdateNavZone()

        $scope.$on 'Player.ngVevoTv.pausedVevoTv', => $scope.handlePausedVevoTv()

        $scope.$on 'Player.ngVevoTv.playedVevoTv', => $scope.handlePlayedVevoTv()

        $scope.$on 'Player.ngVevoTv.playedVevoTvAd', => $scope.handlePlayedVevoTvAd()

        $scope.$broadcast 'FreeWheel.ngVevoTv.playedVevoTvAfterAd', => $scope.handlePlayedVevoTvAfterAd()

        $scope.$on 'TvInput.ngVevotv.addToPlaylist', => $scope.handleAddToPlaylist()

        $scope.$on '$destroy', => $scope.handleDestroy()

    link: (scope, el, attrs) ->

        _timeout = ''
        _darkBackgroundEl = el.find('.dark-background')
        _metadataEl = el.find('#tv-metadata')
        _footerViewEl = el.find('.footer-view')
        _navScheduleEl = el.find('#nav-schedule')
        _adCounterEl = el.find('.ad-counter')
        _vevoTvScheduleOnDisplay = false
        _isInitLoad = true
        _metadataTimeout = ''

        scope.schedules = []
        scope.actionClass = 'pause'

        scope.handleInit = =>

            ComScore.initStreamSensePlaylistForVevoTv()

            $rootScope.loading = $rootScope.strings.loadingVevoTv
            $rootScope.loadingType = 'vevotv'
            Player.play()
            scope.handleResetFullScreenTimer()
            App.VevoTv.isFullScreenMode = true


        _resetSetTimeout = =>
            _handleCancelSetTimeout()

            _timeout = $timeout ->
                scope.handleHideVevoTvElements()
            , 7000

        _handleCancelSetTimeout = =>
            $timeout.cancel _timeout

        # init hide elements timer
        _resetSetTimeout()


        scope.handleHideVevoTvElements = =>
            _hideMetadata()
            _hideControls()
            _hideSchedule()
            _hideDarkBackground()
            $rootScope.$broadcast 'ngVevoTv.ngTopNav.hideTopNav'
            App.VevoTv.vevoTvInfoDisplayed = false
            App.VevoTv.isFullScreenMode = true

        scope.handleShowVevoTvElements = =>
            _showMetadata()
            _showControls()
            _showSchedule()
            _showDarkBackground()
            $rootScope.$broadcast 'ngVevoTv.ngTopNav.showTopNav'
            _resetSetTimeout()
            App.VevoTv.vevoTvInfoDisplayed = true
            App.VevoTv.isFullScreenMode = false

        scope.handleResetFullScreenTimer = =>
            _resetSetTimeout()

        scope.handleUpdateNavZone = =>
            scope.$apply => scope.curNavZone = Nav.curNavZone

        _showMetadata = =>
            videoData = App.VevoTv.metadata
            return if not videoData

            scope.artist = ''
            scope.title = ''
            scope.director = ''
            scope.explicit = ''

            fullArtistName = Util.getFullArtistName videoData

            scope.$apply ->
                scope.title = videoData.title
                scope.artist = fullArtistName
                scope.explicit = videoData.explicit

            creditArr = videoData.credit
            if creditArr.length
                for credit in creditArr
                    if credit.Key is 'Director'
                        scope.$apply -> scope.director = credit.Value
                        break

            UI.animateIn _metadataEl
            _metadataEl.css
                'background-color': 'rgba(252,0,149,.7)'

        _showDarkBackground = =>
            return
            UI.animateIn _darkBackgroundEl


        _hideMetadata = =>
            UI.animateOut _metadataEl

        _hideDarkBackground = =>
            UI.animateOut _darkBackgroundEl


        _showControls = =>
            UI.animateIn _footerViewEl, {opacity:0.8}

        _hideControls = =>
            UI.animateOut _footerViewEl

        _showSchedule = (arr) =>
            return
            arr = App.VevoTv.scheduleArr

            len = arr.length
            return if not len

            if not scope.schedules.length and not _vevoTvScheduleOnDisplay
                _vevoTvScheduleOnDisplay = true

                scheduleArr = []
                now = new Date()
                for schedule in arr
                    date = new Date(schedule.starttime)
                    if date > now || date.getHours() is now.getHours()
                        scheduleArr.push schedule

                scope.$apply -> scope.schedules = scheduleArr

            UI.animateIn _navScheduleEl

        _hideSchedule = =>
            UI.animateOut _navScheduleEl



        scope.handlePausedVevoTv = =>
            scope.$apply -> scope.actionClass = 'play'
            
        scope.handlePlayedVevoTv = =>
            scope.$apply -> scope.actionClass = 'pause'

            # only run the code below once 
            return if not _isInitLoad

            _resetSetTimeout()

            _getSchedule()

            _getMetadata()

        _getSchedule = =>
            API.vevotv.getSchedule (data) =>
                App.VevoTv.scheduleArr = data

        _handleNewIsrc = (curVideoData) =>
            return if App.getView() isnt 'vevotv'

            curIsrc = curVideoData.isrc

            App.VevoTv.isrc = curIsrc

            if curIsrc is 'TIVEVSLATE00' and not _isInitLoad
                # play vevotv ad
                App.VevoTv.metadata = ''
                $rootScope.$broadcast 'ngVevoTv.FreeWheel.initPlayVevoTvAd'
                return

            App.VevoTv.metadata = curVideoData

            if curIsrc isnt 'TIVEVSLATE00'
                # if not an ad

                ComScore.trackVevoTv {action: 'newClipPlay'}

                # for comScore to keep track of when playlist begins
                App.VevoTv.curClipStartTime = new Date()

            _getMetadata()

        _getMetadata = =>
            return if App.getView() isnt 'vevotv'

            API.vevotv.getMetadata (metadataArr) =>
                return if not metadataArr.length

                utc = new Date().getTime()
                utc = Math.floor(utc/1000)

                tempVideoData = {}
                curVideoData = {}
                nextVideoData = {}
                startTime = ''
                firstObj = metadataArr[0]

                if firstObj.StartTime is 0
                    isLastItemInArray = true
                    curVideoData = metadataArr[metadataArr.length - 1]
                    curVideoData = curVideoData.Video
                    startTime = 0

                else
                    for obj, index in metadataArr
                        if index is (metadataArr.length - 1)
                            # last video in array
                            curVideoData = obj.Video
                            isLastItemInArray = true
                            startTime = obj.StartTime
                            break
                        else if utc < obj.StartTime
                            # found in between two clips
                            curVideoData = tempVideoData
                            nextVideoData = metadataArr[index + 1]
                            isLastItemInArray = false
                            break
                        else
                            # continue loop
                            tempVideoData = obj.Video

                if App.VevoTv.isrc isnt curVideoData.isrc

                    # add 10 sec delay
                    delay = if App.VevoTv.isrc then 10000 else 0
                    $timeout =>
                        _handleNewIsrc curVideoData
                    , delay
                    return

                _isInitLoad = false

                if isLastItemInArray
                    # call api again every 6 seconds
                    _metadataTimeout = $timeout =>
                        _getMetadata()
                    , 6000

                # TODO: add else case to handle not last item in array


        scope.handlePlayedVevoTvAd = =>
            $rootScope.$broadcast 'ngVevoTv.ngTopNav.hideTopNav'
            _hideDarkBackground()
            _showAdTimer()

        scope.handlePlayedVevoTvAfterAd = =>
            _getMetadata()

        _showAdTimer = =>
            return if scope.counter

            _countDown = =>
                $timeout ->
                    scope.counter--

                    if scope.counter > 0
                        _countDown()
                    else
                        UI.animateOut _adCounterEl
                        scope.counter = 0

                        $timeout ->
                            if App.VevoTv.adInProgress
                                Player.clear()
                                App.VevoTv.adInProgress = false
                                App.VevoTv.adPlayPosition = 0
                                App.VevoTv.adArr = []
                                Player.play()
                                _getMetadata()
                        , 4000

                        ## return to vevo tv
                        #Player.clear()
                        #App.VevoTv.adInProgress = false
                        #App.VevoTv.adPlayPosition = 0
                        #App.VevoTv.adArr = []
                        #Player.play()
                        #_getMetadata()

                , 1000

            scope.counter = 32

            _countDown()

            UI.animateIn _adCounterEl, {opacity: .5}


        scope.handleAddToPlaylist = =>
            return if App.VevoTv.adInProgress

            videoData = App.VevoTv.metadata
            if $rootScope.storage.username
                Playlists.addToUserSamsungPlaylist videoData
            else
                Playlists.addToPublicPlaylist videoData

            scope.handleHideVevoTvElements()

            ComScore.trackCustomEvent
                vevotv_events: 'watch_later'


        scope.handleDestroy = =>

            clearInterval Player.vevoTvFreeWheelInterval

            _handleCancelSetTimeout()
            if Player.state is Player.PLAYING or Player.state is Player.PAUSED
                Player.clear()
            App.VevoTv.vevoTvInfoDisplayed = false

            App.VevoTv.starttime = ''
            ComScore.endVevoTv()
            $timeout.cancel _metadataTimeout


]







