'use strict'

vevo.directive 'ngVideoModal', ['$rootScope', '$timeout', 'Player', 'API', 'UI', 'App', 'Playlists', 'Util', 'ComScore', 'K', 'Storage'
, ($rootScope, $timeout, Player, API, UI, App, Playlists, Util, ComScore, K, Storage) ->

    restrict: 'A'
    scope: true

    controller: ($scope) ->

        $scope.init = =>
            $timeout -> $scope.handleInit()

        $scope.$on 'TvInput.ngVideoModal.hideVideoElements', =>
            $scope.handleHideVideoElements()

        $scope.$on 'ngVideoModalBrowseSlider.ngVideoModal.hideVideoElements', =>
            $scope.handleHideVideoElements()

        $scope.$on 'ngVideoModalSearchSlider.ngVideoModal.hideVideoElements', =>
            $scope.handleHideVideoElements()

        $scope.$on 'ngVideoModalPlaylistSlider.ngVideoModal.hideVideoElements', =>
            $scope.handleHideVideoElements()

        $scope.$on 'ngPlaylistSelectModal.ngVideoModal.hideVideoElements', =>
            $scope.handleHideVideoElements()

        $scope.$on 'TvInput.ngVideoModal.showVideoElements', =>
            $scope.handleShowVideoElements()

        $scope.$on 'TvInput.ngVideoModal.resetFullScreenTimer', =>
            $scope.handleResetFullScreenTimer()

        $scope.$on 'Player.ngVideoModal.playedVideo', =>
            $scope.handlePlayedVideo()

        $scope.$on 'Player.ngVideoModal.playedVideoAd', =>
            $scope.handlePlayedVideoAd()

        $scope.$on 'Player.ngVideoModal.pausedVideo', =>
            $scope.handlePausedVideo()

        $scope.$on 'Player.ngVideoModal.updateProgressBar', (e, data) =>
            $scope.handleUpdateProgressBar(data)

        $scope.$on 'TvInput.ngVideoModal.addToPlaylist', =>
            $scope.handleAddToPlaylist()

        $scope.$on 'Player.ngVideoModal.showTimeBar', =>
            $scope.handleShowTimeBar()

        $scope.$on 'ngVideoModalBrowseSlider.ngVideoModal.triggerPlayVideo', =>
            $scope.handleTriggerPlayVideo()

        $scope.$on 'ngVideoModalPlaylistSlider.ngVideoModal.triggerPlayVideo', =>
            $scope.handleTriggerPlayVideo()

        $scope.$on 'ngVideoModalSearchSlider.ngVideoModal.triggerPlayVideo', =>
            $scope.handleTriggerPlayVideo()

        $scope.$on 'FreeWheel.ngVideoModal.triggerPlayVideo', =>
            $scope.handleTriggerPlayVideo()

        $scope.$on 'ngConfirmExplicitModal.ngVideoModal.doneConfirmingExplicit', =>
            $scope.handleDoneConfirmingExplicit()


        $scope.$on '$destroy', => $scope.handleDestroy()

        # [BrightLine] Start
        BL.onOverlayClose = =>
            remoteEl = document.getElementById('vevo-remote-key-handler')
            remoteEl.focus()

        BL.onMicrositeOpen = =>
            Player.clear()

        BL.onMicrositeClose = =>
            remoteEl = document.getElementById('vevo-remote-key-handler')
            remoteEl.focus()
            $rootScope.$broadcast 'Player.FreeWheel.videoAdEnded'
            $scope.handleTriggerPlayVideo()
        # [BrightLine] End

    link: (scope, el, attrs) ->

        scope.actionClass = 'pause'

        _continuousPlayWrapEl = el.find('#continuous-play-wrap')
        _darkBackgroundEl = el.find('.dark-background:first')
        _footerViewEl = el.find('.footer-view')
        _videoMetadataEl = el.find('#video-metadata')
        _adCounterEl = el.find('.ad-counter')

        _timeout = ''

        # used for making sure the video loads
        _checkVideoTimeout = ''

        scope.handleInit = =>
            if App.deepLinking
                return _handleDeepLinking()

            curView = App.getView()

            App.VideoModal.isFullScreenMode = true

            ComScore.initStreamSensePlaylistForVideo curView

            switch curView
                when 'browse'

                    section = $rootScope.app.videoModalType
                    if section is 'genres'
                        # track if entered from genres
                        ComScore.trackCustomEvent
                            browse_videos_events: $rootScope.app.videoModalGenreType


                    API.video.getAllVideoOnDemandData (data) =>

                        
                        streamData = data.streamData
                        videoData = data.videoData

                        
                        App.VideoModal.url = streamData.renditions[0].url


                        App.VideoModal.data = videoData

                        scope.handleTriggerPlayVideo()

                        $timeout ->
                            $rootScope.$broadcast 'ngVideoModal.ngVideoModalBrowseSlider.initSlider', videoData
                        , 2000

                when 'browseShow'
                    showName = $rootScope.app.videoModalShowName
                    showName = showName.split(' ').join('_').toLowerCase()
                    ComScore.trackCustomEvent
                        browse_shows_events: showName

                    API.video.getAllVideoOnDemandData (data) =>

                        
                        streamData = data.streamData
                        videoData = data.videoData

                        
                        App.VideoModal.url = streamData.renditions[0].url

                        App.VideoModal.data = videoData

                        scope.handleTriggerPlayVideo()

                        $timeout ->
                            $rootScope.$broadcast 'ngVideoModal.ngVideoModalBrowseSlider.initSlider', videoData
                        , 2000

                when 'search'
                    type = App.VideoModal.searchType


                    if type is 'artist'

                        API.video.getArtistVideosData (data) =>

                            if data.success is false
                                scope.$broadcast 'ngVideoModal.ngConfirmMessage.videoError'
                                return


                            streamData = data.streamData
                            videoData = data.videoData

                            firstVideoData = videoData[0]

                            App.VideoModal.isrc = firstVideoData.isrc
                            arr = streamData.renditions
                            App.VideoModal.renditionsArr = arr
                            App.VideoModal.renditionsIndex = 0
                            App.VideoModal.url = arr[0].url

                            App.VideoModal.data = firstVideoData

                            scope.handleTriggerPlayVideo()
                            
                            $timeout ->
                                $rootScope.$broadcast 'ngVideoModal.ngVideoModalSearchSlider.initSlider', videoData
                            , 2000

                    else if type is 'video'
                        API.video.getAllVideoOnDemandData (data) =>

                            
                            streamData = data.streamData
                            videoData = data.videoData

                            
                            arr = streamData.renditions
                            App.VideoModal.renditionsArr = arr
                            App.VideoModal.renditionsIndex = 0
                            App.VideoModal.url = arr[0].url


                            App.VideoModal.data = videoData

                            scope.handleTriggerPlayVideo()

                            $timeout ->
                                $rootScope.$broadcast 'ngVideoModal.ngVideoModalSearchSlider.initSlider', videoData
                            , 2000

                when 'playlists'

                    curPlaylistVideos = Playlists.getCurPlaylistVideos()
                    App.VideoModal.isrc = curPlaylistVideos[0].isrc

                    API.video.getAllVideoOnDemandData (data) =>
                        streamData = data.streamData
                        videoData = data.videoData

                        
                        App.VideoModal.url = streamData.renditions[0].url


                        App.VideoModal.data = videoData

                        scope.handleTriggerPlayVideo()

                        $timeout ->
                            $rootScope.$broadcast 'ngVideoModal.ngVideoModalPlaylistSlider.initSlider', curPlaylistVideos
                        , 2000

        _handleDeepLinking = ->
            App.VideoModal.isrc = App.deepLinking

            API.video.getAllVideoOnDemandData (data) =>
                streamData = data.streamData
                videoData = data.videoData

                
                App.VideoModal.url = streamData.renditions[0].url


                App.VideoModal.data = videoData

                scope.handleTriggerPlayVideo()

        scope.handleTriggerPlayVideo = =>

            if App.deepLinking
                _handleTriggerPlayVideoForDeepLinking()
                return

            # play ad every 5th video
            if App.VideoModal.adCounter is 5
                # play ad
                App.VideoModal.adCounter = 0

                params =
                    isrc: App.VideoModal.isrc

                API.freeWheel.getVideoFreeWheel params, (data) =>

                    json = window.X2JS.xml_str2json data

                    adArr = json?.VAST?.Ad_asArray
                    App.VideoModal.adArr = adArr

                    if adArr and adArr.length
                        _playVideoAd()
                    else
                        _playVideo()

            else
                # play video
                App.VideoModal.adCounter++
                # [BrightLine] Start
                # Close any overlay or microsite that is open before playing a content video
                BL.close()
                # [BrightLine] End
                _playVideo()

        _handleTriggerPlayVideoForDeepLinking = =>
            # 2/3 chance of playing ad
            arr = [true, true, false]
            rand = Math.floor(Math.random() * 10) % 3
            playAd = arr[rand]

            if playAd and !App.playingAdForDeepLinking
                App.playingAdForDeepLinking = true
                App.VideoModal.adCounter = 0

                params =
                    isrc: App.VideoModal.isrc

                API.freeWheel.getVideoFreeWheel params, (data) =>

                    json = window.X2JS.xml_str2json data

                    adArr = json?.VAST?.Ad_asArray
                    App.VideoModal.adArr = adArr

                    if adArr and adArr.length
                        _playVideoAd()
                    else
                        _playVideo()
            else
                # play video
                App.playingAdForDeepLinking = false
                _playVideo()

        #_playVideoAd = =>
            #index = App.VideoModal.adPlayPosition
            #ad = App.VideoModal.adArr[index]

            #mediaFile = ad.InLine.Creatives.Creative.Linear.MediaFiles.MediaFile

            #src = mediaFile['__cdata']

            #App.VideoModal.adInProgress = true

            #curVideoTime = Player.getCurVideoTime()

            #ComScore.trackVideo
                #action: 'newAdClipPlay'
                #curVideoTime: curVideoTime

            #Player.playAd src

        _playVideoAd = =>
            # [BrightLine] Start
            # Support having more than one creative
            index = App.VideoModal.adPlayPosition
            ad = App.VideoModal.adArr[index]
            creatives = ad.InLine.Creatives.Creative_asArray

            parseCreative = (creative) ->
                mediaFile = (creative?.Linear?.MediaFiles?.MediaFile?.__cdata)
                htmlResource = (creative?.CompanionAds?.Companion?.HTMLResource?.__cdata)

                if mediaFile
                    App.VideoModal.adInProgress = true
                    curVideoTime = Player.getCurVideoTime()

                    ComScore.trackVideo
                        action: 'newAdClipPlay'
                        curVideoTime: curVideoTime

                    Player.playAd mediaFile

                else if htmlResource
                    BL.openOverlay htmlResource

            parseCreative creative for creative in creatives
            # [BrightLine] End

        _playVideo = =>
            ga_view = '/video/' + App.VideoModal.isrc + '-' + App.VideoModal.data.url_safe_title
            _gaq.push(['_trackPageview', ga_view])

            API.video.getVideoStreamData {isrc: App.VideoModal.isrc}, (data) =>
                url = data.url
                url = url.split '?'
                url = url[0]
                if /m3u8$/.test(url) then url = url + K.Stream.hlsFullSuffix

                App.VideoModal.url = url
                Player.clear()

                videoData = App.VideoModal.data
                isExplicit = videoData.explicit
                storageData = Storage.get()

                if isExplicit and not storageData.confirmExplicit
                    $timeout -> $rootScope.subModal = 'confirmExplicit'
                    return

                $rootScope.loading = $rootScope.strings.loading
                ComScore.trackVideo
                    action: 'newClipPlay'
                    curVideoTime: 0

                Player.play()
                _checkLoadingError()

        scope.handleDoneConfirmingExplicit = =>
            $rootScope.loading = $rootScope.strings.loading
            ComScore.trackVideo
                action: 'newClipPlay'
                curVideoTime: 0

            Player.play()
            _checkLoadingError()

        _checkLoadingError = =>

            # this is to check if video actually loads
            _checkVideoTimeout = $timeout ->
                if $rootScope.loading
                    # video stuck at loading
                    $rootScope.loading = false
                    scope.$broadcast 'ngVideoModal.ngConfirmMessage.videoError'
            , 10000


        _resetSetTimeout = =>
            _handleCancelSetTimeout()

            _timeout = $timeout ->
                scope.handleHideVideoElements()
            , 7000


        _handleCancelSetTimeout = =>
            $timeout.cancel _timeout

        scope.handleHideVideoElements = =>
            _hideMetadata()
            _hideControls()
            _hideContinuousPlay()
            _hideDarkBackground()
            App.VideoModal.videoInfoDisplayed = false
            App.VideoModal.isFullScreenMode = true

        scope.handleShowVideoElements = =>
            _showMetadata()
            _showControls()


            videoSection = $rootScope.app.videoModalType

            _showContinuousPlay()
            _showDarkBackground()

            _resetSetTimeout()
            App.VideoModal.videoInfoDisplayed = true
            App.VideoModal.isFullScreenMode = false

        scope.handleShowTimeBar = =>
            _showMetadata()
            _showControls()

        scope.handleResetFullScreenTimer = =>
            _resetSetTimeout()

        scope.handlePlayedVideo = =>

            _resetSetTimeout()
            $timeout.cancel _checkVideoTimeout
            totalTime = App.VideoModal.totalTime
            videoData = App.VideoModal.data

            totalMinute = Math.floor(totalTime/60)
            totalSecond = Math.floor(totalTime%60)

            totalTimeStr = totalMinute + ':'

            if totalSecond is 0
                totalTimeStr += '00'
            else if totalSecond < 10
                totalTimeStr += '0' + totalSecond
            else
                totalTimeStr += totalSecond

            isPremiere = videoData.premiere
            if isPremiere then _showPremiereTag()

            scope.$apply ->
                scope.actionClass = 'pause'
                scope.totalTimeStr = '/' + totalTimeStr
                scope.curTimeStr = '0:00'
                scope.explicit = videoData.explicit

            API.video.callSuperBeacon()

            ComScore.trackPageView {view: 'VOD'}

        scope.handlePlayedVideoAd = =>
            _hideDarkBackground()
            _showAdTimer()

        _showPremiereTag = =>
            premiereTagEl = el.find('.premiere-tag:first')
            premiereTagEl.css
                opacity: 1
            $timeout ->
                UI.animateOut premiereTagEl
            , 4000
            

        _showAdTimer = =>
            index = App.VideoModal.adPlayPosition
            ad = App.VideoModal.adArr[index]

            info = ad.InLine.Creatives.Creative.Linear
            duration = info.Duration

            _countDown = =>
                $timeout ->
                    scope.counter--

                    if scope.counter > 0
                        _countDown()
                    else
                        UI.animateOut _adCounterEl
                , 1000

            if duration is '00:00:30'
                scope.counter = 30
            else
                scope.counter = 15

            _countDown()

            UI.animateIn _adCounterEl, {opacity: .5}

        _showMetadata = =>


            videoData = App.VideoModal.data

            fullArtistName = Util.getFullArtistName videoData

            scope.$apply ->
                scope.title = videoData.title
                scope.artist = fullArtistName

            creditArr = videoData.credit
            if creditArr?.length
                for credit in creditArr
                    if credit.Key is 'Director'
                        scope.director = credit.Value
                        break

            UI.animateIn _videoMetadataEl
            _videoMetadataEl.css
                'background-color': 'rgba(252,0,149,.7)'


        _hideMetadata = =>
            UI.animateOut _videoMetadataEl, {speed: 'slow'}


        _showControls = =>
            UI.animateIn _footerViewEl

        _hideControls = =>
            UI.animateOut _footerViewEl, {speed: 'slow'}

        _showContinuousPlay = =>
            UI.animateIn _continuousPlayWrapEl

        _showDarkBackground = =>
            UI.animateIn _darkBackgroundEl

        _hideContinuousPlay = =>
            UI.animateOut _continuousPlayWrapEl, {speed: 'slow'}

        _hideDarkBackground = =>
            UI.animateOut _darkBackgroundEl, {speed: 'slow'}

        scope.handlePausedVideo = =>
            scope.$apply -> scope.actionClass = 'play'
            

        scope.handleUpdateProgressBar = (data) =>
            time = data.time
            timePercent = (100 * time) / App.VideoModal.totalTime

            curMinute = Math.floor(time/60)
            curSecond = Math.floor(time%60)

            curTimeStr = curMinute + ':'

            if curSecond is 0
                curTimeStr += '00'
            else if curSecond < 10
                curTimeStr += '0' + curSecond
            else
                curTimeStr += curSecond

            scope.$apply ->
                scope.progressBarPercent = timePercent + '%'
                scope.curTimeStr = curTimeStr

        scope.handleAddToPlaylist = =>
            return if App.VideoModal.adInProgress

            if App.VideoModal.isFullScreenMode
                App.VideoModal.isAddCurPlayingVideoToPlaylist = true
            else
                App.VideoModal.isAddCurPlayingVideoToPlaylist = false

            if $rootScope.storage.username

                # show playlist selection modal
                $timeout -> $rootScope.subModal = "playlistSelect"

            else
                Playlists.addToPublicPlaylist()
                scope.handleHideVideoElements()
                ComScore.trackCustomEvent
                    watch_page_events: 'add_to_playlist'

        scope.handleDestroy = =>

            App.VideoModal.isFullScreenMode = false
            App.VideoModal.videoBrowseType = ''
            _handleCancelSetTimeout()
            $timeout.cancel _checkVideoTimeout
            Player.clear()
            ComScore.endVideoPlay()




]

