'use strict'

vevo.directive 'ngVideoModalBrowseSlider', ['$rootScope', '$timeout', 'API', 'Player', 'App'
, ($rootScope, $timeout, API, Player, App) ->

    restrict: 'A'
    scope: true

    controller: ($scope) ->

        $scope.$on 'ngVideoModal.ngVideoModalBrowseSlider.initSlider', =>
            $scope.handleInitSlider()

        $scope.$on 'TvInput.ngVideoModalBrowseSlider.moveSlider', (e, data) =>
            $scope.handleMoveSlider(data)

        $scope.$on 'TvInput.ngVideoModalBrowseSlider.autoPlayPrev', =>
            $scope.handleAutoPlayPrev()


        $scope.$on 'Player.ngVideoModalBrowseSlider.autoPlayPrev', =>
            $scope.handleAutoPlayPrev()

        $scope.$on 'TvInput.ngVideoModalBrowseSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'Player.ngVideoModalBrowseSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'ngConfirmMessage.ngVideoModalBrowseSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'ngConfirmExplicitModal.ngVideoModalBrowseSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'TvInput.ngVideoModalBrowseSlider.selectVideoToPlay', =>
            $scope.handleSelectVideoToPlay()

        $scope.$on 'ngBrowse.ngVideoModalBrowseSlider.updateShowsList', =>
            $scope.handleUpdateShowsList()


    link: (scope, el, attrs) ->

        _curFocusIndex = 0
        _curIndexOnScreen = 0
        _curPlayingIndex = 0
        _distance = 0
        _curSelectedEl = ''
        _curFocusEl = ''
        _sessionId = ''
        _videoList = []
        _videoModalType = $rootScope.app.videoModalType


        scope.handleInitSlider = =>
            _videoList = []
            _sessionId = ''
            _curFocusIndex = 0
            _curIndexOnScreen = 0
            _curPlayingIndex = 0
            _distance = 0

            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex
            App.VideoModal.videoList = _videoList

            if _videoModalType is 'shows'
                _videoList = App.VideoModal.shows
                App.VideoModal.videoList = _videoList
                len = _videoList.length
                width = len * 300

                el.css
                    width: width
                $timeout => scope.videoList = _videoList

            else if _videoModalType is 'lift'
                artistsArr = App.VideoModal.browse.liftArtists
                liftIndex = $rootScope.app.videoModalLiftIndex
                _videoList = artistsArr[liftIndex].videos
                App.VideoModal.videoList = _videoList

                len = _videoList.length
                width = len * 300

                el.css
                    width: width
                $timeout -> scope.videoList = _videoList
            else
                API.video.initContinuousPlayVideos (res) =>
                    _sessionId = res.sessionId
                    _videoList = res.videoArr
                    App.VideoModal.videoList = _videoList

                    $timeout -> scope.videoList = _videoList


        scope.handleMoveSlider = (data) =>
            key = data.key

            if key is 'right'
                _browseNext()
            else if key is 'left'
                _browsePrev()

        _browseNext = =>

            itemEl = _getCurFocusEl()
            nextItemEl = itemEl.next('.nav-item')

            return if not nextItemEl.size()

            itemEl.removeClass 'cur-focus'
            nextItemEl.addClass 'cur-focus'
            _curFocusEl = nextItemEl

            if _curIndexOnScreen is 3
                _distance -= 273
                el.animate
                    left: _distance
            else
                _curIndexOnScreen++

            _curFocusIndex++
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex

            # set max continuous number of videos
            return if _videoList.length >= 25

            # check if 5th to last. if so then load more to list
            if _videoList.length - _curFocusIndex <= 5 and _videoModalType isnt 'shows' and _videoModalType isnt 'lift'
                API.video.getMoreContinuousPlayVideos {sessionId:_sessionId}, (dataArr) =>
                    $.merge _videoList, dataArr
                    App.VideoModal.videoList = _videoList

                    len = _videoList.length
                    width = len * 300

                    el.css
                        width: width

                    scope.videoList = _videoList

        _browsePrev = =>
            return if _curFocusIndex is 0

            itemEl = _getCurFocusEl()
            itemEl.removeClass 'cur-focus'
            prevItemEl = itemEl.prev('.nav-item')
            prevItemEl.addClass 'cur-focus'
            _curFocusEl = prevItemEl

            if _curIndexOnScreen is 0
                _distance += 273
                el.animate
                    left: _distance
            else
                _curIndexOnScreen--

            _curFocusIndex--
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex

        _getCurFocusEl = =>
            if _curFocusEl then return _curFocusEl
            _curFocusEl = el.find('.cur-focus:first')
            _curFocusEl

        _getCurSelectedEl = =>
            if _curSelectedEl then return _curSelectedEl
            _curSelectedEl = el.find('.cur-selected:first')
            _curSelectedEl

        scope.handleAutoPlayPrev = =>
            _curPlayingIndex--

            itemEl = _getCurSelectedEl()
            nextItemEl = itemEl.prev('.nav-item')

            # return to playlist view if no more next item
            if not nextItemEl.size()
                $timeout -> $rootScope.modal = ''
                return

            if itemEl.hasClass 'cur-focus'
                _browsePrev()

            itemEl.removeClass 'cur-selected'
            nextItemEl.addClass 'cur-selected'
            _curSelectedEl = nextItemEl

            videoData = _videoList[_curPlayingIndex]
            App.VideoModal.isrc = videoData.isrc
            App.VideoModal.data = videoData

            $rootScope.$broadcast 'ngVideoModalBrowseSlider.ngVideoModal.triggerPlayVideo'

            _distance = _curPlayingIndex * -273

            el.css
                left: _distance

            if not nextItemEl.hasClass 'cur-focus'
                curFocusEl = _getCurFocusEl()
                curFocusEl.removeClass 'cur-focus'
                nextItemEl.addClass 'cur-focus'
                _curFocusEl = nextItemEl

            _curIndexOnScreen = 0
            _curFocusIndex = _curPlayingIndex
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex

        scope.handleAutoPlayNext = =>

            _curPlayingIndex++

            itemEl = _getCurSelectedEl()
            nextItemEl = itemEl.next('.nav-item')

            if not nextItemEl.size()
                $timeout -> $rootScope.modal = ''
                return


            if itemEl.hasClass 'cur-focus'
                _browseNext()

            itemEl.removeClass 'cur-selected'
            nextItemEl.addClass 'cur-selected'
            _curSelectedEl = nextItemEl

            videoData = _videoList[_curPlayingIndex]

            App.VideoModal.isrc = videoData.isrc
            App.VideoModal.data = videoData

            $rootScope.$broadcast 'ngVideoModalBrowseSlider.ngVideoModal.triggerPlayVideo'

            _distance = _curPlayingIndex * -273

            el.css
                left: _distance

            if not nextItemEl.hasClass 'cur-focus'
                curFocusEl = _getCurFocusEl()
                curFocusEl.removeClass 'cur-focus'
                nextItemEl.addClass 'cur-focus'
                _curFocusEl = nextItemEl

            _curIndexOnScreen = 0
            _curFocusIndex = _curPlayingIndex
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex

        scope.handleSelectVideoToPlay = =>
            return if _curPlayingIndex is _curFocusIndex

            newVideoToPlayData = _videoList[_curFocusIndex]
            App.VideoModal.isrc = newVideoToPlayData.isrc
            App.VideoModal.data = newVideoToPlayData

            $rootScope.$broadcast 'ngVideoModalBrowseSlider.ngVideoModal.triggerPlayVideo'

            itemEl = _getCurSelectedEl()
            itemEl.removeClass 'cur-selected'
            newItemEl = _getCurFocusEl()
            newItemEl.addClass 'cur-selected'
            _curSelectedEl = newItemEl

            $rootScope.$broadcast 'ngVideoModalBrowseSlider.ngVideoModal.hideVideoElements'

            extraDistance = _curIndexOnScreen * 273
            _distance -= extraDistance

            $timeout ->
                el.animate
                    left: _distance
            , 1000

            _curIndexOnScreen = 0
            _curPlayingIndex = _curFocusIndex

        scope.handleUpdateShowsList = =>

            _videoList = App.VideoModal.shows
            App.VideoModal.videoList = _videoList

            len = _videoList.length
            width = len * 300

            el.css
                width: width

            scope.videoList = _videoList





]

