'use strict'

vevo.directive 'ngVideoModalPlaylistSlider', ['$rootScope', '$timeout', 'API', 'Player', 'App', ($rootScope, $timeout, API, Player, App) ->

    restrict: 'A'
    scope: true

    controller: ($scope) ->
        $scope.$on 'ngVideoModal.ngVideoModalPlaylistSlider.initSlider', (e, data) =>
            $scope.handleInitSlider(data)

        $scope.$on 'TvInput.ngVideoModalPlaylistSlider.moveSlider', (e, data) =>
            $scope.handleMoveSlider(data)

        $scope.$on 'TvInput.ngVideoModalPlaylistSlider.autoPlayPrev', =>
            $scope.handleAutoPlayPrev()

        $scope.$on 'TvInput.ngVideoModalPlaylistSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'Player.ngVideoModalPlaylistSlider.autoPlayPrev', =>
            $scope.handleAutoPlayPrev()

        $scope.$on 'Player.ngVideoModalPlaylistSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'ngConfirmMessage.ngVideoModalPlaylistSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'ngConfirmExplicitModal.ngVideoModalPlaylistSlider.autoPlayNext', =>
            $scope.handleAutoPlayNext()

        $scope.$on 'TvInput.ngVideoModalPlaylistSlider.selectVideoToPlay', =>
            $scope.handleSelectVideoToPlay()

        $scope.$on 'Playlists.ngVideoModalPlaylistSlider.addedNewVideoToPlaylist', =>
            $scope.handleAddedNewVideoToPlaylist()

    link: (scope, el, attrs) ->

        _curFocusIndex = 0
        _curIndexOnScreen = 0
        _curPlayingIndex = 0
        _distance = 0
        _viewEl = $('#nav-playlist-continuous-play')
        _curSelectedEl = ''
        _curFocusEl = ''
        _videoList = []


        scope.handleInitSlider = (videosFromPlaylistArr)=>
            _curFocusIndex = 0
            _curIndexOnScreen = 0
            _curPlayingIndex = 0
            _distance = 0
            _videoList = videosFromPlaylistArr

            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex
            App.VideoModal.videoList = _videoList

            len = _videoList.length
            width = len * 300

            el.css
                width: width

            $timeout -> scope.videoList = _videoList


        scope.handleAddedNewVideoToPlaylist = =>
            if $rootScope.storage.username
                _videoList.push App.VideoModal.data
                App.VideoModal.videoList = _videoList
                $timeout -> scope.videoList = _videoList

            len = _videoList.length
            width = len * 300

            el.css
                width: width

        scope.handleMoveSlider = (data) =>
            key = data.key

            if key is 'right'
                _browseNext()
            else if key is 'left'
                _browsePrev()

        _browseNext = =>

            itemEl = _getCurFocusEl()
            nextItemEl = itemEl.next('.nav-item')

            return if not nextItemEl.size()

            itemEl.removeClass 'cur-focus'
            nextItemEl.addClass 'cur-focus'
            _curFocusEl = nextItemEl

            if _curIndexOnScreen is 3
                _distance -= 273
                el.animate
                    left: _distance
            else
                _curIndexOnScreen++

            _curFocusIndex++
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex


        _browsePrev = =>
            return if _curFocusIndex is 0

            itemEl = _getCurFocusEl()
            itemEl.removeClass 'cur-focus'
            prevItemEl = itemEl.prev('.nav-item')
            prevItemEl.addClass 'cur-focus'
            _curFocusEl = prevItemEl

            if _curIndexOnScreen is 0
                _distance += 273
                el.animate
                    left: _distance
            else
                _curIndexOnScreen--

            _curFocusIndex--
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex

        _getCurFocusEl = =>
            if _curFocusEl then return _curFocusEl
            _curFocusEl = _viewEl.find('.cur-focus')
            _curFocusEl

        _getCurSelectedEl = =>
            if _curSelectedEl then return _curSelectedEl
            _curSelectedEl = _viewEl.find('.cur-selected')
            _curSelectedEl

        scope.handleAutoPlayPrev = =>
            _curPlayingIndex--

            itemEl = _getCurSelectedEl()
            nextItemEl = itemEl.prev('.nav-item')
            # return to playlist view if no more next item
            if not nextItemEl.size()
                $timeout -> $rootScope.modal = ''
                return

            if itemEl.hasClass 'cur-focus'
                _browsePrev()

            itemEl.removeClass 'cur-selected'
            nextItemEl.addClass 'cur-selected'
            _curSelectedEl = nextItemEl

            videoData = _videoList[_curPlayingIndex]
            App.VideoModal.isrc = videoData.isrc
            App.VideoModal.data = videoData

            $rootScope.$broadcast 'ngVideoModalPlaylistSlider.ngVideoModal.triggerPlayVideo'

            _distance = _curPlayingIndex * -273

            el.css
                left: _distance

            if not nextItemEl.hasClass 'cur-focus'
                curFocusEl = _getCurFocusEl()
                curFocusEl.removeClass 'cur-focus'
                nextItemEl.addClass 'cur-focus'
                _curFocusEl = nextItemEl

            _curIndexOnScreen = 0
            _curFocusIndex = _curPlayingIndex
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex

        scope.handleAutoPlayNext = =>

            _curPlayingIndex++


            itemEl = _getCurSelectedEl()
            nextItemEl = itemEl.next('.nav-item')
            # return to playlist view if no more next item
            if not nextItemEl.size()
                $timeout -> $rootScope.modal = ''
                return

            if itemEl.hasClass 'cur-focus'
                _browseNext()

            itemEl.removeClass 'cur-selected'
            nextItemEl.addClass 'cur-selected'
            _curSelectedEl = nextItemEl

            videoData = _videoList[_curPlayingIndex]

            App.VideoModal.isrc = videoData.isrc
            App.VideoModal.data = videoData

            $rootScope.$broadcast 'ngVideoModalPlaylistSlider.ngVideoModal.triggerPlayVideo'

            _distance = _curPlayingIndex * -273

            el.css
                left: _distance

            if not nextItemEl.hasClass 'cur-focus'
                curFocusEl = _getCurFocusEl()
                curFocusEl.removeClass 'cur-focus'
                nextItemEl.addClass 'cur-focus'
                _curFocusEl = nextItemEl

            _curIndexOnScreen = 0
            _curFocusIndex = _curPlayingIndex
            App.VideoModal.curFocusOnSliderIndex = _curFocusIndex

        scope.handleSelectVideoToPlay = =>
            return if _curPlayingIndex is _curFocusIndex

            newVideoToPlayData = _videoList[_curFocusIndex]
            App.VideoModal.isrc = newVideoToPlayData.isrc
            App.VideoModal.data = newVideoToPlayData

            $rootScope.$broadcast 'ngVideoModalPlaylistSlider.ngVideoModal.triggerPlayVideo'


            itemEl = _getCurSelectedEl()
            itemEl.removeClass 'cur-selected'
            newItemEl = _getCurFocusEl()
            newItemEl.addClass 'cur-selected'
            _curSelectedEl = newItemEl

            $rootScope.$broadcast 'ngVideoModalPlaylistSlider.ngVideoModal.hideVideoElements'

            extraDistance = _curIndexOnScreen * 273
            _distance -= extraDistance

            el.animate
                left: _distance

            _curIndexOnScreen = 0
            _curPlayingIndex = _curFocusIndex





]

