'use strict'

vevo.service 'API', ['$rootScope', '$timeout', '$http', '$q', 'K', 'Storage', 'Util', 'App', ($rootScope, $timeout, $http, $q, K, Storage, Util, App) ->

    @vevotv = {}
    @video = {}
    @browse = {}
    @activate = {}
    @freeWheel = {}
    @common = {}
    @search = {}
    @playlists = {}

    # Region code
    #############################################################
    @getRegionCode = =>
        $http.get(K.API.regionCodeUrl).success (res) ->
            App.setRegionCode res.country

    # VEVO TV View
    #############################################################
    @vevotv.getMetadata = (callback) =>
        $http.get(K.API.vevotvMetadataUrl).success (res) ->
            callback res.result


    @vevotv.getSchedule = (callback) =>

        $http.get(K.API.scheduleUrl).success (res) ->
            callback res.result[0].shows




    # Video Modal
    #############################################################
    @video.getVideoStreamData  = (params, callback) =>
        isrc = params.isrc
        streamUrl = K.API.videoStreamUrl.replace '{isrc}', isrc
        $timeout ->
            http = $http.get(streamUrl, {headers: {'X-VEVO-Private-Key': K.API.headerPriveKey}})
            http.success (res) ->
                arr = res.result.renditions
                App.VideoModal.renditionsArr = arr
                App.VideoModal.renditionsIndex = 0
                url = arr[0].url

                callback
                    url: url

    @video.getAllVideoOnDemandData = (callback) =>
        isrc = App.VideoModal.isrc
        streamUrl = K.API.videoStreamUrl.replace '{isrc}', isrc
        videoUrl = K.API.videoDataUrl.replace '{isrc}', isrc
        getVideoStream = $http.get streamUrl, {headers: {'X-VEVO-Private-Key': K.API.headerPriveKey}}
        getVideoData = $http.get videoUrl

        $q.all([getVideoStream, getVideoData]).then (arr) ->
            result =
                streamData: arr[0].data.result
                videoData: arr[1].data.result

            if callback then callback result

    @video.getSingleVideoForShow = (params, callback) =>
        isrc = params.isrc
        videoUrl = K.API.videoDataUrl.replace '{isrc}', isrc

        $http.get(videoUrl).success (res) =>
            result = res.result
            if callback then callback(result)

    @video.initContinuousPlayVideos  = (callback) =>
        isrc = App.VideoModal.isrc

        initRecommendVideosUrl = K.API.initRecommendVideosUrl.replace '{isrc}', isrc

        videoUrl = K.API.videoDataUrl.replace '{isrc}', isrc
        arr = []

        $http.get(videoUrl).success (res) =>
            arr.push res.result

            $http.get(initRecommendVideosUrl).success (res) =>

                result = res.result
                nextVideoData = result.recommended_video
                arr.push nextVideoData

                sessionId = result.session_id
                newParam =
                    sessionId: sessionId

                url = K.API.nextVideoUrl.replace '{session_id}', sessionId
                $http.get(url).success (res1) =>
                    $http.get(url).success (res2) =>
                        $http.get(url).success (res3) =>
                            $http.get(url).success (res4) =>

                                resultArr = []
                                resultArr.push res1.result.recommended_video
                                resultArr.push res2.result.recommended_video
                                resultArr.push res3.result.recommended_video
                                resultArr.push res4.result.recommended_video

                                $.merge arr, resultArr

                                obj =
                                    sessionId: result.session_id
                                    videoArr: arr
                                    
                                if callback then callback(obj)




    @video.getMoreContinuousPlayVideos = (params, callback) =>
        sessionId = params.sessionId
        url = K.API.nextVideoUrl.replace '{session_id}', sessionId


        $http.get(url).success (res1) =>

            resultArr = []
            resultArr.push res1.result.recommended_video

            callback resultArr



    @video.getArtistVideosData = (callback) =>

        url = K.API.artistVideosUrl
        url = url.replace '{artistId}', App.VideoModal.artistId

        params =
            max: App.VideoModal.artistNumVideos

        result =
            success: true

        $http.get(url, {params: params}).success (res) ->

            videoArr = res.result
            if not videoArr.length
                return callback {success: false}

            firstVideoData = videoArr[0]

            result.videoData = videoArr

            streamUrl = K.API.videoStreamUrl.replace '{isrc}', firstVideoData.isrc
            $http.get(streamUrl, {headers: {'X-VEVO-Private-Key': K.API.headerPriveKey}}).success (res) ->
                result.streamData = res.result

                callback result

    @video.callSuperBeacon = =>
        storageData = Storage.get()

        url = K.API.superBeaconUrl
        url = url.replace '{isrc}', App.VideoModal.isrc
        url = url.replace '{regionCode}', App.getRegionCode()
        url = url.replace '{userId}', storageData.userId || null

        $http.get(url).success (res) =>


    # Browse View
    #############################################################

    # keep count for browse view
    @browse.loadMoreBrowseData =
        premieres: 0
        today: 0
        week: 0
        certified: 0
        live: 0


    @browse.getBrowseData = (obj, callback) =>

        offset = 5

        params =
            max: 5

        list = obj.list
        switch list
            when 'premieres'
                offset = @browse.loadMoreBrowseData.premieres + offset
                @browse.loadMoreBrowseData.premieres = offset

                params.offset = offset
                params.order = 'mostrecent'
                params.ispremiere = true

            when 'today'
                offset = @browse.loadMoreBrowseData.today + offset
                @browse.loadMoreBrowseData.today = offset

                params.offset = offset
                params.order = 'mostviewedtoday'

            when 'week'
                offset = @browse.loadMoreBrowseData.week + offset
                @browse.loadMoreBrowseData.week = offset

                params.offset = offset
                params.order = 'mostviewedthisweek'

            when 'certified'
                offset = @browse.loadMoreBrowseData.certified + offset
                @browse.loadMoreBrowseData.certified = offset

                params.offset = offset
                params.iscertified = true

            when 'live'
                offset = @browse.loadMoreBrowseData.live + offset
                @browse.loadMoreBrowseData.live = offset

                params.offset = offset
                params.islive = true

        $http
            url: K.API.videosUrl
            method: 'GET'
            params: params
        .success (res) ->
            callback res.result

    @browse.getAllBrowseData = (callback) =>
        videosUrl = K.API.videosUrl

        getPremieresList = $http
            url: videosUrl
            method: 'GET'
            params:
                max: 5
                order: 'mostrecent'
                ispremiere: true


        getTodayList = $http
            url: videosUrl
            method: 'GET'
            params:
                max: 5
                order: 'mostviewedtoday'

        getWeekList = $http
            url: videosUrl
            method: 'GET'
            params:
                max: 5
                order: 'mostviewedthisweek'

        getCertifiedList = $http
            url: videosUrl
            method: 'GET'
            params:
                max: 5
                iscertified: true

        getLiveList = $http
            url: videosUrl
            method: 'GET'
            params:
                max: 5
                islive: true
                order: 'mostviewedtoday'

        getGenreList = $http
            url: K.API.genreListUrl
            method: 'GET'

        getShowsList = $http
            url: K.API.showsListUrl
            method: 'GET'

        getLiftList = $http
            url: K.API.liftArtistsUrl
            method: 'GET'

        $q.all([
            getPremieresList
            getTodayList
            getWeekList
            getCertifiedList
            getLiveList
            getGenreList
            getShowsList
            getLiftList
        ]).then (arr) ->
            # TODO: need to do error checking

            genreListData = arr[5].data.result

            showsData = arr[6].data
            if showsData.result
                showsData = showsData.result

            result =
                premieres: arr[0].data.result
                today: arr[1].data.result
                week: arr[2].data.result
                certified: arr[3].data.result
                live: arr[4].data.result
                popGenre: genreListData[0].videos[0]
                hiphopGenre: genreListData[1].videos[0]
                alternativeGenre: genreListData[2].videos[0]
                soulGenre: genreListData[3].videos[0]
                electronicGenre: genreListData[4].videos[0]
                rockGenre: genreListData[5].videos[0]
                countryGenre: genreListData[6].videos[0]
                jazzGenre: genreListData[7].videos[0]
                reggaeGenre: genreListData[8].videos[0]
                latinoGenre: genreListData[9].videos[0]
                metalGenre: genreListData[10].videos[0]
                classicalGenre: genreListData[11].videos[0]
                holidayGenre: genreListData[12].videos[0]
                shows: showsData
                lift: arr[7].data.result.episodes

            if callback then callback(result)

    @browse.getShowsEpisodes = (params, callback) =>
        url = K.API.showsEpisodesUrl
        url = url.replace '{showId}', params.showId

        $http
            url: url
            method: 'GET'
        .success (res) ->

            if res.success
                callback res.result

    @browse.getLiftArtistPlaylistVideos = (params, callback) =>
        url = K.API.getSinglePlaylistUrl
        url = url.replace '{playlistId}', params.playlistId

        $http
            url: url
            method: 'GET'
        .success (res) ->

            if res.success
                callback res.result



    # Activate Modal
    #############################################################
    @activate.getAuthenticate = (callback) =>

        storageData = Storage.get()

        if not storageData.deviceId
            newDeviceId = Util.generateDeviceId()
            Storage.set
                deviceId: newDeviceId
            K.DeviceId = newDeviceId
        else
            K.DeviceId = storageData.deviceId

        params =
            provider: 'SamsungTv'
            deviceId: K.DeviceId

        $http
            url: K.API.authenticateUrl
            method: 'GET'
            params: params
            headers:
                'X-VEVO-Private-Key': K.API.headerPriveKey
        .success (res) ->
            result =
                success: res.success

            if res.success
                result.user_id = res.user_id
                result.session_token = res.session_token
                result.result = res.result

            else
                result.activationCode = res.link_token

            if callback then callback(result)

    @activate.unlinkAccount = (callback) =>
        storageData = Storage.get()

        params =
            provider: 9
            identifier: storageData.deviceId

        $http
            url: K.API.unlinkAccountUrl
            method: 'GET'
            params: params
            headers:
                'X-VEVO-Session-Token': storageData.sessionToken
        .success (res) ->
            if callback then callback()

    # Free Wheel
    #############################################################
    @freeWheel.getInitVevoTvFreeWheel = (params, callback) =>
        url = K.API.freeWheelVevoTvInitCallUrl
        randomNum = new Date().getTime()

        url = url.replace '{random_num}', randomNum
        url = url.replace '{vevotv_program}', params.program

        $http
            url: url
            method: 'GET'
            withCredentials: true
            headers:
                'Cookie': document.cookie
        .success (res, status, header, config) ->

            _handleCookie header

            callback res

    @freeWheel.getVevoTvFreeWheel = (params, callback) =>
        url = K.API.freeWheelVevoTvCallUrl
        randomNum = new Date().getTime()

        url = url.replace '{random_num}', randomNum
        url = url.replace '{vevotv_program}', params.program
        url = url.replace '{tpos}', App.VevoTv.tpos

        App.VevoTv.tpos++


        $http
            url: url
            method: 'GET'
            withCredentials: true
            headers:
                'Cookie': document.cookie
        .success (res, status, header, config) ->

            _handleCookie header

            callback res


    @freeWheel.getVideoFreeWheel = (params, callback) =>

        url = K.API.freeWheelVideoCallUrl
        randomNum = new Date().getTime()

        url = url.replace '{random_num}', randomNum
        url = url.replace '{isrc}', params.isrc


        $http
            url: url
            method: 'GET'
            withCredentials: true
            headers:
                'Cookie': document.cookie
        .success (res, status, header, config) ->

            _handleCookie header

            callback res

    # Common
    #############################################################
    @common.callUrl = (params, callback) =>
        url = params.url

        $http
            url: url
            method: 'GET'
            withCredentials: true
            headers:
                'Cookie': document.cookie
        .success (res, status, header, config) ->
            if callback then callback res


    _handleCookie = (header) =>
        cookies = header 'Set-Cookie'

        if not cookies
            headers = header()

            cookies = ''

            for key, val of headers
                if /^set_cookie/.test key
                    cookies += val

        cookies = cookies.split ';'
        for cookie in cookies
            document.cookie = cookie

    # Search View
    #############################################################
    @search.getSearchResult = (params, callback) =>
        query = params.query

        url = K.API.searchUrl
        url = url.replace '{query}', query

        $http.get(url).success (res) ->
            callback res




    # Playlists View
    #############################################################
    @playlists.getUserPlaylists = (callback) =>
        storageData = Storage.get()
        url = K.API.userPlaylistsUrl
        url = url.replace '{userId}', storageData.userId


        $http.get(url).success (res) ->
            if res.success
                callback res.result

    @playlists.createPublicPlaylistForUserAfterLogin = (callback) =>
        storageData = Storage.get()
        publicPlaylist = storageData.publicPlaylist
        videoArr = publicPlaylist.videos

        url = K.API.createUserPlaylistUrl

        isrcStr = ''

        for video in videoArr
            isrcStr = if isrcStr then isrcStr + ',' + video.isrc else video.isrc

        params =
            title: K.Playlists.publicTitle
            isrcs: isrcStr
            includePlaylist: true

        $http
            url: url
            method: 'GET'
            headers:
                'X-VEVO-Session-Token': storageData.sessionToken
            params: params
        .success (res) ->
            if res.success
                callback res.result

    @playlists.createPublicPlaylistForUserAfterAdd = (params, callback) =>
        storageData = Storage.get()

        isrc = params.isrc
        url = K.API.createUserPlaylistUrl

        obj =
            title: K.Playlists.publicTitle
            isrcs: isrc
            includePlaylist: true

        $http
            url: url
            method: 'PUT'
            headers:
                'X-VEVO-Session-Token': storageData.sessionToken
            params: obj
        .success (res) ->
            if res.success
                callback res.result


    @playlists.createPlaylistForUser = (params, callback) =>
        storageData = Storage.get()

        url = K.API.createUserPlaylistUrl
        obj =
            title: params.title
            includePlaylist: true

        $http
            url: url
            method: 'GET'
            headers:
                'X-VEVO-Session-Token': storageData.sessionToken
            params: obj
        .success (res) ->
            if res.success
                callback res.result
        
    @playlists.updatePlaylistForUser = (params, callback) =>
        storageData = Storage.get()
        playlistId = params.playlistId
        isrcs = params.isrcs

        url = K.API.updateUserPlaylistUrl
        url = url.replace '{playlistId}', playlistId

        params =
            isrcs: isrcs
            append: true

        $http
            url: url
            method: 'GET'
            headers:
                'X-VEVO-Session-Token': storageData.sessionToken
            params: params
        .success (res) ->
            if res.success
                callback res.result

    @playlists.getSinglePlaylist = (params, callback) =>
        playlistId = params.playlistId

        url = K.API.getSinglePlaylistUrl
        url = url.replace '{playlistId}', playlistId

        $http
            url: url
            method: 'GET'
        .success (res) ->
            if res.success
                callback res.result

    @playlists.deletePlaylist = (params, callback) =>
        storageData = Storage.get()
        playlistId = params.playlistId

        url = K.API.deleteUserPlaylistUrl
        url = url.replace '{playlistId}', playlistId

        $http
            url: url
            method: 'DELETE'
            headers:
                'X-VEVO-Session-Token': storageData.sessionToken
        .success (res) ->
            if res.success
                callback()


]


