'use strict'

vevo.service 'ComScore', ['$rootScope', '$http', '$q', 'K', 'Storage', 'App'
, ($rootScope, $http, $q, K, Storage, App) ->

    _basePageViewTrackingObj = {}
    _basePlayerTrackingObj = {}

    _curPlaylistType = ''
    _curPlayerIsrc = ''

    @init = =>
        _setBasePageViewTrackingObj()
        _setBasePlayerTrackingObj()

        _initStreamSense()

        $rootScope.$on 'Storage.ComScore.userLoggedOut', =>
            @setUserStatus()
            @trackCustomEvent
                user_status_events: 'fb_logoff_success'

        $rootScope.$on 'App.ComScore.trackPageView', (e, data) =>
            @trackPageView data


    @initStreamSensePlaylistForVevoTv = =>
        return if _curPlaylistType is 'vevotv'

        _curPlaylistType = 'vevotv'

        # set initial playlist for vevotv since that plays on app launch
        playlistLabels = {}
        playlistLabels.ns_st_pl = 'vevotv'
        ns_.StreamSense.Puppet.newPlaylist playlistLabels

    @initStreamSensePlaylistForVideo = (view) =>
        return if _curPlaylistType is 'video'

        _curPlaylistType = 'video'

        playlistLabels = {}
        # can probably be improved later
        switch view
            when 'browse'
                playlistLabels.ns_st_pl = 'Continuous-Play-From-Browse-Page'
            when 'search'
                playlistLabels.ns_st_pl = 'Continuous-Play-From-Search-Page'
            when 'playlists'
                playlistLabels.ns_st_pl = 'Continuous-Play-From-Playlists-Page'

        ns_.StreamSense.Puppet.newPlaylist playlistLabels

    @setUserStatus = =>
        storageData = Storage.get()
        if storageData.username
            # user is logged in
            _basePageViewTrackingObj.user_status = 'fb_logged_in' # fb_logged_in, email_logged_in, or logged_out
            _basePageViewTrackingObj.user_id = storageData.userId # set only if logged in

            _basePlayerTrackingObj.user_status = 'logged_in'
        else
            _basePageViewTrackingObj.user_status = 'logged_out' # fb_logged_in, email_logged_in, or logged_out

            _basePlayerTrackingObj.user_status = 'anonymous'


    @trackPageView = (params) =>

        newTrackingObj = $.extend {}, _basePageViewTrackingObj

        view = params.view

        switch view
            when 'home'
                newTrackingObj.name = 'splash_page'
                newTrackingObj.category = 'home'
                newTrackingObj.subsection = 'home'
                newTrackingObj.content_type = 'home'
            when 'vevotv'
                newTrackingObj.name = 'vevotv'
                newTrackingObj.category = 'home'
                newTrackingObj.subsection = 'home'
                newTrackingObj.content_type = 'vevotv'
            when 'browse'
                _handleTrackBrowseView()
                return
            when 'search'
                newTrackingObj.name = 'search_page'
                newTrackingObj.category = 'search'
                newTrackingObj.subsection = 'search_landing'
                newTrackingObj.content_type = 'navigation_page'
            when 'playlists'
                newTrackingObj.name = 'my_playlist_page'
                newTrackingObj.category = 'home'
                newTrackingObj.subsection = 'my_vevo'
                newTrackingObj.content_type = 'navigation_page'
            when 'VOD'
                videoData = App.VideoModal.data
                newTrackingObj.name = 'watch'
                newTrackingObj.category = 'watch'
                newTrackingObj.subsection = 'watch_page'
                newTrackingObj.content_type = 'music_video'
                newTrackingObj.content_title = videoData.url_safe_title
                newTrackingObj.content_id = videoData.isrc
                newTrackingObj.artists = videoData.artists_main[0]?.url_safename


        $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: newTrackingObj
        .success (res) ->

    @trackVevoTv = (params) =>
        action = params.action

        videoData = App.VevoTv.metadata
        curProgramTitle = App.VevoTv.scheduleArr[0]?.title || ''
        if curProgramTitle then curProgramTitle = curProgramTitle.split(' ').join('-')

        now = new Date()
        curVevoTvTime = now - App.VevoTv.curClipStartTime

        switch action
            when 'newClipPlay'
                clipLabels = {}
                clipLabels.video_id = App.VevoTv.isrc
                clipLabels.ns_st_ci = App.VevoTv.isrc
                clipLabels.video_artist = videoData.artists_main[0]?.url_safename
                clipLabels.all_artist = _getAllArtistNames(videoData.artists_main)
                clipLabels.ft_artist = _getAllArtistNames(videoData.artists_featured)
                clipLabels.video_content_type = 'vevotv'
                clipLabels.ns_st_pr = curProgramTitle
                clipLabels.ns_st_ty = 'vtv'
                clipLabels.ad_load = 'no'
                clipLabels.name = _getFullWatchName(videoData)
                clipLabels.ns_st_li = '1'
                clipLabels.ns_st_ev = 'play'

                if _curPlayerIsrc
                    ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.END, {}, curVevoTvTime


                ns_.StreamSense.Puppet.newClip clipLabels
                # setting the position as 1 for all vevotv since there isnt a true position
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PLAY, {}, 0

                _curPlayerIsrc = App.VevoTv.isrc
            when 'newAdClipPlay'
                clipLabels = {}
                clipLabels.video_id = App.VevoTv.isrc
                clipLabels.ns_st_ci = App.VevoTv.isrc
                clipLabels.ns_st_ty = 'vtv'
                clipLabels.name = 'watch.vevotv_ad.' + App.VevoTv.isrc
                clipLabels.ns_st_li = '1'
                clipLabels.ns_st_ev = 'play'
                clipLabels.ad_load = 'yes'
                clipLabels.ad_type = 'preroll'

                if _curPlayerIsrc
                    ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.END, {}, curVevoTvTime

                ns_.StreamSense.Puppet.newClip clipLabels
                # setting the position as 1 for all vevotv since there isnt a true position
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PLAY, {}, 0

            when 'pause'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PAUSE, {}, curVevoTvTime
            when 'resume'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PLAY, {}, curVevoTvTime
            when 'bufferingStart'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.BUFFER, {}, 0
            when 'destroy'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.END, {ns_st_pe: 1}, curVevoTvTime
                _curPlaylistType = ''
                _curPlayerIsrc = ''

    @trackVideo = (params) =>
        action = params.action
        curVideoTime = params.curVideoTime || 0
        curVideoTime = curVideoTime * 1000 # get the milliseconds

        videoData = App.VideoModal.data

        switch action
            when 'newClipPlay'
                clipLabels = {}
                clipLabels.video_id = App.VideoModal.isrc
                clipLabels.ns_st_ci = App.VideoModal.isrc
                clipLabels.video_artist = videoData.artists_main[0]?.url_safename
                clipLabels.all_artist = _getAllArtistNames(videoData.artists_main)
                clipLabels.ft_artist = _getAllArtistNames(videoData.artists_featured)
                clipLabels.ns_st_pr = 'music_video'
                clipLabels.ns_st_ty = 'vod'
                clipLabels.ad_load = 'no'
                clipLabels.name = _getFullWatchName(videoData)
                clipLabels.ns_st_ev = 'play'

                if _curPlayerIsrc
                    ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.END, {}, curVideoTime


                ns_.StreamSense.Puppet.newClip clipLabels
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PLAY, {}, 0

                _curPlayerIsrc = App.VideoModal.isrc

            when 'newAdClipPlay'
                clipLabels = {}
                clipLabels.video_id = App.VideoModal.isrc
                clipLabels.ns_st_ci = App.VideoModal.isrc
                clipLabels.ns_st_ty = 'vod'
                clipLabels.name = 'watch.video_ad.' + App.VideoModal.isrc
                clipLabels.ns_st_ev = 'play'
                clipLabels.ad_load = 'yes'
                clipLabels.ad_type = 'preroll'

                if _curPlayerIsrc
                    ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.END, {}, curVideoTime

                ns_.StreamSense.Puppet.newClip clipLabels
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PLAY, {}, 0

            when 'newClipPlayAfterSkip'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PLAY, {ns_st_ui: 'seeking'}, curVideoTime

            when 'pause'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PAUSE, {}, curVideoTime
            when 'resume'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PLAY, {}, curVideoTime
            when 'bufferingStart'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.BUFFER, {}, 0
            when 'bufferingStartAfterSkip'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.PAUSE, {ns_st_ui: 'seeking'}, curVideoTime
            when 'destroy'
                ns_.StreamSense.Puppet.notify ns_.StreamSense.PlayerEvents.END, {ns_st_pe: 1}, curVideoTime
                _curPlaylistType = ''
                _curPlayerIsrc = ''


    @trackSearch = (params) =>

        newTrackingObj = $.extend {}, _basePageViewTrackingObj

        if params.search_failure
            newTrackingObj.search_failure = params.search_failure
        else
            newTrackingObj.ns_search_term = params.ns_search_term
            newTrackingObj.search_value = params.search_value
            newTrackingObj.search_num_artist = params.search_num_artist
            newTrackingObj.search_num_video = params.search_num_video

        $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: newTrackingObj
        .success (res) ->

    @trackCustomEvent = (params) =>
        newTrackingObj = $.extend {}, _basePageViewTrackingObj

        for own key, val of params
            newTrackingObj[key] = val

        $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: newTrackingObj
        .success (res) ->

    @endVevoTv = =>
        @trackVevoTv {action: 'destroy'}

    @endVideoPlay = =>
        curVideoTime = App.VideoModal.curVideoTime
        @trackVideo
            action: 'destroy'
            curVideoTime: curVideoTime

    _initStreamSense = =>
        ns_.StreamSense.Puppet.newPuppet K.API.comScoreUrl
        ns_.StreamSense.Puppet.setLabels _basePlayerTrackingObj
        @initStreamSensePlaylistForVevoTv()

    _setBasePageViewTrackingObj = =>

        # for production
        _basePageViewTrackingObj.ns_site = 'vevo'
        _basePageViewTrackingObj._site = 'samsung-vevo'

        # for dev
        #_basePageViewTrackingObj.ns_site = 'global-dev'
        #_basePageViewTrackingObj._site = 'samsung-dev'

        _basePageViewTrackingObj.device_type = 'samsung_tv'
        _basePageViewTrackingObj.device_name = 'samsung_tv'
        _basePageViewTrackingObj.app_version = '0.100'
        _basePageViewTrackingObj.connect_type = 'wifi'

        @setUserStatus()

    _setBasePlayerTrackingObj = =>
        # for production
        _basePlayerTrackingObj.ns_site = 'vevo'
        _basePlayerTrackingObj._site = 'samsung-vevo'

        # for dev
        #_basePlayerTrackingObj.ns_site = 'global-dev'
        #_basePlayerTrackingObj._site = 'samsung-dev'

        _basePlayerTrackingObj.video_playback_location = 'samsung_tv'
        _basePlayerTrackingObj.ns_st_cs = '1280x720'
        _basePlayerTrackingObj.ns_st_mp = 'samsung_native_player'
        _basePlayerTrackingObj.player_ver = 'progressive'
        _basePlayerTrackingObj.loc_site = 'vevo'
        _basePlayerTrackingObj.ns_st_ws = 'full'

        @setUserStatus()



    _handleTrackBrowseView = =>
        newTrackingObj = $.extend {}, _basePageViewTrackingObj

        newTrackingObj.category = 'home'
        newTrackingObj.subsection = 'browse'
        newTrackingObj.content_type = 'navigation_page'

        browseFeaturedPageObj = $.extend {}, newTrackingObj
        browseFeaturedPageObj.name = 'browse_featured_page'
        browseFeaturedPage = $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: browseFeaturedPageObj

        browseVideosPageObj = $.extend {}, newTrackingObj
        browseVideosPageObj.name = 'browse_videos_page'
        browseVideosPage = $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: browseVideosPageObj

        browseArtistsPageObj = $.extend {}, newTrackingObj
        browseArtistsPageObj.name = 'browse_artists_page'
        browseArtistsPage = $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: browseArtistsPageObj

        browseShowsPageObj = $.extend {}, newTrackingObj
        browseShowsPageObj.name = 'browse_shows_page'
        browseShowsPage = $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: browseShowsPageObj

        browsePremieresPageObj = $.extend {}, newTrackingObj
        browsePremieresPageObj.name = 'browse_premieres_page'
        browsePremieresPage = $http
            url: K.API.comScoreUrl
            method: 'GET'
            withCredentials: true
            params: browsePremieresPageObj


        $q.all([
            browseFeaturedPage
            browseVideosPage
            browseArtistsPage
            browseShowsPage
            browsePremieresPage
        ])




    _getFullWatchName = (videoData) =>

        mainArtistObj = videoData.artists_main[0]
        title = videoData.url_safe_title
        isrc = videoData.isrc

        str = 'watch.'
        str += mainArtistObj?.url_safename + '.'
        str += title + '.'
        str += isrc
        str




    _getAllArtistNames = (arr) =>
        str = ''
        for artist in arr
            if not str then str = artist.url_safename else str += '|' + artist.url_safename
        str

    window.onunload = =>
        exitDate = new Date()
        duration = exitDate - _appLaunchTimeStamp
        seconds = Math.ceil(duration / 1000)
        @trackCustomEvent
            app_duration_seconds_events: seconds


]
