'use strict'

vevo.service 'FreeWheel', ['$rootScope', '$timeout', 'Player', 'API', 'ComScore', 'App'
, ($rootScope, $timeout, Player, API, ComScore, App) ->

    @init = =>
        $rootScope.$on 'ngVevoTv.FreeWheel.initPlayVevoTvAd', =>
            _handleInitPlayVevoTvAd()

        $rootScope.$on 'Player.FreeWheel.vevoTvAdEnded', =>
            _handleVevoTvAdEnded()

        $rootScope.$on 'Player.FreeWheel.videoAdEnded', =>
            _handleVideoAdEnded()

        $rootScope.$on 'Player.FreeWheel.sendTracking', =>
            _handleSendTracking()

    _handleInitPlayVevoTvAd = =>
        program = encodeURIComponent App.VevoTv.scheduleArr[0].title

        if App.VevoTv.isInitFreeWheelCall
            # init freewheel call
            App.VevoTv.isInitFreeWheelCall = false

            API.freeWheel.getInitVevoTvFreeWheel {program: program}, (data) =>

                
                json = window.X2JS.xml_str2json data

                adArr = json?.VMAP?.AdBreak?.AdSource?.VASTAdData?.VAST?.Ad_asArray
                App.VevoTv.adArr = adArr

                if adArr and adArr.length
                    _playVevoTvAd()
        else
            # subsequent freewheel call
            API.freeWheel.getVevoTvFreeWheel {program: program}, (data) =>

                json = window.X2JS.xml_str2json data

                adArr = json?.VMAP?.AdBreak?.AdSource?.VASTAdData?.VAST?.Ad_asArray
                App.VevoTv.adArr = adArr

                if adArr and adArr.length
                    _playVevoTvAd()

    _playVevoTvAd = =>
        index = App.VevoTv.adPlayPosition
        ad = App.VevoTv.adArr[index]

        mediaFile = ad.InLine.Creatives.Creative.Linear.MediaFiles.MediaFile

        src = mediaFile[mediaFile.length - 1]['__cdata']

        App.VevoTv.adInProgress = true

        ComScore.trackVevoTv {action: 'newAdClipPlay'}
        # for comScore to keep track of when playlist begins
        App.VevoTv.curClipStartTime = new Date()

        Player.clear()
        Player.playAd src


    _handleVevoTvAdEnded = =>

        adArr = App.VevoTv.adArr
        adPlayPosition = App.VevoTv.adPlayPosition
        len = adArr.length
        if len > 1 and adPlayPosition isnt (len - 1)
            # play next ad
            newIndex = adPlayPosition + 1
            App.VevoTv.adPlayPosition = newIndex
            _playVevoTvAd()

        else
            # return to vevo tv
            if App.VevoTv.adInProgress
                Player.clear()
                App.VevoTv.adInProgress = false
                App.VevoTv.adPlayPosition = 0
                App.VevoTv.adArr = []
                Player.play()
                $rootScope.$broadcast 'FreeWheel.ngVevoTv.playedVevoTvAfterAd'

    _handleVideoAdEnded = =>
        # return to video
        App.VideoModal.adInProgress = false
        $rootScope.$broadcast 'FreeWheel.ngVideoModal.triggerPlayVideo'

    _handleSendTracking = =>
        if App.VevoTv.adInProgress
            index = App.VevoTv.adPlayPosition
            ad = App.VevoTv.adArr[index]
        else
            index = App.VideoModal.adPlayPosition
            ad = App.VideoModal.adArr[index]

        info = ad.InLine.Creatives.Creative.Linear
        duration = info.Duration

        trackingData = info.TrackingEvents.Tracking

        impressionUrl = ad.InLine.Impression['__cdata']
        completeUrl = trackingData[0]['__cdata']
        firstQuartileUrl = trackingData[1]['__cdata']
        midpointUrl = trackingData[2]['__cdata']
        thirdQuartileUrl = trackingData[3]['__cdata']

        if duration is '00:00:30'
            firstQuarterTime = 7500
            midepointTime = 15000
            thirdQuarterTime = 22500
            completeTime = 30000
        else
            firstQuarterTime = 3750
            midepointTime = 7500
            thirdQuarterTime = 11250
            completeTime = 15000

        API.common.callUrl {url: impressionUrl}

        $timeout =>
            API.common.callUrl {url: firstQuartileUrl}
        , firstQuarterTime

        $timeout =>
            API.common.callUrl {url: midpointUrl}
        , midepointTime

        $timeout =>
            API.common.callUrl {url: thirdQuartileUrl}
        , thirdQuarterTime

        $timeout =>
            API.common.callUrl {url: completeUrl}
        , completeTime


]
