'use strict'

vevo.service 'Player', ['$rootScope', '$timeout', 'K', 'App', 'API', 'ComScore', ($rootScope, $timeout, K, App, API, ComScore) ->

    @plugin = null
    @state = -1
    @originalSource = null
    @STOPPED = 0
    @PLAYING = 1
    @PAUSED = 2

    _playerCrashCounter = 0
    _playerCrashTime = 0

    _inProcessOfSkip = false
    _isSkipForward = false
    _curVideoTime = 0

    @launchAppAnimation = true
    @initVevoTvFreeWheelCall = true
    @curVevoTvProgram = ''

    @init = =>

        @state = @STOPPED

        @plugin = document.getElementById("pluginPlayer")

        @plugin.OnBufferingStart = => _handlePlayerBufferingStart()
        @plugin.OnBufferingComplete = => _handlePlayerBufferingComplete()
        @plugin.OnCurrentPlayTime = (time) => _handlePlayerTimeUpdate(time)
        @plugin.OnRenderingComplete = => _handlePlayerEnded()

        @plugin.OnRenderError = (code) => _handleRenderError(code)

        @playIntroClip()

        window.onunload = =>
            if @plugin then @plugin.Stop()

    @playIntroClip = =>

        rootPath = window.location.href.split('index.html')[0]
        if unescape(window.location.toString()).indexOf("/usb/") != -1
            # USB
            url = 'file://localhost/dtv/usb/sda1/VEVO/' + K.File.launchAnimationUrl
        else
            # PC-SDK and 2012 TV
            url = unescape(rootPath).split("file://")[1] + K.File.launchAnimationUrl

        @plugin.SetDisplayArea 0, 0, 960, 540
        @plugin.Play url
        @state = @PLAYING

        _gaq.push(['_trackPageview', '/intro'])


    @play = =>
        window.pluginAPI.setOffScreenSaver()

        curView = App.getView()
        if curView is 'vevotv' and @state is @PAUSED
            @clear()
            ComScore.trackVevoTv {action: 'resume'}

        if @state is @PAUSED then return @resume()


        if $rootScope.modal is 'video'
            @plugin.Play App.VideoModal.url
        else
            @plugin.Play K.Stream.vevoTv

        @state = @PLAYING

    @playAd = (src) =>
        window.pluginAPI.setOffScreenSaver()
        if @state is @PLAYING
            @plugin.Stop()
            @state = @STOPPED

        if /m3u8$/.test(src) then src = src + '|BITRATES=1000000~3000000|COMPONENT=HLS'

        @plugin.Play src
        @state = @PLAYING
        if $rootScope.modal is 'video'
            $rootScope.$broadcast 'Player.ngVideoModal.playedVideoAd'
        else
            $rootScope.$broadcast 'Player.ngVevoTv.playedVevoTvAd'

    @clear = =>
        if @state is @PLAYING or @state is @PAUSED or App.VevoTv.adInProgress or App.VideoModal.adInProgress
            window.pluginAPI.setOnScreenSaver()
            @plugin.Stop()
            @state = @STOPPED
    
    @resume = =>
        @plugin.Resume()
        @state = @PLAYING

        window.pluginAPI.setOffScreenSaver()
        if $rootScope.modal is 'video'
            $rootScope.$broadcast 'Player.ngVideoModal.playedVideo'
            ComScore.trackVideo
                action: 'resume'
                curVideoTime: _curVideoTime


    @pause = =>

        if @state is @PLAYING
            @plugin.Pause()
            @state = @PAUSED

            if $rootScope.modal is 'video'
                $rootScope.$broadcast 'Player.ngVideoModal.pausedVideo'
                ComScore.trackVideo
                    action: 'pause'
                    curVideoTime: _curVideoTime
            else
                $rootScope.$broadcast 'Player.ngVevoTv.pausedVevoTv'
                ComScore.trackVevoTv {action: 'pause'}

    @reload = =>
        return if @state isnt @PLAYING

        window.pluginAPI.setOffScreenSaver()

        $rootScope.loading = $rootScope.strings.loading
        @clear()
        @play()


    @skipForward = =>
        if $rootScope.modal is 'video'
            _inProcessOfSkip = true
            _isSkipForward = true
            @plugin.JumpForward 20

    @skipBack = =>
        if $rootScope.modal is 'video'
            _inProcessOfSkip = true
            _isSkipForward = false
            @plugin.JumpBackward 20


    @getCurVideoTime = =>
        _curVideoTime

    _handleRenderError = (code) =>
        # Not in use at the moment
        index = App.VideoModal.renditionsIndex + 1
        obj = App.VideoModal.renditionsArr[0]
        if obj
            App.VideoModal.url = obj.url
            App.VideoModal.renditionsIndex = index
            @play()

    _handlePlayerBufferingStart = =>
        if $rootScope.modal is 'video'
            if _inProcessOfSkip
                ComScore.trackVideo
                    action: 'bufferingStartAfterSkip'
                    curVideoTime: _curVideoTime
            else
                ComScore.trackVideo {action: 'bufferingStart'}
        else
            ComScore.trackVevoTv {action: 'bufferingStart'}

    _handlePlayerBufferingComplete = =>
        window.pluginAPI.setOffScreenSaver()

        loadingDelay = 0
        if $rootScope.loadingType is 'vevotv'
            loadingDelay = 1000
            $rootScope.loadingType = ''
        $timeout ->
            $rootScope.loading = false
        , loadingDelay

        if App.VideoModal.adInProgress or App.VevoTv.adInProgress
            $rootScope.$broadcast 'Player.FreeWheel.sendTracking'
            @plugin.SetDisplayArea 0, 0, 960, 540
            return

        if $rootScope.modal is 'video'

            if _inProcessOfSkip
                _inProcessOfSkip = false
                newTime = _curVideoTime
                if _isSkipForward then newTime += 20 else newTime -= 20
                ComScore.trackVideo
                    action: 'newClipPlayAfterSkip'
                    curVideoTime: newTime

                # show scrub bar when skipping
                $rootScope.$broadcast 'Player.ngVideoModal.showTimeBar'

                return

            videoWidth = Number @plugin.GetVideoWidth()
            videoHeight = Number @plugin.GetVideoHeight()

            fullScreenRatio = 0.5625
            videoRatio = videoHeight / videoWidth

            if videoRatio is fullScreenRatio
                # full screen
                xCoord = 0
                yCoord = 0
                playerWidth = 960
                playerHeight = 540
            else if videoRatio < fullScreenRatio
                # full width
                playerWidth = 960
                playerHeight = (960 * videoHeight) / videoWidth
                heightDiff = 540 - playerHeight
                xCoord = 0
                yCoord = heightDiff / 2
            else
                # full height
                playerHeight = 540
                playerWidth = (540 * videoWidth) / videoHeight
                widthDiff = 960 - playerWidth
                yCoord = 0
                xCoord = widthDiff / 2


            if App.VideoModal.renditionsIndex is 0
                @plugin.SetDisplayArea xCoord, yCoord, playerWidth, playerHeight
            else
                @plugin.SetDisplayArea 0, 0, 960, 540
                App.VideoModal.renditionsIndex = 0

            totalTime = @plugin.GetDuration()
            totalTime = totalTime / 1000
            totalTime = Math.floor totalTime
            App.VideoModal.totalTime = Number totalTime
            $rootScope.$broadcast 'Player.ngVideoModal.playedVideo'

        else

            # ensure vevotv does not played accidentally in background of other pages
            curView = App.getView()
            if curView and curView isnt 'vevotv'
                @plugin.Stop()
                @state = @STOPPED
                return

            @plugin.SetDisplayArea 0, 0, 960, 540

            $rootScope.$broadcast 'Player.ngVevoTv.playedVevoTv'





    _handlePlayerTimeUpdate = (time) =>
        time = time / 1000
        time = Math.floor time

        now = new Date()

        if time is _playerCrashTime
            _playerCrashCounter++

        if @state is @PLAYING and _playerCrashCounter >= 6
            _playerCrashCounter = 0
        else
            _playerCrashCounter = 0



        if $rootScope.modal is 'video'

            if time isnt _curVideoTime
                if $rootScope.loading then $rootScope.loading = false

                if App.VideoModal.adInProgress
                    _curVideoTime = Math.floor(@plugin.GetDuration() / 1000)
                else
                    _curVideoTime = time
                
                App.VideoModal.curVideoTime = _curVideoTime

                $rootScope.$broadcast 'Player.ngVideoModal.updateProgressBar', {time:time}


    _handlePlayerEnded = =>
        if @launchAppAnimation
            @launchAppAnimation = false

            @clear()

            if App.deepLinking
                App.VideoModal.searchType = "video"
                App.VideoModal.isrc = App.deepLinking
                $rootScope.$apply ->
                    $rootScope.modal = 'video'
            else
                App.updateView ['base', 'vevotv']
            return

        if App.deepLinking and not App.playingAdForDeepLinking
            widgetAPI.sendReturnEvent()
            return

        if $rootScope.modal is 'video'
            if App.VideoModal.adInProgress
                $rootScope.$broadcast 'Player.FreeWheel.videoAdEnded'
            else
                modalView = $rootScope.app.modalView
                switch modalView
                    when 'browse' then $rootScope.$broadcast 'Player.ngVideoModalBrowseSlider.autoPlayNext'
                    when 'browseShow' then $rootScope.$broadcast 'Player.ngVideoModalBrowseSlider.autoPlayNext'
                    when 'search' then $rootScope.$broadcast 'Player.ngVideoModalSearchSlider.autoPlayNext'
                    when 'playlists' then $rootScope.$broadcast 'Player.ngVideoModalPlaylistSlider.autoPlayNext'
        else if App.VevoTv.adInProgress
            $rootScope.$broadcast 'Player.FreeWheel.vevoTvAdEnded'






        



]


