'use strict'

vevo.service 'Playlists', ['$rootScope', 'Storage', 'API', 'K', 'App', ($rootScope, Storage, API, K, App) ->

    _userPlaylists = []
    _curPlaylistVideos = []
    _isFocusOnPlaylist = false

    @playlistIdToDelete = ''
    @playlistNameToDelete = ''

    @init = =>

        $rootScope.$on 'Storage.Playlists.createPublicPlaylist', =>
            _handleCreatePublicPlaylist()

        $rootScope.$on 'ngActivateModal.Playlists.getUserPlaylistsAfterLogin', =>
            _handleGetUserPlaylistsAfterLogin()

        $rootScope.$on 'Storage.Playlists.getUserPlaylistsOnAppLoad', =>
            _handleFetchUserPlaylists()

        $rootScope.$on 'TvInput.Playlists.userLoggedOut', =>
            _handleUserLoggedOut()

    @getIsFocusOnPlaylist = =>
        _isFocusOnPlaylist

    @setIsFocusOnPlaylist = (val) =>
        _isFocusOnPlaylist = val

    @getCurPlaylistVideos = =>
        _curPlaylistVideos

    @setCurPlaylistVideos = (arr) =>
        _curPlaylistVideos = arr

    @deleteSinglePlaylist = =>
        arr = []
        for userPlaylist in _userPlaylists
            if userPlaylist.playlist_id isnt @playlistIdToDelete
                arr.push userPlaylist

        _userPlaylists = arr

    @getUserPlaylists = =>
        _userPlaylists

    @addNewUserPlaylist = (playlist) =>
        _userPlaylists.unshift playlist

    @getPublicPlaylist = =>
        storageData = Storage.get()
        storageData.publicPlaylist

    @addToPublicPlaylist = =>
        publicPlaylist = @getPublicPlaylist()

        videosArr = publicPlaylist.videos

        if $rootScope.modal is 'video'
            videoData = App.VideoModal.data
        else
            videoData = App.VevoTv.metadata

        return if not videoData

        if not videosArr.length
            publicPlaylist.image_url = videoData.image_url

        if videosArr.length >= 7
            # prevent public user from adding more to public playlist
            $rootScope.$broadcast 'Playlists.ngConfirmMessage.reachedMaxVideosInPublicPlaylist'
            return

        videosArr.push videoData
        publicPlaylist.videos = videosArr
        publicPlaylist.videocount = videosArr.length

        Storage.set
            publicPlaylist: publicPlaylist

        $rootScope.$broadcast 'Playlists.ngConfirmMessage.addedToWatchLaterPlaylist',
            title: videoData.title
        $rootScope.$broadcast 'Playlists.ngVideoModalPlaylistSlider.addedNewVideoToPlaylist'

    @addToUserPlaylist = (data, callback) =>
        playlistId = data.playlistId
        params =
            playlistId: playlistId
            isrcs: data.isrcs

        image_url = data.image_url

        API.playlists.updatePlaylistForUser params, (data) =>
            for userPlaylist, index in _userPlaylists
                if userPlaylist.playlist_id is playlistId
                    obj = userPlaylist
                    videocount = Number(userPlaylist.videocount)
                    if videocount is 0
                        obj.image_url = image_url
                    obj.videocount = videocount + 1
                    _userPlaylists.splice index, 1
                    _userPlaylists.unshift obj

                    $rootScope.$broadcast 'Playlists.ngVideoModalPlaylistSlider.addedNewVideoToPlaylist'
                    break
            if callback then callback()

    @addToUserSamsungPlaylist = (videoData) =>
        userHasSamsungPlaylist = false

        for userPlaylist in _userPlaylists
            if userPlaylist.title is K.Playlists.publicTitle
                userHasSamsungPlaylist = true
                break

        if userHasSamsungPlaylist
            # user has existing playlist title "Samsung Watch Later".
            for userPlaylist in _userPlaylists
                if userPlaylist.title is K.Playlists.publicTitle
                    playlistId = userPlaylist.playlist_id
                    break

            params =
                playlistId: playlistId
                isrcs: videoData.isrc

            API.playlists.updatePlaylistForUser params, (data) =>
                for userPlaylist, index in _userPlaylists
                    if userPlaylist.title is K.Playlists.publicTitle
                        obj = userPlaylist
                        videocount = Number(userPlaylist.videocount)
                        if videocount is 0
                            obj.image_url = videoData.image_url
                        obj.videocount = videocount + 1
                        _userPlaylists.splice index, 1
                        _userPlaylists.unshift obj
                        break
                $rootScope.$broadcast 'Playlists.ngConfirmMessage.addedToWatchLaterPlaylist',
                    title: videoData.title

        else
            # create a new playlist for user

            params =
                isrc: videoData.isrc

            API.playlists.createPublicPlaylistForUserAfterAdd params, (data) =>
                _userPlaylists.unshift data
                $rootScope.$broadcast 'Playlists.ngConfirmMessage.addedToWatchLaterPlaylist',
                    title: videoData.title


    @fetchUserPlaylists = (callback) =>
        API.playlists.getUserPlaylists (data) =>
            _userPlaylists = data
            callback()

    _handleUserLoggedOut = =>
        _userPlaylists = []


    _handleGetUserPlaylistsAfterLogin = =>

        API.playlists.getUserPlaylists (data) =>
            _userPlaylists = data
            _handleAddPublicPlaylistToUser data

    _handleFetchUserPlaylists = =>
        API.playlists.getUserPlaylists (data) =>
            _userPlaylists = data


    _handleAddPublicPlaylistToUser = (userPlaylistsArr) =>
        storageData = Storage.get()
        publicPlaylist = storageData.publicPlaylist
        return if publicPlaylist.videocount is 0

        userHasPublicPlaylist = false
        for userPlaylist in userPlaylistsArr
            if userPlaylist.title is K.Playlists.publicTitle
                userHasPublicPlaylist = true
                break

        if not userHasPublicPlaylist
            # add public playlist to user
            API.playlists.createPublicPlaylistForUserAfterLogin  (data) =>
                _userPlaylists.unshift data
                $rootScope.$broadcast 'Playlists.ngPlaylists.reloadViewAfterCreatingNewPublicPlaylistForUser'



    _handleCreatePublicPlaylist = =>

        Storage.set
            publicPlaylist:
                title: K.Playlists.publicTitle
                image_url: K.Playlists.publicImage
                videocount: 0
                videos: []




]

