'use strict'

vevo.service 'TvInput', ['$rootScope', '$timeout', 'App', 'Nav', 'Player', 'Storage', 'Playlists', 'API'
, ($rootScope, $timeout, App, Nav, Player, Storage, Playlists, API) ->

    _keyInProgress = false

    @init = =>

        allowedKeys = {}
        allowedKeys[tvKey.KEY_UP] = 1
        allowedKeys[tvKey.KEY_DOWN] = 1
        allowedKeys[tvKey.KEY_RIGHT] = 1
        allowedKeys[tvKey.KEY_LEFT] = 1
        allowedKeys[tvKey.KEY_ENTER] = 1
        allowedKeys[tvKey.KEY_INFO] = 1
        allowedKeys[tvKey.KEY_PAUSE] = 'playerControls'
        allowedKeys[tvKey.KEY_STOP] = 'playerControls'
        allowedKeys[tvKey.KEY_PLAY] = 'playerControls'

        # [BrightLine] Start
        remoteEl = document.getElementById('vevo-remote-key-handler')
        remoteEl.focus()
        # [BrightLine] End

        angular.element(remoteEl).keydown (e) =>

            keyCode = e.keyCode

            if keyCode in [tvKey.KEY_RETURN, tvKey.KEY_PANEL_RETURN]
                widgetAPI.blockNavigation e

            # prevent all keys when video modal is loading
            if $rootScope.modal is 'video' and $rootScope.loading
                widgetAPI.blockNavigation e
                return

            # To prevent user from pressing keys too fast
            if _keyInProgress then return
            _keyInProgress = true
            $timeout ->
                _keyInProgress = false
            , 300

            # prevent every key when an AD is playing
            if App.VideoModal.adInProgress or App.VevoTv.adInProgress
                if App.deepLinking
                    if keyCode is tvKey.KEY_EXIT
                        widgetAPI.sendExitEvent()
                    else if keyCode is tvKey.KEY_RETURN
                        widgetAPI.sendReturnEvent()
                        widgetAPI.blockNavigation e
                else
                    widgetAPI.blockNavigation e
                return


            # prevent directional keys when loading spinner is displayed
            if $rootScope.loading and keyCode in [tvKey.KEY_RIGHT, tvKey.KEY_LEFT, tvKey.KEY_UP, tvKey.KEY_DOWN]
                return

            subModal = $rootScope.subModal
            modal = $rootScope.modal
            curView = App.getView()
            isAllowedKeys = allowedKeys[keyCode]
            isFullScreenMode = App.VevoTv.isFullScreenMode or App.VideoModal.isFullScreenMode
            isVideoModal = modal is 'video' and not subModal
            isVevoTvView = curView is 'vevotv'
            isPlayerView = isVevoTvView or isVideoModal
            isExitModal = subModal is 'exit'

            if isAllowedKeys and isFullScreenMode and isPlayerView and not isExitModal

                if $rootScope.modal is 'video'

                    curView = App.getView()

                    if keyCode is tvKey.KEY_RIGHT
                        switch curView
                            when 'browse' then $rootScope.$broadcast 'TvInput.ngVideoModalBrowseSlider.autoPlayNext'
                            when 'search' then $rootScope.$broadcast 'TvInput.ngVideoModalSearchSlider.autoPlayNext'
                            when 'playlists' then $rootScope.$broadcast 'TvInput.ngVideoModalPlaylistSlider.autoPlayNext'
                    else if keyCode is tvKey.KEY_LEFT
                        switch curView
                            when 'browse' then $rootScope.$broadcast 'TvInput.ngVideoModalBrowseSlider.autoPlayPrev'
                            when 'search' then $rootScope.$broadcast 'TvInput.ngVideoModalSearchSlider.autoPlayPrev'
                            when 'playlists' then $rootScope.$broadcast 'TvInput.ngVideoModalPlaylistSlider.autoPlayPrev'
                    else
                        if keyCode isnt tvKey.KEY_STOP
                            _handleExitFullScreenMode()
                else
                    _handleExitFullScreenMode()

                return if allowedKeys[keyCode] isnt 'playerControls'
            else if not isExitModal
                if isVideoModal
                    $rootScope.$broadcast 'TvInput.ngVideoModal.resetFullScreenTimer'
                else if isVevoTvView
                    $rootScope.$broadcast 'TvInput.ngVevoTv.resetFullScreenTimer'


            switch keyCode
                when tvKey.KEY_UP then _handleNavigate 'up'
                when tvKey.KEY_DOWN then _handleNavigate 'down'
                when tvKey.KEY_RIGHT then _handleNavigate 'right'
                when tvKey.KEY_LEFT then _handleNavigate 'left'
                when tvKey.KEY_ENTER then _handleEvent()
                when tvKey.KEY_PAUSE then _handlePlayer 'pause'
                when tvKey.KEY_STOP then _handlePlayer 'stop'
                when tvKey.KEY_PLAY then _handlePlayer 'play'
                when tvKey.KEY_RETURN, tvKey.KEY_PANEL_RETURN
                    _handleReturn()
                    widgetAPI.blockNavigation e
                when tvKey.KEY_FF then _handlePlayer 'skipForward'
                when tvKey.KEY_RW then _handlePlayer 'skipBack'
                when tvKey.KEY_RED then _handleHotkey 'A'
                when tvKey.KEY_GREEN then _handleHotkey 'B'
                when tvKey.KEY_YELLOW then _handleHotkey 'C'
                when tvKey.KEY_BLUE then _handleHotkey 'D'
                when tvKey.KEY_EXIT
                    _handleExit()
                    widgetAPI.blockNavigation e

    _handleNavigate = (key) =>
        subModal = $rootScope.subModal
        if subModal is 'playlistSelect'

            if key is 'up' or key is 'down'
                $rootScope.$broadcast 'TvInput.ngPlaylistSelectModal.navigate', {key: key}

        else if subModal is 'exit'
            if key is 'left' or key is 'right'
                $rootScope.$broadcast 'TvInput.ngExitModal.navigate', {key: key}
        else if subModal is 'confirmExplicit'
            if key is 'left' or key is 'right'
                $rootScope.$broadcast 'TvInput.ngConfirmExplicitModal.navigate', {key: key}
        else if subModal is 'playlistDelete'
            if key is 'left' or key is 'right'
                $rootScope.$broadcast 'TvInput.ngPlaylistDeleteModal.navigate', {key: key}
        else if $rootScope.modal is 'video' and subModal isnt 'playlistCreate'

            # for the video page
            modalView = $rootScope.app.modalView
            switch modalView
                when 'browse' then $rootScope.$broadcast 'TvInput.ngVideoModalBrowseSlider.moveSlider', {key: key}
                when 'browseShow' then $rootScope.$broadcast 'TvInput.ngVideoModalBrowseSlider.moveSlider', {key: key}
                when 'search' then $rootScope.$broadcast 'TvInput.ngVideoModalSearchSlider.moveSlider', {key: key}
                when 'playlists' then $rootScope.$broadcast 'TvInput.ngVideoModalPlaylistSlider.moveSlider', {key: key}
        else
            Nav.navigate key

    _handleHotkey = (key) =>
        return if App.deepLinking

        curView = App.getView()
        subModal = $rootScope.subModal
        storageData = Storage.get()

        switch key
            when 'A'
                if $rootScope.modal is 'video'
                    $rootScope.$broadcast 'TvInput.ngVideoModal.addToPlaylist'
                else if curView is 'vevotv'
                    $rootScope.$broadcast 'TvInput.ngVevotv.addToPlaylist'
                else if curView is 'playlists' and storageData?.username
                    $rootScope.$broadcast 'TvInput.ngPlaylists.showCreateModal'

            when 'B'
                if curView is 'playlists'
                    isFocusOnPlaylist = Playlists.getIsFocusOnPlaylist()

                    if isFocusOnPlaylist
                        $rootScope.$broadcast 'TvInput.ngPlaylists.showDeleteModal'
            #when 'D'
                # for development only
                #Storage.removeAll()

    _handleReturn = =>

        if App.deepLinking
            widgetAPI.sendReturnEvent()
            return

        curView = App.getView()
        modal = $rootScope.modal
        subModal = $rootScope.subModal

        if $rootScope.dropdown is 'account'
            accountNavItemEl = angular.element('#nav-item-account')
            accountNavItemEl.addClass 'nav-current-item-focus'
            Nav.curNavFocusItemEl = accountNavItemEl
            Nav.curNavZone = accountNavItemEl.parent().attr 'id'
            return $rootScope.$apply -> $rootScope.dropdown = ''

        if subModal is 'network'
            widgetAPI.sendExitEvent()
        else if subModal is 'confirmExplicit'
            $timeout ->
                $rootScope.subModal = ''
                $rootScope.modal = ''
        else if subModal in ['playlistSelect', 'playlistCreate', 'playlistDelete', 'exit']
            $rootScope.$apply -> $rootScope.subModal = ''
        else if modal is 'video'
            if App.VideoModal.isFullScreenMode
                $rootScope.$apply ->
                    $rootScope.modal = ''
                    $rootScope.loading = false
            else
                $rootScope.$broadcast 'TvInput.ngVideoModal.hideVideoElements'
        else if modal is 'activate'
            $rootScope.$broadcast 'TvInput.ngActivateModal.closeModal'
        else if curView is 'browse'
            $rootScope.$broadcast 'TvInput.ngBrowse.handleReturnKey'
            $rootScope.$broadcast 'TvInput.ngTopNav.handleReturnKey'
        else if curView is 'search'
            $rootScope.$broadcast 'TvInput.ngSearch.handleReturnKeyOnSearchView'
        else if curView is 'playlists'
            $rootScope.$broadcast 'TvInput.ngPlaylists.shiftFocusToTopNav'
        else if curView is 'vevotv'
            if App.VevoTv.isFullScreenMode
                _handleExit()
            else
                $rootScope.$broadcast 'TvInput.ngVevoTv.hideVevoTvElements'


    _handleExit = =>
        if App.deepLinking
            widgetAPI.sendExitEvent()
            return

        subModal = $rootScope.subModal
        if subModal is 'network'
            widgetAPI.sendExitEvent()
        else if subModal isnt 'exit'
            $rootScope.$apply -> $rootScope.subModal = 'exit'
        else
            widgetAPI.sendExitEvent()



    _handleExitFullScreenMode = =>
        curView = App.getView()
        if curView is 'vevotv'
            $rootScope.$broadcast 'TvInput.ngVevoTv.showVevoTvElements'
        else if $rootScope.modal is 'video'
            $rootScope.$broadcast 'TvInput.ngVideoModal.showVideoElements'


    _handlePlayer = (action) =>
        curView = App.getView()
        return if curView isnt 'vevotv' and $rootScope.modal isnt 'video'

        
        if action is 'pause' and Player.state is Player.PLAYING
            Player.pause()
        else if action is 'stop'
            if curView is 'vevotv' and Player.state is Player.PLAYING
                Player.pause()
            else if $rootScope.modal is 'video'

                if App.deepLinking
                    widgetAPI.sendReturnEvent()
                    return

                $rootScope.modal = ''

        else if action is 'play' and Player.state isnt Player.PLAYING
                Player.play()
        else if action is 'skipForward'
            if $rootScope.modal is 'video'
                Player.skipForward()
        else if action is 'skipBack'
            if $rootScope.modal is 'video'
                Player.skipBack()



    _handleEvent = =>
        subModal = $rootScope.subModal
        if subModal is 'playlistSelect'
            $rootScope.$broadcast 'TvInput.ngPlaylistSelectModal.enterKey'
            return
        else if subModal is 'exit'
            $rootScope.$broadcast 'TvInput.ngExitModal.enterKey'
            return
        else if subModal is 'confirmExplicit'
            $rootScope.$broadcast 'TvInput.ngConfirmExplicitModal.enterKey'
            return
        else if subModal is 'playlistDelete'
            $rootScope.$broadcast 'TvInput.ngPlaylistDeleteModal.enterKey'
            return
        else if $rootScope.modal is 'video' and subModal isnt 'playlistCreate'
            modalView = $rootScope.app.modalView
            if modalView is 'browse' or modalView is 'browseShow'
                $rootScope.$broadcast 'TvInput.ngVideoModalBrowseSlider.selectVideoToPlay'
            else if modalView is 'search'
                $rootScope.$broadcast 'TvInput.ngVideoModalSearchSlider.selectVideoToPlay'
            else if modalView is 'playlists'
                $rootScope.$broadcast 'TvInput.ngVideoModalPlaylistSlider.selectVideoToPlay'
            return

        curFocusItemEl = Nav.getCurNavFocusItem()
        event = curFocusItemEl.data 'enter'

        return if not event
        curView = App.getView()

        # TODO: find better way to do this
        updateSelected = =>
            angular.element('.nav-current-item-selected').removeClass 'nav-current-item-selected'
            curFocusItemEl.addClass 'nav-current-item-selected'

        switch event
            when 'showVevoTvView'
                updateSelected()
                App.updateView ['base', 'vevotv']
            when 'showPlaylistsView'
                updateSelected()
                App.updateView ['base', 'playlists']
            when 'showVideoModalFromBrowse'
                section = curFocusItemEl.closest('.section-wrap').attr 'data-section'

                App.VideoModal.isrc = curFocusItemEl.data 'isrc'
                $rootScope.$apply ->
                    $rootScope.app.modalView = 'browse'
                    $rootScope.app.videoModalType = section
                    $rootScope.app.videoModalGenreType = curFocusItemEl.attr 'data-genre' || ''
                    $rootScope.app.videoModalLiftIndex = curFocusItemEl.attr 'data-lift-index' || ''
                    $rootScope.app.videoModalLiftName = curFocusItemEl.attr 'data-lift-name' || ''
                    $rootScope.modal = 'video'
            when 'showVideoModalFromBrowseShows'
                showId = curFocusItemEl.attr 'data-show-id'
                showName = curFocusItemEl.attr 'data-show-name'
                $rootScope.app.videoModalShowName = showName
                obj =
                    showId: showId
                    showName: showName

                $rootScope.$broadcast 'TvInput.ngBrowse.playShow', obj
            when 'showBrowseView'
                updateSelected()
                App.updateView ['base', 'browse']
            when 'showSearchView'
                updateSelected()
                if curView is 'search'
                    $rootScope.$broadcast 'TvInput.ngSearch.focusInSearchBox'
                else
                    App.updateView ['base', 'search']
            when 'showAccount'
                storageData = Storage.get()
                if storageData?.username
                    if $rootScope.dropdown is 'account'
                        $rootScope.$apply -> $rootScope.dropdown = ''
                    else
                        $rootScope.$apply -> $rootScope.dropdown = 'account'
                else
                    $rootScope.$apply -> $rootScope.modal = 'activate'
            when 'logout'
                API.activate.unlinkAccount =>

                Storage.handleUserLogout()
                Nav.collapseDropdown()
                $rootScope.$broadcast 'TvInput.Playlists.userLoggedOut'
                $rootScope.$broadcast 'TvInput.ngPlaylists.userLoggedOut'

            when 'keypadKey'
                char = curFocusItemEl.attr 'data-char'
                $rootScope.$broadcast 'TvInput.ngKeypad.enterKeypadKey', {char: char}
            when 'keypadDone'
                if subModal is 'playlistCreate'
                    $rootScope.$broadcast 'TvInput.ngPlaylistCreateModal.createPlaylist'
                else if curView is 'search'
                    artistRowEl = angular.element('#nav-search-artist')
                    videoRowEl = angular.element('#nav-search-video')

                    $rootScope.$broadcast 'TvInput.ngSearch.handleReturnKeyOnSearchView'

                    if artistRowEl.length or videoRowEl.length
                        Nav.navigate 'down'
            when 'keypadSpace'
                $rootScope.$broadcast 'TvInput.ngKeypad.enterKeypadSpace'
            when 'keypadDelete'
                $rootScope.$broadcast 'TvInput.ngKeypad.enterKeypadDelete'
            when 'keypadClear'
                $rootScope.$broadcast 'TvInput.ngKeypad.enterKeypadClear'
            when 'keypadCaps'
                $rootScope.$broadcast 'TvInput.ngKeypad.enterKeypadCaps'
            when 'keypadShift'
                $rootScope.$broadcast 'TvInput.ngKeypad.enterKeypadShift'
            when 'showVideoModalFromSearch'
                type = curFocusItemEl.attr 'data-search-type'
                App.VideoModal.searchType = type

                if type is 'artist'
                    App.VideoModal.artistId = curFocusItemEl.attr 'data-artist-id'
                    App.VideoModal.artistNumVideos = curFocusItemEl.attr 'data-num-videos'
                    artistName = curFocusItemEl.attr 'data-artist-name'
                    name = artistName
                else if type is 'video'
                    App.VideoModal.isrc = curFocusItemEl.attr 'data-isrc'
                    name = curFocusItemEl.attr 'data-video-name'

                section = curFocusItemEl.closest('.section-wrap').attr 'data-section'

                $rootScope.$broadcast 'TvInput.ngSearch.comScoreTrackSearch',
                    name: name

                $rootScope.$apply ->
                    $rootScope.modal = 'video'
                    $rootScope.app.modalView = 'search'
                    $rootScope.app.videoModalType = section
                    $rootScope.app.videoModalArtistName = artistName

            when 'showVideoModalFromPlaylists'

                playlistName = curFocusItemEl.attr 'data-playlist-title'

                $rootScope.$apply ->
                    $rootScope.app.modalView = 'playlists'
                    $rootScope.app.videoModalType = 'playlist'
                    $rootScope.app.videoModalPlaylistName = playlistName

                if $rootScope.storage.username
                    playlistId = curFocusItemEl.attr 'data-playlist-id'
                    $rootScope.$broadcast 'TvInput.ngPlaylists.initVideoModalFromPlaylistForUser', {playlistId: playlistId}
                else
                    $rootScope.$broadcast 'TvInput.ngPlaylists.initVideoModalFromPlaylistForPublic'
                    

        if event isnt 'showAccount' and $rootScope.dropdown is 'account'
            $rootScope.$apply -> $rootScope.dropdown = ''






]

