function Conviva() { };
function ConvivaPrivateLoader() { };
function LoadLivePass() {
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-
// Copyright Conviva Inc 2010
// Author: George Necula (necula@conviva.com)

// Library for SLASH generated Javascript
// This is included first in the LivePass.js

// ConvivaPrivateLoader is defined at top-level in LivePass.js
// ConvivaPrivateTestingLoader is defined at top-level in TetingLivePassModule.js, loaded in tests after LivePass.js
// ConvivaPrivateModule is defined at top-level in LivePassModule.js (loaded after LivePass.js)
//   - this contains duplicate of classes from ConvivaPrivateLoader (the common ones)
//   - it is cleaned on LivePassInit.Cleanup ()
function registerName(cls, clsname) {
    if( typeof(ConvivaPrivateModule) != "undefined") {
        ConvivaPrivateModule[clsname] = cls;
    } else if( typeof(ConvivaPrivateTestingModule) != "undefined") {
        ConvivaPrivateTestingModule[clsname] = cls;
    } else {
        // We must be populating the Loader classes
        ConvivaPrivateLoader[clsname] = cls;
    }
}
registerName(registerName,"registerName");

// A special function used to mark identifiers for renaming
function __id(x) { return x; }
registerName(__id,"__id");


// Find the definition of a Conviva type by name
function getConvivaType(className) {
    if(typeof(ConvivaPrivateModule) != "undefined" && ConvivaPrivateModule.hasOwnProperty(className)) {
        return ConvivaPrivateModule[className];
    } else if(typeof(ConvivaPrivateTestingModule) != "undefined" && ConvivaPrivateTestingModule.hasOwnProperty(className)) {
        return ConvivaPrivateTestingModule[className];
    } else if(ConvivaPrivateLoader.hasOwnProperty(className)) {
        return ConvivaPrivateLoader[className];
    } else {
        return null;
    }
}
registerName(getConvivaType, "getConvivaType");

/* Construct a JS program that defines in the current scope all the names 
 * from ConvivaPrivate
 */
function importConvivaPrivateProgram(fromWhere, fromWhereName) {
    var res="";
    var p;
    for(p in fromWhere) {
        if(fromWhere.hasOwnProperty(p)) {
            res += "var "+p+"="+__id(fromWhereName)+"."+p+";"
        }
    }
    return res;
}
registerName(importConvivaPrivateProgram,"importConvivaPrivateProgram");

// A special object user to mark the static initializer
function STAT_INIT () {
    return "STAT_INIT";
}
registerName(STAT_INIT,"STAT_INIT");

// Call the static initializer
function statInit(cls, clsname) {
    cls.call(STAT_INIT);
    registerName(cls, clsname);
}
registerName(statInit,"statInit");

// Define a public method
function defPubMeth(obj, name, m) {
    if(obj != STAT_INIT) {
        if(obj[name] == undefined) {
            obj[name] = m;
        } else {
            // overwriting
            obj[__id("super_")+name] = m;
        }
    }    
}
registerName(defPubMeth,"defPubMeth");

// Define a private method
function defPrivMeth(obj,name,m) {
    if(obj != STAT_INIT) obj[name] = m;
}
registerName(defPrivMeth,"defPrivMeth");

// Define a static method
function defStatMeth(obj,cls,name,m) {
    if(obj == STAT_INIT) cls[name]= m;
}
registerName(defStatMeth,"defStatMeth");

// Define a getter
function defGet(obj,name,m) {
    if(obj != STAT_INIT) { //NO_RENAME: Object,configurable,enumerable,get
        if (typeof(Object.defineProperty) != 'undefined') {
            Object.defineProperty(obj, name, {
                configurable : true,
                enumerable : true,
                get : m
            });
        } else { 
            obj.__defineGetter__(name, m);
        }
    }
}
registerName(defGet,"defGet");

function defSet(obj,name,m) {
    if(obj != STAT_INIT) { //NO_RENAME: Object,configurable,set
        if (typeof(Object.defineProperty) != 'undefined') {
            Object.defineProperty(obj, name, {
                configurable : true,
                set : m
            });
        } else { // The browser must support either defineProperty or __defineSetter__, or we will completely fail.
            obj.__defineSetter__(name, m);
        }
    }
}
registerName(defSet,"defSet");

function defStatGet(obj,cls,name,m) {
    if(obj == STAT_INIT) defGet(cls, name, m);
}
registerName(defStatGet,"defStatGet");

function defStatSet(obj,cls,name,m) {
    if(obj == STAT_INIT) defSet(cls, name, m);
}
registerName(defStatSet,"defStatSet");

function slIsArray(inObj) {
    // Is it an array, or an Object
    if(inObj.constructor == Array) {
        return true;
    } else if(typeof(inObj.length) == 'undefined') {
        return false;
    } else {
        return true;
    }
}
registerName(slIsArray,"slIsArray");

function slForEachPropValue(a, f) {
    var ist = slIsArray(a);
    if(slIsArray(a)) {
        for(var i=0;i<a.length;i++) {
            f(a[i]);
        }
    } else {
        for(var p in a) {
            if(a.hasOwnProperty(p)) f(a[p]);
        }
    }
}
registerName(slForEachPropValue,"slForEachPropValue");

function slForEachProp(a, f) {
    if(slIsArray(a)) {
        for(var i=0;i<a.length;i++) {
            f(i);
        }
    } else {
        for(var p in a) {
            if(a.hasOwnProperty(p)) f(p);
        }
    }
}
registerName(slForEachProp,"slForEachProp");

/* Support for declaring test classes and methods */
// Upon static initialization, declare the test class
function declTestClass(obj,name,cls,meta) {
    if(obj == STAT_INIT) jstest.DeclareTestClass(name, cls, meta);
}
registerName(declTestClass,"declTestClass");

/* On static initialization, declare the test method; on instance 
 * construction, set the method closure. We need to do it this late
 * to get the right closure. 
 */
function declTestMethod(obj,clsname,mname,mcode,meta) {
    if(obj == STAT_INIT) {
        jstest.DeclareTestMethod(clsname, mname, meta);
    } else {
        jstest.SetTestMethodClosure(clsname, mname, mcode);
    }
}
registerName(declTestMethod,"declTestMethod");
             
// Slash unsigned integers
function sluint () {
    sluint.two32 = 0x100000000;
    
    sluint.Cast = function (v) {
        var vi = parseInt(v);
        if(vi > sluint.MaxValue) {
            vi = vi % sluint.two32; // 32-bits, positive
        } else if(vi < 0) {
            vi = (- vi) % sluint.two32; // 32-bits of negation, use % only on positive numbers
            vi =  sluint.two32 - vi;
        }
        return vi;
    };

    sluint.uintRegex = new RegExp("^[0-9]+$");
    sluint.Parse = function (v) {
        Lang.parseChecker(v, sluint.uintRegex);
        return sluint.Cast(v);
    }
    
    sluint.MaxValue = sluint.two32 - 1;
    sluint.MinValue = 0;
}
statInit(sluint,"sluint");

// Slash signed integers (do not use "int" because Firefox claims it is a reserved word)
function slint () {
    slint.Cast = function (v) {
        // Cast first to uint to bring it in the range [0 .. 2^32)
        var vu = sluint.Cast(v);
        if(vu > slint.MaxValue) {
            vu = vu - sluint.two32;
        }
        return vu;
    };

    slint.intRegex = new RegExp("^[+-]?[0-9]+$");
    slint.Parse = function (v) {
        Lang.parseChecker(v, slint.intRegex);
        return slint.Cast(v);
    }

    slint.MaxValue =   0x7FFFFFFF;
    slint.MinValue = - 0x80000000;
}
statInit(slint,"slint");

function Int64() {
    var _s = this;
    
    if(_s == STAT_INIT) Int64.TWO_TO_32 /* : Number*/ = 4294967296.0;

    /** We store the int64 as two integers, the most significant bits
     * are a _signed_ 32-bit integer, and the least significant bits are
     * an _unsigned_ 32-bit integer.
     *
     * Together they denote the Number:
     *    Number(_h) * Number(2^32) + Number(l)
     **/

    /** Make a new Int64 with the value 0 */
    function __constr() {
        _s._l = 0;
        _s._h = 0;
    };
    
    /** Initialize from an unsigned int */
    defStatMeth(_s,Int64,"fromUnsignedInt", __fromUnsignedInt);
    function __fromUnsignedInt ( i /*: uint*/ ) /*: Int64*/ {
        var res /*: Int64*/ = new Int64 ();
        res._h = 0;
        res._l = i;
        return res;
    };
        
    /** Initialize from a signed int */
    defStatMeth(_s,Int64,"fromInt", __fromInt);
    function __fromInt( i /*: int*/ ) /*: Int64*/  {
        var res /*: Int64*/ = new Int64 ();
        if(i >= 0) {
            res._h = 0;
            res._l = sluint.Cast(i);
        } else {
            res._h = -1;
            res._l = sluint.Cast(i);
        }
        return res;
    };
        
    /** Initialize from a Number */
    defStatMeth(_s,Int64, "fromNumber", __fromNumber);
    function __fromNumber ( n /*: Number*/ ) /*: Int64*/ {
        var l1 /*: Number*/ = n % Int64.TWO_TO_32;
        // In AS, the remainder has the same sign as n
        if(l1 < 0) {
            l1 = Number(sluint.MaxValue) + 1.0 + l1;
        }
        var res /*: Int64*/ = new Int64 ();
        res._l = sluint.Cast(l1 + 0.5); // round to nearest
        res._h = slint.Cast((n - l1) / Int64.TWO_TO_32);
        // ensure _h * 2^32 + _l = n
        return res;
    };
    

    /** Convert to a Number */
    defGet(_s,"asNumber", __asNumber);
    function __asNumber () /*: Number*/ {
        return Number(_s._h) * Int64.TWO_TO_32 + Number(_s._l);
    };


    if(_s != STAT_INIT) {
        this.toString = function () { 
            return _s.asNumber.toString();
        }
    }

    if(_s != STAT_INIT) __constr.apply(this, arguments);
    
};
statInit(Int64,"Int64");


function UInt64() {

    var _s = this;

    function __constr() {
        // Inherit a bunch of stuff from Int64
        Int64.call(_s);
        _s._l = 0;
        _s._h = 0;
    }

    /** Initialize from an unsigned int */
    defStatMeth(_s, UInt64,"fromUnsignedInt",__fromUnsignedInt);
    function __fromUnsignedInt(i /*: uint*/) /*: UInt64*/ {
        var res /*: UInt64*/ = new UInt64();
        res._h = 0;
        res._l = i;
        return res;
    };
        
    /** Initialize from a Number */
    defStatMeth(_s, UInt64,"fromNumber",__fromNumber);
    function __fromNumber ( n /*: Number*/ ) /*: UInt64*/ {
        var res /*: UInt64*/ = new UInt64();
        res._h = Math.floor(n / Int64.TWO_TO_32);  // 2^32
        res._l = n % Int64.TWO_TO_32; 
        return res;
    };


    if(_s != STAT_INIT) __constr.apply(arguments);
};
statInit(UInt64,"UInt64");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../csClient/CwsClient/ConvivaContentInfo.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: Gary Kumfert (gary@conviva.com) (Original AS3 code)
// Author: George Necula (necula@conviva.com) (Ported to C#)
// Author: Henry Milner (henry@conviva.com) (Unified C# and AS3 versions)
// Author: Yan Li (yan@conviva.com)


// namespace 
    // API_ENTER

    ///<summary>
    /// Encapsulates all the information required to play video content
    ///</summary>
    function ConvivaContentInfo() {
        var _s = this;

        ///////////////////////////////////////////////////////////////////////
        // Begin constants
        ///////////////////////////////////////////////////////////////////////

        // Potential values for cdnName.
        // 
        //NOTE(henry): These constants should be kept consistent with the
        // enumeration of CDN names in CommonWithReporting.xml.  Enum names
        // in that file should equal string values in this file.  For example,
        // CDN_NAME_AKAMAI has a value of "AKAMAI" here and corresponds to
        // the CDNName.AKAMAI enum in CommonWithReporting.xml.
        // 
        // CDNName.UNKNOWN is the one exception to this rule.  That enum
        // should not be part of the client API, since the client should never
        // report it.

        /// <summary>
        /// The value of the cdnName field for videos streamed from Akamai
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_AKAMAI /* : String */ = "AKAMAI";

        /// <summary>
        /// The value of the cdnName field for videos streamed from Amazon.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_AMAZON /* : String */ = "AMAZON";

        /// <summary>
        /// The value of the cdnName field for videos streamed from AT&T CDN.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_ATT /* : String */ = "ATT";

        /// <summary>
        ///  The value of the cdnName field for videos streamed from BitGravity.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_BITGRAVITY /* : String */ = "BITGRAVITY";

        /// <summary>
        /// The value of the cdnName field for videos streamed from BT (British Telecom).
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_BT /* : String */ = "BT";

        /// <summary>
        /// The value of the cdnName field for videos streamed from CDNetworks
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_CDNETWORKS /* : String */ = "CDNETWORKS";

        /// <summary>
        /// The value of the cdnName field for videos streamed from ChinaCache.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_CHINACACHE /* : String */ = "CHINACACHE";

        /// <summary>
        /// The value of the cdnName field for videos streamed from EdgeCast.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_EDGECAST /* : String */ = "EDGECAST";

        /// <summary>
        ///  The value of the cdnName field for videos streamed from Highwinds.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_HIGHWINDS /* : String */ = "HIGHWINDS";

        /// <summary>
        /// The value of the cdnName field for videos streamed from Internap.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_INTERNAP /* : String */ = "INTERNAP";

        /// <summary>
        /// The value of the cdnName field for videos streamed from Level3 Communications.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_LEVEL3 /* : String */ = "LEVEL3";

        /// <summary>
        /// The value of the cdnName field for videos streamed from Limelight Networks.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_LIMELIGHT /* : String */ = "LIMELIGHT";

        /// <summary>
        /// The value of the cdnName field for videos streamed via Octoshape.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_OCTOSHAPE /* : String */ = "OCTOSHAPE";

        /// <summary>
        /// The value of the cdnName field for videos streamed via Swarmcast.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_SWARMCAST /* : String */ = "SWARMCAST";

        /// <summary>
        ///  The value of the cdnName field for videos streamed from Velocix.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_VELOCIX /* : String */ = "VELOCIX";

        /// <summary>
        /// The value of the cdnName field for videos streamed from Telefonica.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_TELEFONICA /* : String */ = "TELEFONICA";

        /// <summary>
        /// The value of the cdnName field for videos streamed from Microsoft.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_MICROSOFT /* : String */ = "MICROSOFT";
        
        /// <summary>
        /// The value of the cdnName field for videos streamed from in-house servers.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_INHOUSE /* : String */ = "INHOUSE";

        /// <summary>
        /// The value of the cdnName field for videos streamed from Comcast.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_COMCAST /* : String */ = "COMCAST";
        
        /// <summary>
        /// The value of the cdnName field for videos streamed from a CDN not listed here.  
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.CDN_NAME_OTHER /* : String */ = "OTHER";

        /// <summary>
        ///  The value of the frameworkName field for Brightcove-based players.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.FRAMEWORK_NAME_BRIGHTCOVE /* : String */ = "Brightcove";

        /// <summary>
        /// The value of the frameworkName field for Kaltura-based players.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.FRAMEWORK_NAME_KALTURA /* : String */ = "Kaltura";

        /// <summary>
        /// The value of the frameworkName field for Ooyala-based players.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.FRAMEWORK_NAME_OOYALA /* : String */ = "Ooyala";

        /// <summary>
        /// The value of the frameworkName field for OSMF-based players.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.FRAMEWORK_NAME_OSMF /* : String */ = "OSMF";

        /// <summary>
        /// The value of the frameworkName field for players based on thePlatform.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.FRAMEWORK_NAME_THE_PLATFORM /* : String */ = "thePlatform";

        /// <summary>
        /// The value of the deviceType field for gaming consoles.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.DEVICE_TYPE_CONSOLE /* : String */ = "Console";

        /// <summary>
        /// The value of the deviceType field for mobile devices.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.DEVICE_TYPE_MOBILE /* : String */ = "Mobile";

        /// <summary>
        ///  The value of the deviceType field for personal computers.
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.DEVICE_TYPE_PC /* : String */ = "PC";

        /// <summary>
        /// The value of the deviceType field for set-top boxes
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.DEVICE_TYPE_SET_TOP_BOX /* : String */ = "Settop";

        /// <summary>
        /// The value of the pluginName field for Kaltura plugin (insights+precision).
        /// </summary>
        if(_s == STAT_INIT) ConvivaContentInfo.PLUGIN_NAME_KALTURA /* : String */ = "ConvivaKalturaPlugin";

        ///////////////////////////////////////////////////////////////////////
        // End constants
        ///////////////////////////////////////////////////////////////////////

        ///////////////////////////////////////////////////////////////////////
        // Begin public fields
        ///////////////////////////////////////////////////////////////////////


        /// <summary>
        /// A unique identifier for the content, preferably human-readable.
        /// </summary>
        if(_s != STAT_INIT) _s.assetName /* : String */ = undefined;


        if(_s != STAT_INIT) _s._tags /* : DictionaryCS(string, string) */ = null;

        /// <summary>
        /// A set of key-value pairs used in resource selection and policy evaluation 
        /// </summary>

        defGet(_s,"tags",__get_tags);
        function __get_tags()  { return  Lang.StringDictionaryToRepr(_s._tags); }
        defSet(_s,"tags",__set_tags);
        function __set_tags(value /* : Object */)  {  _s._tags =  Lang.DictionaryFromRepr/*string, string*/(value); }


        /// <summary>
        /// Set this to the bitrate (1000 bits-per-second) to be used for the integrations
        /// where the streamer does not know the bitrate being player. This value is used
        /// until the streamer reports a bitrate. 
        /// </summary>
        if(_s != STAT_INIT) _s.defaultReportingBitrateKbps /* : int */ = -1;


        /// <summary>
        /// A string identifying the CDN used to stream video.  This must be
        /// chosen from the list of CDN_NAME_* constants in this class.
        /// 
        /// If you use a CDN whose name is not in the list of CDN_NAME_*
        /// constants, please use CDN_NAME_OTHER temporarily and initiate a 
        /// service request with Conviva so that we can add your CDN to the
        /// list.
        /// 
        /// If content is served in-house instead of using a CDN, use CDN_NAME_IN_HOUSE.
        /// </summary> 
        if(_s != STAT_INIT) _s.defaultReportingCdnName /* : String */ = ConvivaContentInfo.CDN_NAME_OTHER;


        /// <summary>
        /// Set this to a string that will be used as the resource name for the integrations
        /// where the streamer does not itself know the resource being played. If this is null, 
        /// then the value of cdnName is used for this purpose.
        /// </summary>
        if(_s != STAT_INIT) _s.defaultReportingResource /* : String */ = null;




        /// <summary>
        /// A string identifying the code framework for the player, if any.
        /// Check the list of FRAMEWORK_NAME_* constants in this class for
        /// an applicable name.  If none is available, choose an appropriate
        /// name instead.
        /// 
        /// For example, players developed with OSMF should set this to
        /// FRAMEWORK_NAME_OSMF.  For players developed with an OVPP, this
        /// should be the same as the OVPP name.
        /// 
        /// This field is optional.
        /// </summary>
        if(_s != STAT_INIT) _s.frameworkName /* : String */ = null;


        /// <summary>
        /// The version number of the framework code, if any.  For example, a player
        /// using version 1.0 of OSMF should set this to "1.0".
        /// 
        /// This field is optional and should only be populated if frameworkName
        /// is populated.
        /// </summary>
        if(_s != STAT_INIT) _s.frameworkVersion /* : String */ = null;

        /// <summary>
        /// @private
        /// 
        /// A string identifying the Conviva plugin used for the integration.
        /// For example, an OSMF-based player using the Conviva OSMF plugin
        /// for OSMF 1.0 and Flash 10.1 should set this to 
        /// "ConvivaOSMFPlugin_OSMF1_0_FP10_1".
        /// 
        /// This field is optional.
        /// </summary>
        if(_s != STAT_INIT) _s.pluginName /* : String */ = null;

        /// <summary>
        /// @private
        /// 
        /// The version number of the Conviva plugin used for the integration,
        /// if any.  For example, "2.30.0.12345" or "2.30.0".
        /// 
        /// This field is optional and should only be populated if pluginName
        /// is populated.
        /// </summary>
        if(_s != STAT_INIT) _s.pluginVersion /* : String */ = null;

        /// <summary>
        /// A string identifying the viewer.
        /// 
        /// This field is optional.
        /// </summary>
        if(_s != STAT_INIT) _s.viewerId /* : String */ = null;

        /// <summary>
        /// A string identifying the device in use.  This should not be used
        /// to identify the type of device, but rather the particular instance
        /// of the device on which this session is running.  For example, if
        /// the session is running on a SmartPhone 2 revision 1.2.3.4 with ID
        /// #0123456789, set this to "0123456789".
        /// 
        /// This field is optional.
        /// </summary>
        if(_s != STAT_INIT) _s.deviceId /* : String */ = null;

        /// <summary>
        /// A string identifying the kind of device being used to play video.
        /// If specified, this must be chosen from the list of DEVICE_TYPE_*
        /// constants.  For example, a session on a mobile device should use
        /// DEVICE_TYPE_MOBILE.
        /// 
        /// LivePass may be able to infer the device type in some cases.  The
        /// integration guide for your platform will note whether it is
        /// necessary to set this field.
        /// </summary>
        if(_s != STAT_INIT) _s.deviceType /* : String */ = null;

        /// <summary>
        /// A string identifying the player in use, preferably human-readable.
        /// If you have multiple players, this can be used to distinguish between them.
        /// 
        /// This field is optional.
        /// </summary>
        if(_s != STAT_INIT) _s.playerName /* : String */ = null;

        /// <summary>
        /// The URL from which video is loaded.
        /// 
        /// Note: If this changes during a session, there is no need to update
        /// this value - just use the URL from which loading initially occurs.
        /// </summary>
        if(_s != STAT_INIT) _s.streamUrl /* : String */ = null;

        /// <summary>
        /// Set to true if the session includes live content, and false otherwise.
        /// </summary>
        if(_s != STAT_INIT) _s._isLive /* : Boolean */ = false;

        defGet(_s,"isLive",__get_isLive);
        function __get_isLive()  { return  _s._isLive; }
        defSet(_s,"isLive",__set_isLive);
        function __set_isLive(value /* : Boolean */)  { 
                  if (value !== true && value !== false) { 
                      var utils /* : Utils */ = Utils.getInstance();
                      utils.err("Invalid value for ConvivaContentInfo.isLive, expected boolean.");
                      value = false;
                  }
                  _s._isLive = value;
                }



        ///////////////////////////////////////////////////////////////////////
        // End public fields
        ///////////////////////////////////////////////////////////////////////

        if(_s == STAT_INIT) ConvivaContentInfo.MAX_PARAMETER_LENGTH /* : int */ = 128;

        if(_s == STAT_INIT) ConvivaContentInfo.DEFAULT_ASSET_NAME /* : String */ = "Null title";

        ///<summary>
        ///Create a new ConvivaContentInfo having assetName @pAssetName, 
        ///candidateResources @pCandidateResources, and tags @pTags.
        ///
        /// </summary>
        function _constr(assetName /* : String */, tags /* : Object */) {
            _s.assetName = assetName;
            _s.tags = tags; // Use the setter
            if (_s._tags == null) {
                _s._tags = new DictionaryCS/*<string, string>*/();
            }

        }


       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_AKAMAI
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_AMAZON
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_ATT
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_BITGRAVITY
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_BT
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_CDNETWORKS
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_CHINACACHE
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_COMCAST
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_EDGECAST
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_HIGHWINDS
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_INHOUSE
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_INTERNAP
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_LEVEL3
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_LIMELIGHT
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_MICROSOFT
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_OCTOSHAPE
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_OTHER
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_SWARMCAST
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_TELEFONICA
       //JSRENAME:constant:_s:ConvivaContentInfo:CDN_NAME_VELOCIX
       //JSRENAME:constant:_s:ConvivaContentInfo:DEFAULT_ASSET_NAME
       //JSRENAME:constant:_s:ConvivaContentInfo:DEVICE_TYPE_CONSOLE
       //JSRENAME:constant:_s:ConvivaContentInfo:DEVICE_TYPE_MOBILE
       //JSRENAME:constant:_s:ConvivaContentInfo:DEVICE_TYPE_PC
       //JSRENAME:constant:_s:ConvivaContentInfo:DEVICE_TYPE_SET_TOP_BOX
       //JSRENAME:constant:_s:ConvivaContentInfo:FRAMEWORK_NAME_BRIGHTCOVE
       //JSRENAME:constant:_s:ConvivaContentInfo:FRAMEWORK_NAME_KALTURA
       //JSRENAME:constant:_s:ConvivaContentInfo:FRAMEWORK_NAME_OOYALA
       //JSRENAME:constant:_s:ConvivaContentInfo:FRAMEWORK_NAME_OSMF
       //JSRENAME:constant:_s:ConvivaContentInfo:FRAMEWORK_NAME_THE_PLATFORM
       //JSRENAME:constant:_s:ConvivaContentInfo:MAX_PARAMETER_LENGTH
       //JSRENAME:constant:_s:ConvivaContentInfo:PLUGIN_NAME_KALTURA
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(ConvivaContentInfo,"ConvivaContentInfo");
    // API_EXIT



// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../csClient/CwsClient/ConvivaStreamerProxy.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: George Necula (necula@conviva.com), Henry Milner (henry@conviva.com), Yan Li (yan@conviva.com)



// namespace 
    // API_ENTER
    /// <summary>
    /// Abstract superclass for a generic content streamer for LivePass.
    /// </summary>
    function ConvivaStreamerProxy() {
        var _s = this;
        
        // Constants for the valid playing states
	/// STOPPED: player state of stopped
        if(_s == STAT_INIT) ConvivaStreamerProxy.STOPPED /* : String */ = "STOPPED";
	/// PLAYING: player state of playing
        if(_s == STAT_INIT) ConvivaStreamerProxy.PLAYING /* : String */ = "PLAYING";
	/// BUFFERING: player state of buffering
        if(_s == STAT_INIT) ConvivaStreamerProxy.BUFFERING /* : String */ = "BUFFERING";
	/// PAUSED: player state of paused
        if(_s == STAT_INIT) ConvivaStreamerProxy.PAUSED /* : String */ = "PAUSED";
        /// <summary>
	/// SEEKING: The streamer is currently seeking to a new playhead time.
	///
	/// Some streamers do not have a separate state for seeking.  In such cases,
        /// enter this state whenever a seek happens and then immediately transition
        /// to another state.
	/// </summary>
        if(_s == STAT_INIT) ConvivaStreamerProxy.SEEKING /* : String */ = "SEEKING";
	/// ERROR: player state of error
        if(_s == STAT_INIT) ConvivaStreamerProxy.ERROR /* : String */ = "ERROR";
	/// UNKNOWN: unknown player state
        if(_s == STAT_INIT) ConvivaStreamerProxy.UNKNOWN /* : String */ = "UNKNOWN";

        // Constants for the required metadata keys
	/// <summary>
        /// METADATA_DURATION: The duration of the content, in seconds. 
	/// 
	/// Useful for communicating content duration when it becomes known after the start of streaming.
	/// </summary>
        if(_s == STAT_INIT) ConvivaStreamerProxy.METADATA_DURATION /* : String */ = "duration";
        /// METADATA_FILESIZE: The file size of the content, in bytes.
        if(_s == STAT_INIT) ConvivaStreamerProxy.METADATA_FILESIZE /* : String */ = "filesize";
        /// METADATA_ENCODED_FRAMERATE: The encoded frame rate of the content, in frames per second.
        if(_s == STAT_INIT) ConvivaStreamerProxy.METADATA_ENCODED_FRAMERATE /* : String */ = "framerate";

        /** Constants for the valid switching statuses */
        /// <summary>
        /// The switch has not yet started.
        /// </summary>
        if(_s == STAT_INIT) ConvivaStreamerProxy.PENDING /* : String */ = "PENDING";
        /// <summary>
        /// The switch has started but has not completed.
        /// </summary>
        if(_s == STAT_INIT) ConvivaStreamerProxy.IN_PROGRESS /* : String */ = "IN_PROGRESS";
        /// <summary>
        /// The switch has completed successfully. The user has begun to get media from
        /// the target stream.
        /// </summary>
        if(_s == STAT_INIT) ConvivaStreamerProxy.SUCCEEDED /* : String */ = "SUCCEEDED";
        /// <summary>
        /// The switch failed due to an error. The original stream is still in use.
        /// </summary>
        if(_s == STAT_INIT) ConvivaStreamerProxy.FAILED /* : String */ = "FAILED";
        /// <summary>
        /// During the switch, user action caused the switch to be aborted, but no error 
        /// occurred. The original stream is still in use.
        /// </summary>
        if(_s == STAT_INIT) ConvivaStreamerProxy.INTERRUPTED /* : String */ = "INTERRUPTED";


        /// Default resource name for an implementation that does not use resources
        if(_s == STAT_INIT) ConvivaStreamerProxy.NO_RESOURCE /* : String */ = "no_resource";

        // Constants for boolean values that may also be unavailable
        if(_s == STAT_INIT) ConvivaStreamerProxy.BOOLEAN_TRUE /* : int */ = 1;
        if(_s == STAT_INIT) ConvivaStreamerProxy.BOOLEAN_FALSE /* : int */ = -1;
        if(_s == STAT_INIT) ConvivaStreamerProxy.BOOLEAN_UNAVAILABLE /* : int */ = 0;

        // Constant for encoding the capabilities
        if(_s == STAT_INIT) ConvivaStreamerProxy.CAPABILITY_VIDEO /* : int */ = 1;
        if(_s == STAT_INIT) ConvivaStreamerProxy.CAPABILITY_QUALITY_METRICS /* : int */ = 2;
        if(_s == STAT_INIT) ConvivaStreamerProxy.CAPABILITY_BITRATE_METRICS /* : int */ = 4;

        function _constr() {
            // Explicit constructor here for inheritance in Lua.
        }

        /// <summary>
        /// Cleanup : This function is called when the monitoring session is terminated.
        /// </summary>
        defPubMeth(_s,"Cleanup",__Cleanup);
        function __Cleanup() /* : void */  {
        }

        /// Upon construction of the ConvivaStreamerProxy, a StreamerProxyState object should be constructed
        /// and should be used to store the current playing state, and to notify of changes. This ensures
        /// that the Conviva LivePass gets the proper notifications. 
        /// 
        defPubMeth(_s,"GetStreamerStateManager",__GetStreamerStateManager);
        function __GetStreamerStateManager() /* : StreamerStateManager */  {
            throw new Error("GetStreamerStateManager must be implemented in subclasses");
        }

        /// Get a bitmap with the capabilities supported by this ConvivaStreamerProxy
        defPubMeth(_s,"GetCapabilities",__GetCapabilities);
        function __GetCapabilities() /* : int */  {
            return ConvivaStreamerProxy.CAPABILITY_VIDEO + ConvivaStreamerProxy.CAPABILITY_QUALITY_METRICS; 
        }

        /// <summary>
        /// GetPlayheadTimeMs : 
        /// return the current position of the play head, in milliseconds since the start of the content.
        /// 
        /// -1 if not available. 
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetPlayheadTimeMs",__GetPlayheadTimeMs);
        function __GetPlayheadTimeMs() /* : int */  {
            return -1;
        }
        

        /// <summary>
        /// GetBufferLengthMs : 
        /// return the number of milliseconds-worth of data present in the buffer. 
        /// 
        /// -1 if not available.
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetBufferLengthMs",__GetBufferLengthMs);
        function __GetBufferLengthMs() /* : int */  {
            return -1;
        }


        /// <summary>
        /// GetStartingBufferLengthMs : 
        /// return the buffer length threshold (in milliseconds) at which the streamer will 
        /// transition from buffering to playing. 
	/// 
	/// Most streamers do not begin playing immediately
        /// when data is available, but rather wait for the buffer to reach a certain threshold before
        /// playing, ensuring smoother playback. For example, if this number is 1000, and the streamer 
        /// enters the buffering state with an empty buffer at time 0, the streamer will begin playing 
        /// once the buffer is sufficient to play 1000ms of media.
        /// 
        /// This threshold can change.
        /// 
        /// -1 if unknown or unavailable.
        /// </summary>
        /// \remark Conviva Quality Insights
        defPubMeth(_s,"GetStartingBufferLengthMs",__GetStartingBufferLengthMs);
        function __GetStartingBufferLengthMs() /* : int */  {
            return -1;
        }


        /// <summary>
        /// GetRenderedFrameRate : 
        /// return the current rendering frame rate (frames per second).
        /// 
        /// -1.0 if not available.
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetRenderedFrameRate",__GetRenderedFrameRate);
        function __GetRenderedFrameRate() /* : Number */  {
            return -1.0;
        }

        /// <summary>
        /// GetDroppedFrames : 
        /// return the number of dropped frames, cumulative since the start of the session.
        /// 
        /// -1 if not available. 
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetDroppedFrames",__GetDroppedFrames);
        function __GetDroppedFrames() /* : int */  {
            return -1;
        }

        /// <summary>
        /// GetCpuPercent : return the percentage of CPU usage.
        /// 
        /// -1.0 if not available.
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetCpuPercent",__GetCpuPercent);
        function __GetCpuPercent() /* : Number */  {
            return -1.0;
        }

        /// <summary>
        /// GetMemoryPercent : return the percentage of memory usage
        /// 
        /// -1.0 if not available.
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetMemoryPercent",__GetMemoryPercent);
        function __GetMemoryPercent() /* : Number */  {
            return -1.0;
        }

        /// <summary>
        /// GetCapacityKbps : 
        /// return an estimation of the current capacity, in kilo-bits-per-second (i.e. 1000 bits per 
        /// second), of the connection between this client and the specified resource.
        /// 
        /// -1 if not available.
        /// </summary>
        /// <param name="r">Instance of Resource for which the bandwidth capacity is requested</param>
 	/// \remark Conviva Quality Insights
        defPubMeth(_s,"GetCapacityKbps",__GetCapacityKbps);
        function __GetCapacityKbps(r /* : StreamInfo */) /* : int */  {
            return -1;
        }

 
        /// <summary>
        /// GetTotalLoadedBytes : 
        /// return the number of bytes downloaded so far (cumulative, includes the bytes in the buffer), 
        /// for the specific resource.
        /// 
        /// -1.0  if not available.
        /// </summary>
        /// <param name="r">Instance of Resource for which the downloaded bytes is requested</param>
	/// \remark Conviva Quality Insights
        defPubMeth(_s,"GetTotalLoadedBytes",__GetTotalLoadedBytes);
        function __GetTotalLoadedBytes(r /* : StreamInfo */) /* : Number */  {
            return -1.0;
        }


        /// <summary>
        /// GetServerAddress : 
        /// return the server address for the current playing resource. 
	/// 
	/// The returned string should be
        /// a representation of the IP address if available (e.g., "127.0.0.1"), or a domain name
        /// otherwise.
        /// 
        /// Null if the server address is not known. 
        /// </summary>
	/// \remark Conviva Quality Insights
        defPubMeth(_s,"GetServerAddress",__GetServerAddress);
        function __GetServerAddress() /* : String */  {
            return null;
        }


        /// <summary>
        /// GetIsLive :
        /// return ConvivaStreamerProxy.BOOLEAN_TRUE if the streamer is streaming live media, and
        /// ConvivaStreamerProxy.BOOLEAN_FALSE otherwise.
        /// 
        /// ConvivaStreamerProxy.BOOLEAN_UNAVAILABLE if unknown.
        /// </summary>
	/// \remark Conviva Quality Insights
        defPubMeth(_s,"GetIsLive",__GetIsLive);
        function __GetIsLive() /* : int */  {
            return ConvivaStreamerProxy.BOOLEAN_UNAVAILABLE;
        }

        /// <summary>
        /// GetStreamerType : 
        /// return an identifier for the streamer this is proxying. 
	/// 
	/// This enables special
        /// switching logic that is particular to the streamer, if there is any. For 
        /// example, a chunk-based streamer made by Conviva might identify itself here as
        /// "ConvivaChunkStreamer" so that LivePass can make switching decisions tailored for 
        /// that particular streamer. The identifier should be negotatied with Conviva during 
        /// integration.
        /// 
        /// This should not change throughout the life of this ConvivaStreamerProxy.
        /// 
        /// null if the streamer type is not known or does not need to be identified.
        /// </summary>
	/// \remark Conviva Quality Insights, Conviva PreCision: Bitrate Switching, Resource Switching
        defPubMeth(_s,"GetStreamerType",__GetStreamerType);
        function __GetStreamerType() /* : String */  {
            return null;
        }


        /// <summary>
        /// GetStreamerVersion : 
        /// return a version number associated with the StreamerType above. 
	///  
	/// At a minimum,
        /// this number should be changed whenever the streamer implementation changes in a 
        /// way that could substantially impact switching.
        /// 
        /// -1 if the streamer type is null or if it does not have multiple versions.
        /// </summary>
	/// \remark Conviva Quality Insights, Conviva PreCision: Bitrate Switching, Resource Switching
        defPubMeth(_s,"GetStreamerVersion",__GetStreamerVersion);
        function __GetStreamerVersion() /* : int */  {
            return -1;
        }


        /// <summary>
        /// StartPlay :
        /// Called by LivePass to indicate to the streamer that it should start play 
        /// with the specified candidate stream.
        /// </summary>
	    /// <param name="selectionReturnCode"> success or failure returned by initial stream selection. If the selectionReturnCode is not LivePass.STREAM_SELECTION_SUCC, the other two params should not  be used. </param>
        /// <param name="c">Instance of CandidateStream representing the selected stream to start with.</param>
	    /// <param name="byConviva"> bool if it's ConvivaStreamerProxy.BOOLEAN_FALSE, 
	    /// then this call is a result of an initialStreamSelection call made to the LivePass; If it's ConvivaStreamerProxy.BOOLEAN_TRUE, then this is the result of Conviva initiated play</param>
        /// \remark Conviva PreCision: Bitrate Switching, Resource Switching
        defPubMeth(_s,"StartPlay",__StartPlay);
        function __StartPlay(selectionReturnCode /* : int */, candidateStreams /* : Object */, byConviva /* : int */) /* : void */  {
        }

        /// <summary>
        /// DoSwitch : 
        /// Called by LivePass to indicate to the streamer that it should execute a switch to the target
        /// stream.
        /// </summary>
	/// <param name="selectionReturnCode"> success or failure returned by initial stream selection. If the selectionReturnCode is not LivePass.STREAM_SELECTION_SUCC, the other two params should not  be used. </param>
        /// <param name="c">Instance of CandidateStream representing the target stream to switch to</param>
	/// <param name="byConviva"> bool if it's ConvivaStreamerProxy.BOOLEAN_FALSE, 
	/// then this call is a result of an initialStreamSelection call made to the LivePass; If it's ConvivaStreamerProxy.BOOLEAN_TRUE, then this is the result of Conviva initiated play</param>
        /// \remark Conviva PreCision: Bitrate Switching, Resource Switching
        defPubMeth(_s,"DoSwitch",__DoSwitch);
        function __DoSwitch(selectionReturnCode /* : int */, candidateStreams /* : Object */, byConviva /* : int */) /* : void */  {
        }

       //JSRENAME:constant:_s:ConvivaStreamerProxy:BOOLEAN_FALSE
       //JSRENAME:constant:_s:ConvivaStreamerProxy:BOOLEAN_TRUE
       //JSRENAME:constant:_s:ConvivaStreamerProxy:BOOLEAN_UNAVAILABLE
       //JSRENAME:constant:_s:ConvivaStreamerProxy:BUFFERING
       //JSRENAME:constant:_s:ConvivaStreamerProxy:CAPABILITY_BITRATE_METRICS
       //JSRENAME:constant:_s:ConvivaStreamerProxy:CAPABILITY_QUALITY_METRICS
       //JSRENAME:constant:_s:ConvivaStreamerProxy:CAPABILITY_VIDEO
       //JSRENAME:constant:_s:ConvivaStreamerProxy:ERROR
       //JSRENAME:constant:_s:ConvivaStreamerProxy:FAILED
       //JSRENAME:constant:_s:ConvivaStreamerProxy:INTERRUPTED
       //JSRENAME:constant:_s:ConvivaStreamerProxy:IN_PROGRESS
       //JSRENAME:constant:_s:ConvivaStreamerProxy:METADATA_DURATION
       //JSRENAME:constant:_s:ConvivaStreamerProxy:METADATA_ENCODED_FRAMERATE
       //JSRENAME:constant:_s:ConvivaStreamerProxy:METADATA_FILESIZE
       //JSRENAME:constant:_s:ConvivaStreamerProxy:NO_RESOURCE
       //JSRENAME:constant:_s:ConvivaStreamerProxy:PAUSED
       //JSRENAME:constant:_s:ConvivaStreamerProxy:PENDING
       //JSRENAME:constant:_s:ConvivaStreamerProxy:PLAYING
       //JSRENAME:constant:_s:ConvivaStreamerProxy:SEEKING
       //JSRENAME:constant:_s:ConvivaStreamerProxy:STOPPED
       //JSRENAME:constant:_s:ConvivaStreamerProxy:SUCCEEDED
       //JSRENAME:constant:_s:ConvivaStreamerProxy:UNKNOWN
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(ConvivaStreamerProxy,"ConvivaStreamerProxy");
    // API_EXIT

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../csClient/CwsClient/LivePass.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: George Necula (necula@conviva.com), Henry Milner (henry@conviva.com), Yan Li (yan@conviva.com)



// namespace 
function LivePass() {
        var _s = this;
    /// READY : LivePass state is ready for use.
    if(_s == STAT_INIT) LivePass.READY /* : String */ = "READY";


    ///STREAM_SELECTION_SUCC : return code for initial or midstream selection success
    if(_s == STAT_INIT) LivePass.STREAM_SELECTION_SUCC /* : int */ = 0;
    /// STREAM_SELECTION_FAILURE : return code for initial or midstream selection failure
    if(_s == STAT_INIT) LivePass.STREAM_SELECTION_FAILURE /* : int */ = 1;
    /// STREAM_SELECTION_TIMEOUT : return code for initial or midstream selection timeout
    if(_s == STAT_INIT) LivePass.STREAM_SELECTION_TIMEOUT /* : int */ = 2;

    /// state: is LivePass ready?
    if(_s == STAT_INIT) LivePass.readyState /* : Boolean */ = false;

    //@owner
    if(_s == STAT_INIT) LivePass._settings /* : Settings */ = null;

    //@owner
    if(_s == STAT_INIT) LivePass._utils /* : Utils */ = null;
    //@owner
    if(_s == STAT_INIT) LivePass._sessionFactory /* : SessionFactory */ = null;

    //flag indicating if toggleTraces is set in LivePass. Needed if the traces are toggled before LivePass is loaded
    if(_s == STAT_INIT) LivePass._toggleTracesSet /* : Boolean */ = false;
    //the set value of toggleTraces
    if(_s == STAT_INIT) LivePass._toggleTracesSetValue /* : Boolean */ = false;

    // The id of the global session. -1 if not yet created. 
    if(_s == STAT_INIT) LivePass._globalSessionId /* : int */ = -1;

    /// <summary>
    /// init : initialize the LivePass module
    ///
    /// see Conviva integration guide for details of how to use this method.
    /// </summary>
    /// <param name="apiKey">a key assigned by Conviva to uniquely identify a Conviva customer </param>
    defStatMeth(_s,LivePass,"init",__init);
    function __init(apiKey /* : String */) /* : void */  {
        LivePass._utils = Utils.CreateUtils(null); // Retrieve, or create Utils object
        LivePass._utils.startFetchClientId();
        LivePass._settings = LivePass._utils.getSettings();
        LivePass._settings.clientInstanceId = LivePass._utils.randInt();
        LivePass._settings.apiKey = apiKey;
        LivePass._utils.log("Cws.init url=" + LivePass._settings.gatewayUrl + " apiKey=" + LivePass._settings.apiKey);
        LivePass._utils.runProtectedSync(
            function () { 
                if (LivePass._toggleTracesSet) {
                    LivePass._settings.enableLogging = LivePass._toggleTracesSetValue;
                }
                PlayerStates.init();
                LivePass._sessionFactory = new SessionFactory();
                LivePass.readyState = true;
                LivePass._utils.log("LivePass.init(): Done.");
             }, "LivePass.init");
    }


    /// <summary>
    /// initWithSettings : initialize LivePass, with advanced settings parameters. 
    /// This function should be used only under guidance from Conviva.
    /// </summary>
    /// <param name="apiKey">a key assigned by Conviva to uniquely identify a Conviva customer </param>
    /// <param name="settings">a dictionary with string keys and object values</param>
    defStatMeth(_s,LivePass,"initWithSettings",__initWithSettings);
    function __initWithSettings(apiKey /* : String */, settings /* : Object */) /* : void */  {
        /* We prefer to bundle the settings with the init, to ensure that the settings take effect 
         * as soon as possible, without possibility of interleaving other work between settings and init. 
         */ 
        Utils.CreateUtils(settings); // update the settings
        LivePass.init(apiKey);
    }


    /// <summary>
    /// cleanup : method to clean up the LivePass when exiting. 
    /// </summary>
    defStatMeth(_s,LivePass,"cleanup",__cleanup);
    function __cleanup() /* : void */  {
        if (LivePass._utils != null) {
            LivePass._utils.runProtected(
                function () { 
                    LivePass._utils.log("LivePass.cleanup()");
                    if (LivePass._sessionFactory != null) {
                        LivePass._sessionFactory.cleanup(); // Cleans up all sessions, too.
                    }
                    LivePass._sessionFactory = null;
                    LivePass._globalSessionId  = -1;
                    if (LivePass._utils != null) {
                        LivePass._utils.cleanup();
                    }
                    LivePass._utils = null;
                }, "LivePass.cleanup");
        }
    }

    /// <summary>
    /// createSession : Create a monitoring session, without Conviva PreCision control.
    /// 
    /// Please refer to Conviva integration guide for more details and examples.
    /// </summary>
    /// <param name="streamer"> a streamer or a ConvivaStreamerProxy object for this session</param>
    /// <param name="contentInfo"> the ConvivaContentInfo object used for this session</param>
    defStatMeth(_s,LivePass,"createSession",__createSession);
    function __createSession(streamer /* : Object */, contentInfo /* : ConvivaContentInfo */) /* : int */  {
        // We generate the session id outside runProtected because the latter may be asynchronous
        if (! LivePass.readyState) {
            LivePass._utils.ping("createSession before LivePass.init");
            throw new Error("createSession before LivePass.init");
        }
        var sid /* : int */ = LivePass._sessionFactory.newSessionId();
        LivePass._utils.runProtected(
            function () { 
                var session /* : Session */ = LivePass._sessionFactory.makeSession(streamer, contentInfo, sid);
            }, "LivePass.createSession");
        return sid;
    }


    /// <summary>
    /// setCurrentStreamInfo : Set the current bitrate and/or current resource
    /// </summary>
    /// <param name="streamInfo">The new stream info. The bitrate is ignored if -1, the resource is ignored if null</param>
    defStatMeth(_s,LivePass,"setCurrentStreamInfo",__setCurrentStreamInfo);
    function __setCurrentStreamInfo(sessionId /* : int */, streamInfo /* : StreamInfo */) /* : void */  {
        if (! LivePass.readyState) {
            LivePass._utils.ping("setCurrentStreamInfo before LivePass.init");
            return;
        }
        LivePass._utils.runProtected(
            function () { 
                var session /* : Session */ = LivePass._sessionFactory.getSession(sessionId);
                session.setCurrentStreamInfo(streamInfo);
            }, "setCurrentStreamInfo");
    }


    /// <summary>
    /// setCurrentStreamMetadata : Set various metadata parameters for the stream
    /// The metadata object should be a dictionary from metadata field names to metadata values (as strings).
    /// The names of the valid keys are defined in ConvivaStreamerProxy as constants:
    ///  - duration  (duration of the stream in seconds)
    ///  - framerate (encoded framerate)
    ///  - filesize  (filesize in bytes)
    /// Other keys are ignored.
    /// 
    /// If the callback is called multiple times, the most recent value for each key will be used. For
    /// example, calling the callback first with { "duration":"100" } and immediately thereafter with 
    /// { "framerate":"30" } is equivalent to calling it once with { "duration":"100", "framerate":"30" }.
    /// </summary>

    /// </summary>
    /// <param name="metadata">The new metadata</param>
    defStatMeth(_s,LivePass,"setCurrentStreamMetadata",__setCurrentStreamMetadata);
    function __setCurrentStreamMetadata(sessionId /* : int */, metadata /* : Object */) /* : void */  {
        if (!LivePass.readyState) {
            LivePass._utils.ping("setCurrentMetadata before LivePass.init");
            return;
        }
        LivePass._utils.runProtected(
            function () { 
                var session /* : Session */ = LivePass._sessionFactory.getSession(sessionId);
                session.setCurrentStreamMetadata(metadata);
            }, "setCurrentStreamMetadata");
    }

    /// <summary>
    /// After a session is created, this function can be used to get the initial resource and bitrate selection from Conviva.
    /// When this function completes, it will call the candidateStreamsCallback with a list of StreamInfo objects ordered in 
    /// decreasing order of preference. 
    /// The application is supposed to pick one of the StreamInfo objects, direct the streamer to play the selected stream, 
    /// and then call setCurrentStreamInfo to communicate to LivePass the choice. 
    /// 
    /// </summary>
    /// <param name="sessionId"> returned by the createSession, createPrecisionSession </param>
    defStatMeth(_s,LivePass,"initialResourceBitrateSelection",__initialResourceBitrateSelection);
    function __initialResourceBitrateSelection(sessionId /* : int */, candidateStreamsCallback /* : Function(Action(object)) */) /* : void */  {
        if (! LivePass.readyState) {
            LivePass._utils.ping("createSession before LivePass.init");
            return;
        }
        LivePass._utils.runProtected(
            function () { 
                var session /* : Session */ = LivePass._sessionFactory.getSession(sessionId);
                session.initialResourceBitrateSelection();
            }, "LivePass.initialResourceBitrateSelection");

    }


    /// <summary>
    /// createPrecisionSession : create a session that has Conviva PreCision enabled.
    /// 
    /// When a session has Conviva PreCision enabled, LivePass will invoke initial and midStream selection automatically.
    ///
    /// </summary>
    /// <param name="streamer"> a streamer or a ConvivaStreamerProxy object for this session</param>
    /// <param name="contentInfo"> the ConvivaContentInfo object used for this session</param>
    defStatMeth(_s,LivePass,"createPrecisionSession",__createPrecisionSession);
    function __createPrecisionSession(streamer /* : Object */, contentInfo /* : ConvivaContentInfo */) /* : int */  {
        return 0;
    }

    /// <summary>
    /// This function is called any time that switch control decided that it may needs to switch to other resources or bitrate.
    ///
    /// The function will call DoSwitch through ConvivaStreamerProxy when it returns. Application can decide what to do then.
    /// </summary>
    /// <param name="sessionId"> id returned by the createSession, createPrecisionSession </param>
    /// <param name="switchTrigger"> a string to represent the reason triggered for this mid stream selection </param>
    defStatMeth(_s,LivePass,"midStreamResourceBitrateSelection",__midStreamResourceBitrateSelection);
    function __midStreamResourceBitrateSelection(sessionId /* : int */, switchTrigger /* : String */, candidateStreamsCallback /* : Function(Action(object)) */) /* : void */  {
        if (!LivePass.readyState) {
            LivePass._utils.ping("createSession before LivePass.init");
            return;
        }
        LivePass._utils.runProtected(
            function () { 
                var session /* : Session */ = LivePass._sessionFactory.getSession(sessionId);
                session.midStreamResourceBitrateSelection(switchTrigger);
            }, "LivePass.midStreamResourceBitrateSelection");
    }


    /// <summary>
    /// This function is used to send Conviva Player Insights Event, associated with a session. 
    /// Events have a name and a list of key value pair as event attributes.
    /// 
    /// </summary>
    /// <param name="sessionId"> returned by the createSession, createPrecisionSession </param>
    /// <param name="eventName"> a name for the event </param>
    /// <param name="eventAttributes"> a dictionary of key value pair associated with the event </param>
    defStatMeth(_s,LivePass,"sendSessionEvent",__sendSessionEvent);
    function __sendSessionEvent(sessionId /* : int */, eventName /* : String */, eventAttributes /* : Object */) /* : void */  {
        if (!LivePass.readyState) {
            LivePass._utils.ping("sendEvent before LivePass.init");
            return;
        }
        var eventAttrDictCS /* : DictionaryCS(string, object) */ = null;
        eventAttrDictCS = Lang.DictionaryFromRepr/*string, string*/(eventAttributes);
        LivePass._utils.runProtected(
            function () { 
                var session /* : Session */ = LivePass._sessionFactory.getSession(sessionId);
                session.sendCustomEvent(eventName, eventAttrDictCS);
            }, "LivePass.sendEvent");
    }


    /// <summary>
    /// This function is used to send Conviva Player Insights Event, not assocated with a video session
    /// Events have a name and a list of key value pair as event attributes.
    /// 
    /// </summary>
    /// <param name="eventName"> a name for the event </param>
    /// <param name="eventAttributes"> a dictionary of key value pair associated with the event </param>
    defStatMeth(_s,LivePass,"sendEvent",__sendEvent);
    function __sendEvent(eventName /* : String */, eventAttributes /* : Object */) /* : void */  {

        if (!LivePass.readyState) {
            LivePass._utils.ping("sendGlobalEvent before LivePass.init");
            return;
        }
        LivePass._utils.runProtected(
            function () { 
                // Have we created the global session already ? 
                if (LivePass._globalSessionId < 0) {
                    var cci /* : ConvivaContentInfo */ = new ConvivaContentInfo("", new DictionaryCS/*<string,string>*/ ());
                    LivePass._globalSessionId = LivePass.createSession(null, cci);
                }
                LivePass.sendSessionEvent(LivePass._globalSessionId, eventName, eventAttributes);
            }, "LivePass.sendGlobalEvent"
        );

    }

    /// <summary>
    /// cleanupSession : should be called when a video session is over and to be cleaned up.
    /// 
    /// </summary>
    /// <param name="sessionId"> id returned by the createSession, createPrecisionSession </param>
    defStatMeth(_s,LivePass,"cleanupSession",__cleanupSession);
    function __cleanupSession(sessionId /* : int */) /* : void */  {
        if (!LivePass.readyState)
            return;
        LivePass._utils.runProtected(
            function () { 
                LivePass._sessionFactory.cleanupSession(sessionId);
            }, "Livepass.cleanupSession");
    }
    
    /// <summary>
    /// Toggle on/off console traces.
    ///
    /// </summary>
    /// <param name="b"> true if enble console trace. false otherwise </param>
    defStatMeth(_s,LivePass,"toggleTraces",__toggleTraces);
    function __toggleTraces(b /* : Boolean */) /* : void */  {
        // We cannot use _utils.runProtected because this function can be used before we LivePass.init
        // Fortunately, there is not much that can go wrong in this function, but be extra careful.
        LivePass._toggleTracesSet = true;
        if (LivePass._settings != null) {
            LivePass._settings.enableLogging = b;
        } else {
            LivePass._toggleTracesSetValue = b;
        }
    }

       //JSRENAME:constant:_s:LivePass:READY
       //JSRENAME:constant:_s:LivePass:STREAM_SELECTION_FAILURE
       //JSRENAME:constant:_s:LivePass:STREAM_SELECTION_SUCC
       //JSRENAME:constant:_s:LivePass:STREAM_SELECTION_TIMEOUT
}
statInit(LivePass,"LivePass");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../csClient/CwsClient/StreamInfo.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: George Necula (necula@conviva.com), Henry Milner (henry@conviva.com), Yan Li (yan@conviva.com)


// namespace 
    // API_ENTER
    /// <summary>
    /// A stream to which a ConvivaStreamerProxy can potentially switch.
    /// This object contains no information about the actual URL of the stream it represents.
    /// </summary>
    function StreamInfo() {
        var _s = this;

        /// <summary>
        /// Construct a stream having bitrate <paramref name="bitrate"/> in kilo-bits-per-second and streaming
        /// from a resource named <paramref name="resource"/>.

        // The bitrate of the stream, in kilo-bits-per-second, or 1000 bits per second. Pass -1 if the bitrate of this stream is unknown or irrelevant.
        if(_s != STAT_INIT) _s._bitrateKbps /* : int */ = -1;

        // The name of the resource from which this stream is loaded. For example, this could be the name of a CDN. 
        // Use null if the resource of this stream is unknown or irrelevant
        if(_s != STAT_INIT) _s._resource /* : String */ = null;

        // The name of the CDN on which the resource exists. 
        // Use null if the CDN is unknown 
        if(_s != STAT_INIT) _s._cdnName /* : String */ = null;

        /// </summary>
        /// <param name="bitrate">The bitrate of the stream, in kilo-bits-per-second, or 1000 bits per second. Pass -1 if the bitrate of this stream is unknown or irrelevant.</param>
        /// <param name="cdnName">The name of the CDN from which the resource is loaded. Pass null if the CDN is unknown</param>
        /// <param name="resource">The name of the resource from which this stream is loaded. Pass null if the resource of this stream is unknown or irrelevant.</param>
        function _constr(bitrateKbps /* : int */, cdnName /* : String */, resource /* : String */) {
            _s._bitrateKbps = bitrateKbps;
            _s._cdnName = cdnName;
            if(_s._cdnName == null) {
                _s._cdnName = ConvivaContentInfo.CDN_NAME_OTHER;
            }
            _s._resource = resource;
        }

        /// <summary>
        /// The bitrate of the stream, in 1000 bits per second; -1 if unknown
        /// </summary>

        defGet(_s,"bitrateKbps",__get_bitrateKbps);
        function __get_bitrateKbps()  { return  _s._bitrateKbps; }

        /// <summary>
        /// The CDN name. 
        /// </summary>

        defGet(_s,"cdnName",__get_cdnName);
        function __get_cdnName()  { return  _s._cdnName; }

        /// <summary>
        /// The resource from which the stream is loaded. null if unknown
        /// </summary>

        defGet(_s,"resource",__get_resource);
        function __get_resource()  { return  _s._resource; }

        defPubMeth(_s,"equals",__equals);
        function __equals(other /* : StreamInfo */) /* : Boolean */  {
            return _s._bitrateKbps == other.bitrateKbps && _s._resource == other.resource && _s._cdnName == other.cdnName;
        }

        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(StreamInfo,"StreamInfo");
    // API_EXIT

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../csClient/CwsClient/StreamerError.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: George Necula (necula@conviva.com), Henry Milner (henry@conviva.com), Yan Li (yan@conviva.com)


// namespace 
    function StreamerError() {
        var _s = this;

        //error scope
        //unknown scope.
        if(_s == STAT_INIT) StreamerError.eErrorScope_Unknown /* : int */ = 0;
        //single media segment
        if(_s == STAT_INIT) StreamerError.ERROR_SCOPE_STREAM_SEGMENT /* : int */ = 1;
        //single cdn, bitrate
        if(_s == STAT_INIT) StreamerError.ERROR_SCOPE_STREAM /* : int */ = 2;
        //whole cdn
        if(_s == STAT_INIT) StreamerError.ERROR_SCOPE_RESOURCE /* : int */ = 3;


        //error severity level
        /// <summary>
        /// Severity level of warning.
        /// </summary>
        if(_s == STAT_INIT) StreamerError.SEVERITY_WARNING /* : int */ = 0;
        /// <summary>
        /// Severity level of fatal error.
        /// </summary>
        if(_s == STAT_INIT) StreamerError.SEVERITY_FATAL /* : int */ = 1;

        /// <summary>
        /// Create an Error object without any scope
        /// </summary>
        /// <param name="errorCode">String identifying the encountered error</param>
        /// <param name="severity">Level of severity. Must be selected from the constant defined in PrecisionConsts</param>
        /// <returns>Error object</returns>
        defStatMeth(_s,StreamerError,"makeUnscopedError",__makeUnscopedError);
        function __makeUnscopedError(errorCode /* : String */, severity /* : int */) /* : StreamerError */  {
            return new StreamerError(errorCode, null, -1, severity, StreamerError.eErrorScope_Unknown);
        }

        /// <summary>
        ///  Create an Error instance with stream scope.
        /// </summary>
        /// <param name="errCode">String identifying the encountered error</param>
        /// <param name="severity">Level of severity. Must be selected from the constant defined in PrecisionConsts</param>
        /// <param name="stream">Instance of CandidateStream that the error is encountered for</param>
        /// <returns>Error object</returns>
        defStatMeth(_s,StreamerError,"makeStreamError",__makeStreamError);
        function __makeStreamError(errCode /* : String */, severity /* : int */, stream /* : StreamInfo */) /* : StreamerError */  {
            return new StreamerError(errCode, stream, -1, severity, StreamerError.ERROR_SCOPE_STREAM);
        }

        /// <summary>
        ///  Create an Error instance with segment scope.
        /// </summary>
        /// <param name="errCode">String identifying the encountered error</param>
        /// <param name="severity">Level of severity. Must be selected from the constant defined in PrecisionConsts</param>
        /// <param name="stream">Instance of CandidateStream that the error is encountered for</param>
        /// <param name="idx">Index of media segment that the error is encountered for</param>
        /// <returns>Error object</returns>
        defStatMeth(_s,StreamerError,"makeStreamSegmentError",__makeStreamSegmentError);
        function __makeStreamSegmentError(errCode /* : String */, severity /* : int */, stream /* : StreamInfo */, idx /* : int */) /* : StreamerError */  {
            return new StreamerError(errCode, stream, idx, severity, StreamerError.ERROR_SCOPE_STREAM_SEGMENT);
        }

        function _constr(errorCode /* : String */, stream /* : StreamInfo */, index /* : int */, severity /* : int */, scope /* : int */) {
            _s._errorCode = errorCode;

            _s._stream = stream;
            _s._index = index;
            _s._severity = severity;
            _s._scope = scope;
        }

        if(_s != STAT_INIT) _s._errorCode /* : String */ = undefined;
        defPubMeth(_s,"getErrorCode",__getErrorCode);
        function __getErrorCode() /* : String */  {
            return _s._errorCode;
        }


        if(_s != STAT_INIT) _s._severity /* : int */ = undefined;
        defPubMeth(_s,"getSeverity",__getSeverity);
        function __getSeverity() /* : int */  {
            return _s._severity;
        }

        if(_s != STAT_INIT) _s._scope /* : int */ = undefined;
        defPubMeth(_s,"getScope",__getScope);
        function __getScope() /* : int */  {
            return _s._scope;
        }

        if(_s != STAT_INIT) _s._stream /* : StreamInfo */ = undefined;
        defPubMeth(_s,"getStream",__getStream);
        function __getStream() /* : StreamInfo */  {
            return _s._stream;
        }

        if(_s != STAT_INIT) _s._index /* : int */ = undefined;
        defPubMeth(_s,"getIndex",__getIndex);
        function __getIndex() /* : int */  {
            return _s._index;
        }
       //JSRENAME:constant:_s:StreamerError:ERROR_SCOPE_RESOURCE
       //JSRENAME:constant:_s:StreamerError:ERROR_SCOPE_STREAM
       //JSRENAME:constant:_s:StreamerError:ERROR_SCOPE_STREAM_SEGMENT
       //JSRENAME:constant:_s:StreamerError:SEVERITY_FATAL
       //JSRENAME:constant:_s:StreamerError:SEVERITY_WARNING
       //JSRENAME:constant:_s:StreamerError:eErrorScope_Unknown
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(StreamerError,"StreamerError");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../csClient/CwsClient/StreamerStateManager.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: George Necula (necula@conviva.com), Henry Milner (henry@conviva.com), Yan Li (yan@conviva.com)


// namespace 
    // API_ENTER

    /// <summary>
    /// This class is provided as a convenience to the implementers of ConvivaStreamerProxy. It holds some of the 
    /// state needed by the Conviva LivePass, and provides the transparent notification on state changes.
    /// </summary>
    function StreamerStateManager() {
        var _s = this;
        // The current state, with their initial values

        // The current resource and bitrate
        // @nonnull
        if(_s != STAT_INIT) _s._stream /* : StreamInfo */ = undefined;

        if(_s != STAT_INIT) _s._playingState /* : String */ = ConvivaStreamerProxy.UNKNOWN;

        // The last metadata received, or null if none
        if(_s != STAT_INIT) _s._lastMetadata /* : Object */ = null;

        // The last error encountered, or null if none
        if(_s != STAT_INIT) _s._lastError /* : StreamerError */ = null;

        if(_s != STAT_INIT) _s._notifier /* : IMonitorNotifier */ = null;


        function _constr() {
            _s._stream = new StreamInfo(-1, ConvivaContentInfo.CDN_NAME_OTHER, null);
        }

        // The Monitor will call this function to start monitoring
        defPubMeth(_s,"startMonitoring",__startMonitoring);
        function __startMonitoring(notifier /* : IMonitorNotifier */) /* : void */  {
            _s._notifier = notifier;
        }

        /// <summary>
        /// GetPlayingState :
        /// return the current playing state. One of the playing state constants defined in ConvivaStreamerProxy
        ///  
        /// UNKNOWN if unknown.
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetPlayingState",__GetPlayingState);
        function __GetPlayingState() /* : String */  {
            return _s._playingState;
        }

        /// <summary>
        /// SetPlayingState:
        /// set the current playing state. 
        /// <param name="newState">the new state (one of the constants defined in ConvivaStreamerProxy)</param>
        /// </summary>
        defPubMeth(_s,"SetPlayingState",__SetPlayingState);
        function __SetPlayingState(newState /* : String */) /* : void */  {
            if (newState != _s._playingState && _s._notifier != null) {
                _s._notifier.SetPlayingState(PlayerStates.stateToInt.GetValue(newState));
            }
            _s._playingState = newState;
        }


        /// <summary>
        /// GetBitrateKbps :
        /// return the bitrate of the currently-playing media, in kilo-bits-per-second (i.e. 1000 bits per second). 
        ///
        /// When this changes, the SetStream method of the notifier should be called.
        /// 
        /// -1 if not available.
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetBitrateKbps",__GetBitrateKbps);
        function __GetBitrateKbps() /* : int */  {
            return _s._stream.bitrateKbps;
        }

        /// <summary>
        /// GetCdnName :
        /// return the current CDN name
        ///  

        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetCdnName",__GetCdnName);
        function __GetCdnName() /* : String */  {
            return _s._stream.cdnName;
        }


        /// <summary>
        /// GetResource :
        /// return the current resource
        ///  
        /// null if unknown.
        /// </summary>
        /// \remark Conviva Quality Insights 
        defPubMeth(_s,"GetResource",__GetResource);
        function __GetResource() /* : String */  {
            return _s._stream.resource;
        }

        /// <summary>
        /// The current CandidateStream
        /// </summary>
        /// <returns></returns>
        defPubMeth(_s,"GetStream",__GetStream);
        function __GetStream() /* : StreamInfo */  {
            return _s._stream;
        }

        /// <summary>
        /// SetStream : 
        /// Set a new resource and bitrate switch
        /// </summary>
        /// <param name="updatedStream">the updated stream.</param>
        /// <impacts> Conviva PreCision: Bitrate Switching, Resource Switching</impacts>
        defPubMeth(_s,"SetStream",__SetStream);
        function __SetStream(updatedStream /* : StreamInfo */) /* : void */  {
            if (_s._notifier != null && ! _s._stream.equals(updatedStream)) {
                _s._notifier.SetStream(updatedStream);
            }
            _s._stream = updatedStream;
        }


        /// <summary>
        /// OnError : 
        /// The streamer proxy should invoke this function whenever an error is encountered.
        /// 
        /// For streamers that support Bitrate Switching or CDN Resource Switching, it is especially important to report as many errors
        /// as possible. In particular, any errors that might be resolved by a switch should be reported.
        /// </summary>
        /// <param name="error">An instance of Error must be passed into the callback. 
        ///                     The Error instance should be created by calling one of the makeError_* function defined in Error. 
        /// </param>
        /// \remark Conviva Quality Insights
        defPubMeth(_s,"OnError",__OnError);
        function __OnError(error /* : StreamerError */) /* : void */  {
            if (_s._notifier != null) {
                _s._notifier.OnError(error);
            }
            _s._lastError = error;
        }

        /// <summary>
        /// Return the last error, or null, if none
        /// </summary>
        /// <returns></returns>
        defPubMeth(_s,"GetLastError",__GetLastError);
        function __GetLastError() /* : StreamerError */  {
            return _s._lastError;
        }


        /// <summary>
        /// OnMetadata : 
        /// The streamer proxy should invoke the callback whenever metadata is received for the streaming content.
        ///
        /// The metadata object should be a dictionary from metadata field names to metadata values.
        /// The names of the valid keys are defined in ConvivaStreamerProxy as constants.  Other keys are ignored.
        /// 
        /// If the callback is called multiple times, the most recent value for each key will be used. For
        /// example, calling the callback first with { "duration":"100" } and immediately thereafter with 
        /// { "framerate":"30" } is equivalent to calling it once with { "duration":"100", "framerate":"30" }.
        /// </summary>
        /// <param name="callback"> Callback function that takes native dictionary representing the metadata as a parameter
        /// </param>
        /// \remark Conviva Quality Insights
        defPubMeth(_s,"OnMetadata",__OnMetadata);
        function __OnMetadata(metadata /* : Object */) /* : void */  {
            if (_s._notifier != null) {
                _s._notifier.OnMetadata(metadata);
            }
            _s._lastMetadata = metadata;
        }

        /// <summary>
        /// Get the last received metadata, or null if none
        /// </summary>
        /// <returns></returns>
        defPubMeth(_s,"GetLastMetadata",__GetLastMetadata);
        function __GetLastMetadata() /* : Object */  {
            return _s._lastMetadata;
        }


        /// <summary>
        /// The ConvivaStreamerProxy can use this function to add a log message to Conviva LivePass.
        /// </summary>
        /// <param name="message"></param>
        defPubMeth(_s,"Log",__Log);
        function __Log(message /* : String */) /* : void */  {
            if (_s._notifier != null) {
                _s._notifier.Log(message);
            }
        }


        /// <summary>
        /// The ConvivaStreamerProxy can use this function to add an error log message to Conviva LivePass.
        /// </summary>
        /// <param name="message"></param>
        defPubMeth(_s,"LogError",__LogError);
        function __LogError(message /* : String */) /* : void */  {
            if (_s._notifier != null) {
                _s._notifier.Log("ERROR:" + message);
            }
        }

        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(StreamerStateManager,"StreamerStateManager");
    // API_EXIT

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/monitor/Monitor.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: Henry Milner (henry@conviva.com), George Necula (necula@conviva.com), Yan Li (yan@conviva.com)




// namespace monitor

    function Monitor() {
        var _s = this;


        if(_s != STAT_INIT) _s._sessionId /* : int */ = 0; // Used to identify the session in logs

        //@owner
        if(_s != STAT_INIT) _s._streamer /* : ConvivaStreamerProxy */ = null;
        if(_s != STAT_INIT) _s._streamerState /* : StreamerStateManager */ = null;

        if(_s != STAT_INIT) _s._eventQueue /* : EventQueue */ = null;
        if(_s != STAT_INIT) _s._contentInfo /* : ConvivaContentInfo */ = null;
        if(_s != STAT_INIT) _s._nextHeartbeat /* : DictionaryCS(string, object) */ = null;
        if(_s != STAT_INIT) _s._utils /* : Utils */ = null;


        if(_s != STAT_INIT) _s._startTimeMs /* : Number */ = 0;
        if(_s != STAT_INIT) _s._lastStateUpdateTimeMs /* : Number */ = 0;

        // Proxy state
        if(_s != STAT_INIT) _s._playingState /* : int */ = -1; // current state (as int). Change with SetPlayingState
        if(_s != STAT_INIT) _s._streamInfo /* : StreamInfo */ = null; // Current resource and bitrate

        // A map from PlayerStates to cumulative time spent in each state.
        if(_s != STAT_INIT) _s._cumulativeTimePerState /* : DictionaryCS(int, int) */ = null;
        // The time spent in all non-playing states before the first playing
        // state.  Negative iff no playing state has been observed yet.
        // There are two special negative values:
        //  o -1: Not yet joined.
        //  o -2: An error caused this session to fail to join.  (If the
        //        somehow joins anyway, this is ignored.)
        if(_s != STAT_INIT) _s._joinTimeMs /* : int */ = -1;
        if(_s != STAT_INIT) _s._bufferingEventCount /* : int */ = 0;

        
        // Total of all nominal bits played
        if(_s != STAT_INIT) _s._nominalPlayingBitsTotal /* : Number */ = 0;

        if(_s != STAT_INIT) _s._encodedFps /* : int */ = -1;
        if(_s != STAT_INIT) _s._contentLengthSec /* : int */ = -1;

        // Number of times the framerate has been checked while in playing state.
        if(_s != STAT_INIT) _s._playingFpsObservationCount /* : int */ = 0;
        // Total of all playing FPS observations
        if(_s != STAT_INIT) _s._playingFpsTotal /* : Number */ = 0;

        function _constr(streamer /* : Object */, eventQueue /* : EventQueue */, contentInfo /* : ConvivaContentInfo */, sessionId /* : int */) {
            
            _s._eventQueue = eventQueue;
            _s._utils = Utils.getInstance();
            _s._contentInfo = contentInfo;
            _s._sessionId = sessionId;
            _s._nextHeartbeat = new DictionaryCS/*<string, object>*/();

            _s._startTimeMs = 0;
            _s._lastStateUpdateTimeMs = 0;


            _s._cumulativeTimePerState = new DictionaryCS/*<int, int>*/();
            var _for_array_1 =  PlayerStates.stateToInt.KeyValuePairs;
            for(var _for_idx_2=0; _for_idx_2 < _for_array_1.length; _for_idx_2++) {
              var statePair /* : KeyValuePairCS */ = _for_array_1[_for_idx_2];

                _s._cumulativeTimePerState.SetValue(statePair.Value, 0);
            }
            _s._joinTimeMs = -1;
            _s._bufferingEventCount = 0;

            _s._nominalPlayingBitsTotal = 0;


            _s._encodedFps = -1;
            _s._contentLengthSec = -1;
            _s._playingFpsObservationCount = 0;
            _s._playingFpsTotal = 0;


            _s._streamer = Monitor.wrapInConvivaStreamerProxy(streamer, sessionId);
        }

        defPubMeth(_s,"start",__start);
        function __start(nowMs /* : Number */) /* : void */  {
            _s._startTimeMs = nowMs;
            _s._lastStateUpdateTimeMs = nowMs;
            // Get a reference to the state object
            _s._streamerState = _s._streamer.GetStreamerStateManager();
            _s._streamerState.startMonitoring(_s);

            // Read the initial state
            _s._playingState = PlayerStates.eUninitialized;
            _s.SetPlayingState(PlayerStates.stateToInt.GetValue(_s._streamerState.GetPlayingState()));

            _s._streamInfo = _s._streamerState.GetStream();

            var initialBitrateKbps /* : int */ = -1;
            var initialResource /* : String */ = null;
            var initialCdnName /* : String */ = ConvivaContentInfo.CDN_NAME_OTHER;

            if (_s._contentInfo != null) {
                initialCdnName = _s._contentInfo.defaultReportingCdnName;
                if (_s._contentInfo.defaultReportingResource == null ||
                    _s._contentInfo.defaultReportingResource == ConvivaStreamerProxy.NO_RESOURCE) {
                    initialResource = initialCdnName;
                } else {
                    initialResource = _s._contentInfo.defaultReportingResource;
                }
                initialBitrateKbps = _s._contentInfo.defaultReportingBitrateKbps;
            }
            if(_s._streamerState.GetBitrateKbps() > 0) {
                // If the streamer already has a bitrate, use it
                initialBitrateKbps = _s._streamerState.GetBitrateKbps();
            }
            if(_s._streamerState.GetResource() != null) {
                // If the streamer already has a resource, use it
                initialResource = _s._streamerState.GetResource();
            }
            var initialStream /* : StreamInfo */ = new StreamInfo(initialBitrateKbps, initialCdnName, initialResource);
            // This will call the SetStream
            _s._streamerState.SetStream(initialStream);
           
               
            if (_s._streamerState.GetLastMetadata() != null) {
                _s.OnMetadata(_s._streamerState.GetLastMetadata());
            }
            if (_s._streamerState.GetLastError() != null) {
                _s.OnError(_s._streamerState.GetLastError());
            }

        }



        defGet(_s,"streamer",__get_streamer);
        function __get_streamer()  { return  _s._streamer; }



        // For MonitorNotifier
        defPubMeth(_s,"Log",__Log);
        function __Log(message /* : String */) /* : void */  {
            _s._utils.logSession(message, _s._sessionId);
        }

        // For MonitorNotifier
        defPubMeth(_s,"SetPlayingState",__SetPlayingState);
        function __SetPlayingState(newState /* : int */) /* : void */  {
            _s._utils.runProtected(
                function () { 
                    if (_s._playingState == newState) {
                        return;
                    }
                    _s.updateStateCumulativeTime();
                    var hasJoined /* : Boolean */ = _s._joinTimeMs >= 0;
                    if (!hasJoined && newState == PlayerStates.ePlaying) {
                        // We have a join.
                        _s._joinTimeMs = slint.Cast(_s._utils.epochTimeMs() - _s._startTimeMs);
                        // Buffering before the join is not counted as buffering.
                        _s._cumulativeTimePerState.SetValue(PlayerStates.eBuffering, 0);
                    }
                    if (hasJoined && newState == PlayerStates.eBuffering) {
                        _s._bufferingEventCount++;
                    }
                    _s._playingState = newState;
                }, "SetPlayingState");
        }




        // For MonitorNotifier
        defPubMeth(_s,"SetStream",__SetStream);
        function __SetStream(targetStream /* : StreamInfo */) /* : void */  {
            _s._utils.runProtected(
                function () { 
                    if (targetStream != null) {
                        if (targetStream.bitrateKbps > 0) {
                            _s.enqueueBitrateChangeEvent(_s._streamInfo.bitrateKbps, targetStream.bitrateKbps);
                        }
                        _s._streamInfo = targetStream;
                    }
                }, "OnSwitchingStatusChange");
        }


        // For MonitorNotifier
        defPubMeth(_s,"OnError",__OnError);
        function __OnError(e /* : StreamerError */) /* : void */  {
            _s._utils.runProtected(
                function () { 
                    // When an error happens, we switch briefly to ERROR state to deliver the error, then back to the prior state.
                    var currentState /* : int */ = _s._playingState;

                    _s.SetPlayingState(PlayerStates.eError);
                    _s.declareError(e.getErrorCode(), (e.getSeverity() == StreamerError.SEVERITY_FATAL));
                    _s.SetPlayingState(currentState);
                }, "OnError");
        }

        // For MonitorNotifier
        defPubMeth(_s,"OnMetadata",__OnMetadata);
        function __OnMetadata(metadata /* : Object */) /* : void */  {
            _s._utils.runProtected(
                function () { 
                    var metadataDict /* : DictionaryCS(string, string) */ = Lang.DictionaryFromRepr/*string, string*/(metadata);
                    if (metadataDict.ContainsKey("framerate")) {                    
                        _s._encodedFps = _s._utils.parseInt(metadataDict.GetValue("framerate"), -1);
                        _s.Log("Monitor: Received framerate metadata: " + _s._encodedFps);
                    }
                    if (metadataDict.ContainsKey("duration")) {
                        _s._contentLengthSec = _s._utils.parseInt(metadataDict.GetValue("duration"), -1);
                        _s.Log("Monitor: Received duration metadata: " + _s._contentLengthSec);
                    }
                }, "OnMetadata");
        }




        /** Update @heartbeat with current metrics. */
        defPubMeth(_s,"updateHeartbeat",__updateHeartbeat);
        function __updateHeartbeat(heartbeat /* : DictionaryCS(string, object) */) /* : void */  {
            _s.updateMetrics();
            var playingTime /* : int */ = _s._cumulativeTimePerState.GetValue(PlayerStates.ePlaying);
            var hasJoined /* : Boolean */ = (_s._joinTimeMs >= 0);
            var bufferingTime /* : int */ = 0;
            if (hasJoined) {
                bufferingTime = _s._cumulativeTimePerState.GetValue(PlayerStates.eBuffering);
                // Initial buffering time is ignored.
            }
            var averageBitrateKbps /* : int */ = -1;
            if (hasJoined) {
                averageBitrateKbps = slint.Cast((_s._nominalPlayingBitsTotal + 0.0) / playingTime); // bits / ms = kbits / s
            }
            var averageFps /* : int */ = 0;
            if (_s._playingFpsObservationCount > 0) {
                averageFps = slint.Cast((_s._playingFpsTotal + 0.0) / _s._playingFpsObservationCount);
            }
            NativeLang.setDictValue(heartbeat, "ps", _s._playingState);
            NativeLang.setDictValue(heartbeat, "tpt", playingTime);
            NativeLang.setDictValue(heartbeat, "tbt", bufferingTime);
            NativeLang.setDictValue(heartbeat, "tpst", _s._cumulativeTimePerState.GetValue(PlayerStates.ePaused));
            NativeLang.setDictValue(heartbeat, "tst", _s._cumulativeTimePerState.GetValue(PlayerStates.eStopped));
            NativeLang.setDictValue(heartbeat, "jt", _s._joinTimeMs);
            NativeLang.setDictValue(heartbeat, "tbe", _s._bufferingEventCount);
            if (_s._contentLengthSec > 0) {
                NativeLang.setDictValue(heartbeat, "cl", _s._contentLengthSec);
            }
            if (_s._streamInfo.bitrateKbps > 0) {
                NativeLang.setDictValue(heartbeat, "cbr", _s._streamInfo.bitrateKbps);
            }
            if (averageBitrateKbps > 0) {
                NativeLang.setDictValue(heartbeat, "abr", averageBitrateKbps);
            }
            NativeLang.setDictValue(heartbeat, "rs", _s._streamInfo.resource);
            NativeLang.setDictValue(heartbeat, "cdn", _s._streamInfo.cdnName);

            if (_s._encodedFps >= 0) {
                NativeLang.setDictValue(heartbeat, "efps", _s._encodedFps);
            }
            if (averageFps >= 0) {
                NativeLang.setDictValue(heartbeat, "afps", averageFps);
            }
        }

        defPubMeth(_s,"cleanup",__cleanup);
        function __cleanup() /* : void */  {
            if (_s._streamer != null) {
                _s._streamer.Cleanup();
            }
            _s._streamer = null;
            _s._eventQueue = null;
            _s._contentInfo = null;
            _s._nextHeartbeat = null;
        }

        defPrivMeth(_s,"updateMetrics",__updateMetrics);
        function __updateMetrics() /* : void */  {
            if (_s._playingState == PlayerStates.ePlaying) {
                _s._playingFpsTotal += _s._streamer.GetRenderedFrameRate();
                _s._playingFpsObservationCount++;
            }
            _s.updateStateCumulativeTime();
        }




        defPrivMeth(_s,"enqueueEvent",__enqueueEvent);
        function __enqueueEvent(type /* : String */, eventData /* : DictionaryCS(string, object) */) /* : void */  {
            _s._eventQueue.enqueueEvent(type, eventData, slint.Cast(_s._utils.epochTimeMs() - _s._startTimeMs));
        }



        defPrivMeth(_s,"enqueueBitrateChangeEvent",__enqueueBitrateChangeEvent);
        function __enqueueBitrateChangeEvent(oldBitrateKbps /* : int */, newBitrateKbps /* : int */) /* : void */  {
            if (oldBitrateKbps == newBitrateKbps) {
                return;
            }
            _s.updateStateCumulativeTime();
            var bitrateSwitchEvent /* : DictionaryCS(string, object) */ = new DictionaryCS/*<string, object>*/();
            bitrateSwitchEvent.SetValue("pbr", oldBitrateKbps);
            bitrateSwitchEvent.SetValue("nbr", newBitrateKbps);
            _s.enqueueEvent("CwsBitrateChangeEvent", bitrateSwitchEvent);
        }


        defPrivMeth(_s,"updateStateCumulativeTime",__updateStateCumulativeTime);
        function __updateStateCumulativeTime() /* : void */  {
            var now /* : Number */ = _s._utils.epochTimeMs();
            if (_s._playingState != PlayerStates.eUninitialized) {
                var delta /* : int */ = slint.Cast(now - _s._lastStateUpdateTimeMs);
                _s._cumulativeTimePerState.SetValue(_s._playingState, _s._cumulativeTimePerState.GetValue(_s._playingState) + delta);

                if (_s._playingState == PlayerStates.ePlaying && _s._streamInfo.bitrateKbps != -1) {
                    _s._nominalPlayingBitsTotal += (delta * _s._streamInfo.bitrateKbps); // ms * kilobits / s = bits
                }
            }
            _s._lastStateUpdateTimeMs = now;
        }

        defPrivMeth(_s,"declareError",__declareError);
        function __declareError(errorMsg /* : String */, isFatal /* : Boolean */) /* : void */  {
            if (_s._joinTimeMs < 0 && isFatal) {
                // Join failure.
                _s._joinTimeMs = -2;
            }
            var data /* : DictionaryCS(string, object) */ = new DictionaryCS/*<string, object>*/();
            data.SetValue("ft", isFatal);
            data.SetValue("err", errorMsg);
            _s.enqueueEvent("CwsErrorEvent", data);
            _s.SetPlayingState(PlayerStates.eError);
        }

        defStatMeth(_s,Monitor,"wrapInConvivaStreamerProxy",__wrapInConvivaStreamerProxy);
        function __wrapInConvivaStreamerProxy(streamer /* : Object */, sessionId /* : int */) /* : ConvivaStreamerProxy */  {
            // Add wrappers for new streamer types here.  For now we assume we
            // are directly receiving a ConvivaStreamerProxy.
            if(streamer.hasOwnProperty('getStreamerType') && streamer.getStreamerType() == 'Samsung') {
                return SamsungStreamerProxy.createSamsungStreamerProxy(streamer);
            } else if (streamer.networkState !== undefined && streamer.readyState !== undefined) {
                return new Html5ConvivaStreamerProxy(streamer);
            } else {
                return streamer;
            }
        }
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(Monitor,"Monitor");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/monitor/PlayerStates.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-


// namespace monitor
    function PlayerStates() {
        var _s = this;
        if(_s == STAT_INIT) PlayerStates.stateToInt /* : DictionaryCS(string, int) */ = null;

        if(_s == STAT_INIT) PlayerStates.UNINITIALIZED /* : String */ = "UNINITIALIZED";
        if(_s == STAT_INIT) PlayerStates.eUninitialized /* : int */ = 0;
        if(_s == STAT_INIT) PlayerStates.PLAYING /* : String */ = "PLAYING";
        if(_s == STAT_INIT) PlayerStates.ePlaying /* : int */ = 3;
        if(_s == STAT_INIT) PlayerStates.STOPPED /* : String */ = "STOPPED";
        if(_s == STAT_INIT) PlayerStates.eStopped /* : int */ = 1;
        if(_s == STAT_INIT) PlayerStates.PAUSED /* : String */ = "PAUSED";
        if(_s == STAT_INIT) PlayerStates.ePaused /* : int */ = 12;
        if(_s == STAT_INIT) PlayerStates.BUFFERING /* : String */ = "BUFFERING";
        if(_s == STAT_INIT) PlayerStates.eBuffering /* : int */ = 6;
        if(_s == STAT_INIT) PlayerStates.ERROR /* : String */ = "ERROR";
        if(_s == STAT_INIT) PlayerStates.eError /* : int */ = 99;
        if(_s == STAT_INIT) PlayerStates.UNKNOWN /* : String */ = "UNKNOWN";
        if(_s == STAT_INIT) PlayerStates.eUnknown /* : int */ = 100;

        defStatMeth(_s,PlayerStates,"init",__init);
        function __init() /* : void */  {
            PlayerStates.stateToInt = new DictionaryCS/*<string, int>*/();
            PlayerStates.stateToInt.SetValue(PlayerStates.UNINITIALIZED, PlayerStates.eUninitialized);
            PlayerStates.stateToInt.SetValue(PlayerStates.PLAYING, PlayerStates.ePlaying);
            PlayerStates.stateToInt.SetValue(PlayerStates.STOPPED, PlayerStates.eStopped);
            PlayerStates.stateToInt.SetValue(PlayerStates.PAUSED, PlayerStates.ePaused);
            PlayerStates.stateToInt.SetValue(PlayerStates.BUFFERING, PlayerStates.eBuffering);
            PlayerStates.stateToInt.SetValue(PlayerStates.ERROR, PlayerStates.eError);
            PlayerStates.stateToInt.SetValue(PlayerStates.UNKNOWN, PlayerStates.eUnknown);
        }

        defStatMeth(_s,PlayerStates,"cleanup",__cleanup);
        function __cleanup() /* : void */  {
            PlayerStates.stateToInt = null;
        }
       //JSRENAME:constant:_s:PlayerStates:BUFFERING
       //JSRENAME:constant:_s:PlayerStates:ERROR
       //JSRENAME:constant:_s:PlayerStates:PAUSED
       //JSRENAME:constant:_s:PlayerStates:PLAYING
       //JSRENAME:constant:_s:PlayerStates:STOPPED
       //JSRENAME:constant:_s:PlayerStates:UNINITIALIZED
       //JSRENAME:constant:_s:PlayerStates:UNKNOWN
       //JSRENAME:constant:_s:PlayerStates:eBuffering
       //JSRENAME:constant:_s:PlayerStates:eError
       //JSRENAME:constant:_s:PlayerStates:ePaused
       //JSRENAME:constant:_s:PlayerStates:ePlaying
       //JSRENAME:constant:_s:PlayerStates:eStopped
       //JSRENAME:constant:_s:PlayerStates:eUninitialized
       //JSRENAME:constant:_s:PlayerStates:eUnknown
    }
statInit(PlayerStates,"PlayerStates");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/session/EventQueue.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-



// namespace session
    function EventQueue() {
        var _s = this;
        if(_s != STAT_INIT) _s._maxSize /* : int */ = 0;
        //NOTE(henry): For speed, we implement _events as a native data
        // structure rather than a ListCS.
        if(_s != STAT_INIT) _s._events /* : ListCS(object) */ = null;
        if(_s != STAT_INIT) _s._nextSeqNumber /* : int */ = 0;

        function _constr(maxSize /* : int */) {
            _s._maxSize = maxSize;
            _s._events = NativeLang.makeList();
        }

        defPubMeth(_s,"enqueueEvent",__enqueueEvent);
        function __enqueueEvent(type /* : String */, data /* : DictionaryCS(string, object) */, timeSinceSessionStart /* : int */) /* : void */  {
            data.SetValue("t", type);
            data.SetValue("st", timeSinceSessionStart);
            data.SetValue("seq", _s._nextSeqNumber);
            _s._nextSeqNumber++;
            NativeLang.addListValue(_s._events, Lang.StringDictionaryToRepr(data));
            if (_s._events.Count > _s._maxSize) {
                NativeLang.removeListValueAt(_s._events, 0); //TODO(henry): Implement a queue and/or implement in native code.
            }
        }
        
        //NOTE(henry): Not really a DictionaryCS - native dictionary.
        defPubMeth(_s,"flushEvents",__flushEvents);
        function __flushEvents() /* : ListCS(object) */  {
            var currentEvents /* : ListCS(object) */ = _s._events;
            _s._events = NativeLang.makeList();
            return currentEvents;
        }
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(EventQueue,"EventQueue");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/session/Session.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-



// namespace session
    function Session() {
        var _s = this;
        //@owner
        if(_s != STAT_INIT) _s.streamer /* : Object */ = null;

        //@owner
        if(_s != STAT_INIT) _s._contentInfo /* : ConvivaContentInfo */ = null;
        // A copy of tags from _contentInfo, in native dictionary form for
        // faster encoding.
        if(_s != STAT_INIT) _s._nativeReprTags /* : Object */ = null;
        if(_s != STAT_INIT) _s._utils /* : Utils */ = null;
        if(_s != STAT_INIT) _s._settings /* : Settings */ = null;
        //@owner
        if(_s != STAT_INIT) _s._monitor /* : Monitor */ = null; // May be null for a non-video session
        //@owner
        if(_s != STAT_INIT) _s._eventQueue /* : EventQueue */ = null;

        if(_s != STAT_INIT) _s._clientIdWaiter /* : Function */ = null;
        if(_s != STAT_INIT) _s._heartbeatTimer /* : Cleanable */ = null;
        if(_s != STAT_INIT) _s._encodeHeartbeatTimer /* : Cleanable */ = null;

        if(_s != STAT_INIT) _s._sendLogs /* : Boolean */ = false; // Whether we are sending logs 

        if(_s != STAT_INIT) _s._sessionId /* : int */ = 0;
        if(_s != STAT_INIT) _s._sessionFlags /* : int */ = 0;
        if(_s != STAT_INIT) _s._startTimeMs /* : Number */ = 0;
        if(_s != STAT_INIT) _s._heartbeatSequenceNumber /* : int */ = 0;

        function _constr(streamer /* : Object */, contentInfo /* : ConvivaContentInfo */) {
            // The streamer may be null for a non-video session
            _s.streamer = streamer;
            _s._contentInfo = contentInfo;
            _s._utils = Utils.getInstance();
            _s._settings = _s._utils.getSettings();
            _s._sessionId = _s._utils.randInt();
            /* Some sanity checking for tags. The tags are represented as a native dictionary */
            var langTags /* : DictionaryCS(string, string) */ = Lang.DictionaryFromRepr/*string, string*/(_s._contentInfo.tags);
            var keysToCorrect /* : ListCS(string) */ = new ListCS/*string*/(); // Cannot modify the tags while iterating
            var _for_array_1 =  langTags.Keys;
            for(var _for_idx_2=0; _for_idx_2 < _for_array_1.length; _for_idx_2++) {
              var tkey /* : String */ = _for_array_1[_for_idx_2];

                if(langTags.GetValue(tkey) == null) {
                    keysToCorrect.Add(tkey);
                }
            }
            var _for_array_3 =  keysToCorrect.Values;
            for(var _for_idx_4=0; _for_idx_4 < _for_array_3.length; _for_idx_4++) {
              var tkey1 /* : String */ = _for_array_3[_for_idx_4];

                _s.log("WARNING: correcting null value for tag "+tkey1);
                NativeLang.setDictValue/*string, string*/(_s._contentInfo.tags, tkey1, "null");
            }
            _s._nativeReprTags = Lang.StringDictionaryToRepr(_s._contentInfo.tags);
            
            _s._eventQueue = new EventQueue(_s._settings.maxEventsPerHeartbeat);
            if (streamer != null) {
                _s._monitor = new Monitor(streamer, _s._eventQueue, _s._contentInfo, _s._sessionId);
                _s._sessionFlags = _s._monitor.streamer.GetCapabilities();
            } else {
                _s._monitor = null;
                _s._sessionFlags = 0; // Non-video session
            }
        }

        /// <summary>
        /// isVideo : is this a video session ?
        /// </summary>
        /// <returns></returns>
        defPubMeth(_s,"isVideo",__isVideo);
        function __isVideo() /* : Boolean */  {
            return (_s._sessionFlags & ConvivaStreamerProxy.CAPABILITY_VIDEO) != 0;
        }

        defPubMeth(_s,"start",__start);
        function __start() /* : void */  {
            _s.log("Session.start assetName=" + _s._contentInfo.assetName);
            _s._startTimeMs = _s._utils.epochTimeMs();
            if (_s._monitor != null) {
                _s._monitor.start(_s._startTimeMs);
            }
            _s._heartbeatSequenceNumber = 0;

            if (_s._utils.clientIdLoadingDone == null) {
                //we've got a client id, so send out heartbeat immediately
                _s.sendHeartbeat();
            } else {
                //we should always listen to clientIdAvailable in case that
                //the current client id is 0. The listener will be removed
                //in onClientIdAvailable once a valid client id is received
                _s._clientIdWaiter = _s.onClientIdLoaded;
                _s._utils.clientIdLoadingDone.AddHandler(_s._clientIdWaiter);
            }
            _s._heartbeatTimer = null;
            _s.resetHeartbeatTimer();
        }

        defPubMeth(_s,"cleanup",__cleanup);
        function __cleanup() /* : void */  {
            _s.log("Session.cleanup");
            if (_s._heartbeatTimer != null) {
                _s._heartbeatTimer.cleanup();
            }
            _s._heartbeatTimer = null;
            _s._utils.logSession("Schedule the last hb before session cleanup", _s._sessionId);

            var urgentHb /* : Object */ = _s.makeHeartbeat();
            if (urgentHb != null) {
                _s.encodeAndPostHeartbeat(urgentHb,
                    function () { 
                        //cleanup the rest of the session
                        _s.cleanupAll();
                    });
            } else {
                //no hb to send
                _s.cleanupAll();
            }            
        }

        /* Restart the heartbeat timer using the current settings */
        defPrivMeth(_s,"resetHeartbeatTimer",__resetHeartbeatTimer);
        function __resetHeartbeatTimer() /* : void */  {
            if (_s._heartbeatTimer != null) {
                _s._heartbeatTimer.cleanup();
            }
            _s._heartbeatTimer = _s._utils.createTimer(_s.sendHeartbeat, _s._settings.heartbeatIntervalMs, "sendHeartbeat");
        }

        defPrivMeth(_s,"cleanupAll",__cleanupAll);
        function __cleanupAll() /* : void */  {
            if (_s._clientIdWaiter != null) {
                _s._utils.clientIdLoadingDone.DeleteHandler(_s._clientIdWaiter);
                _s._clientIdWaiter = null;
            }
            if (_s._monitor != null) {
                _s._monitor.cleanup();
            }
            if (_s._encodeHeartbeatTimer != null) {
                _s._encodeHeartbeatTimer.cleanup();
                _s._encodeHeartbeatTimer = null;
            }
            _s._monitor = null;
            _s.streamer = null;
        }

        defPrivMeth(_s,"log",__log);
        function __log(message /* : String */) /* : void */  {
            _s._utils.logSession(message, _s._sessionId);
        }

        // The API uses this to update the StreamInfo
        defPubMeth(_s,"setCurrentStreamInfo",__setCurrentStreamInfo);
        function __setCurrentStreamInfo(streamInfo /* : StreamInfo */) /* : void */  {
            _s.log("cws.setCurrentStreamInfo bitrate=" + streamInfo.bitrateKbps + " resource=" + streamInfo.resource);
            if (_s._monitor != null && _s._monitor.streamer != null) {
                var stateManager /* : StreamerStateManager */ = _s._monitor.streamer.GetStreamerStateManager();
                // Push it directly into the StreamStateManager
                _s._monitor.streamer.GetStreamerStateManager().SetStream(streamInfo);
            }
        }

        // The API uses this to update the stream metadata
        defPubMeth(_s,"setCurrentStreamMetadata",__setCurrentStreamMetadata);
        function __setCurrentStreamMetadata(metadata /* : Object */) /* : void */  {
            _s.log("cws.setCurrentStreamMetadata");  
            if (_s._monitor != null && _s._monitor.streamer != null) {
                var stateManager /* : StreamerStateManager */ = _s._monitor.streamer.GetStreamerStateManager();
                // Push it directly into the StreamStateManager
                _s._monitor.streamer.GetStreamerStateManager().OnMetadata(metadata);
            }
        }

        defPubMeth(_s,"sendCustomEvent",__sendCustomEvent);
        function __sendCustomEvent(name /* : String */, eventData /* : DictionaryCS(string, object) */) /* : void */  {
            _s.log("cws.sendEvent " + name);
            eventData.SetValue("name", name);
            _s._eventQueue.enqueueEvent("CwsCustomEvent", eventData, slint.Cast(_s._utils.epochTimeMs() - _s._startTimeMs));
        }

        defPrivMeth(_s,"sendHeartbeat",__sendHeartbeat);
        function __sendHeartbeat() /* : void */  {
            var heartbeat /* : Object */ = null;
            if (_s._encodeHeartbeatTimer != null) {
                _s._encodeHeartbeatTimer.cleanup();
            }
            _s._encodeHeartbeatTimer = _s._utils.scheduleAction(
                function () { 
                    heartbeat = _s.makeHeartbeat();
                    if (heartbeat != null) {
                        _s._encodeHeartbeatTimer = _s._utils.scheduleAction(
                            function () { 
                                _s.encodeAndPostHeartbeat(heartbeat, null);
                                _s._encodeHeartbeatTimer = null;
                            }, 1, "encodeAndSendHeartbeat");
                    }
                }, 1, "makeHeartbeat");
        }

        //NOTE(henry): Not really a DictionaryCS - a native dictionary.  Any
        // structures contained in the heartbeat are also native.  This is the
        // simplest and fastest way to ensure that the Slash data structures
        // are not actually encoded when it is time to convert the heartbeat to
        // JSON.  However, it means that this object cannot be interacted with
        // after its creation, except by the JSON encoder.
        defPrivMeth(_s,"makeHeartbeat",__makeHeartbeat);
        function __makeHeartbeat() /* : Object */  {
            var heartbeat = {};
            NativeLang.setDictValue(heartbeat, "t", "CwsSessionHb");
            NativeLang.setDictValue(heartbeat, "cid", _s._settings.apiKey);
            NativeLang.setDictValue(heartbeat, "clid", _s._utils.clientId);
            NativeLang.setDictValue(heartbeat, "sid", _s._sessionId);
            NativeLang.setDictValue(heartbeat, "seq", _s._heartbeatSequenceNumber);
            NativeLang.setDictValue(heartbeat, "pver", _s._settings.protocolVersion);
            NativeLang.setDictValue(heartbeat, "clv", _s._settings.clientVersion);
            NativeLang.setDictValue(heartbeat, "iid", _s._settings.clientInstanceId);
            NativeLang.setDictValue(heartbeat, "sf", _s._sessionFlags);
            var platformMetadata /* : Object */ = Lang.StringDictionaryToRepr/*string*/(_s._settings.platformApi.getPlatformMetadata());
            if (platformMetadata != null) {
                NativeLang.setDictValue(heartbeat, "pm", platformMetadata);
            }
            
            if (_s._utils.device != null) {
                NativeLang.setDictValue(heartbeat, "dv", _s._utils.device);
            }
            if (_s._utils.deviceVersion != null) {
                NativeLang.setDictValue(heartbeat, "dvv", _s._utils.deviceVersion);
            }
            if(_s._utils.deviceType != null) {
                NativeLang.setDictValue(heartbeat, "dvt", _s._utils.deviceType);
            } else if(_s._contentInfo.deviceType != null) {
                NativeLang.setDictValue(heartbeat, "dvt", _s._contentInfo.deviceType);
            }
            NativeLang.setDictValue(heartbeat, "tags", _s._nativeReprTags);
            if (_s._utils.os != null) {
                NativeLang.setDictValue(heartbeat, "os", _s._utils.os);
            }
            if (_s._utils.osVersion != null) {
                NativeLang.setDictValue(heartbeat, "osv", _s._utils.osVersion);
            }
            if (_s._utils.platform != null) {
                NativeLang.setDictValue(heartbeat, "pt", _s._utils.platform);
            }
            if (_s._contentInfo.viewerId != null) {
                NativeLang.setDictValue(heartbeat, "vid", _s._contentInfo.viewerId);
            }
            if (_s._contentInfo.streamUrl != null) {
                NativeLang.setDictValue(heartbeat, "url", _s._contentInfo.streamUrl);
            }
            if (_s._contentInfo.playerName != null) {
                NativeLang.setDictValue(heartbeat, "pn", _s._contentInfo.playerName);
            }
            NativeLang.setDictValue(heartbeat, "lv", _s._contentInfo.isLive);

            // Call the monitor.updateHeartbeat before we package the events
            if (_s._monitor != null) {
                NativeLang.setDictValue(heartbeat, "an", _s._contentInfo.assetName);
                _s._monitor.updateHeartbeat(heartbeat);
            }
            var currentEvents /* : ListCS(object) */ = _s._eventQueue.flushEvents();
            NativeLang.setDictValue(heartbeat, "evs", currentEvents);
            // If this is a non-video session, with no events, suppress this HB
            // Do this check before we consume the logs, or increase the heartbeatSequenceNumber
            if (_s._monitor == null && ! _s.isVideo() && NativeLang.listCount(currentEvents) == 0) {
                return null;
            }

            if (_s._sendLogs) {
                NativeLang.setDictValue(heartbeat, "lg", Lang.ListToRepr/*string*/(_s._utils.getLogs(_s._sessionId)));
            }
            // Put the timestamps last
            NativeLang.setDictValue(heartbeat, "st", slint.Cast(_s._utils.epochTimeMs() - _s._startTimeMs));
            NativeLang.setDictValue(heartbeat, "cts", (_s._utils.epochTimeMs() / 1000.0));
            _s._heartbeatSequenceNumber++;
            return heartbeat;
        }

        /* Encode and POST a heartbeat asynchronously. 
         * afterPostCbk - callback to call after encoding and posting
         */ 
        defPrivMeth(_s,"encodeAndPostHeartbeat",__encodeAndPostHeartbeat);
        function __encodeAndPostHeartbeat(heartbeat /* : Object */, afterPostCbk /* : Function */) /* : void */  {
            _s._utils.jsonEncode(heartbeat, 
                              function (jsonHeartbeat /* : String */) { 
                                  var url /* : String */ = _s._settings.gatewayUrl + _s._settings.gatewayPath;
                                  var contentType /* : String */ = "application/json";
                                  _s.log("Send HB[" + (_s._heartbeatSequenceNumber - 1) + "]");
                                  _s._utils.httpRequest(true, url, jsonHeartbeat, contentType, _s.onHeartbeatResponse);
                                  if (afterPostCbk != null) {
                                      afterPostCbk();
                                  }
                              });
        }



        defPrivMeth(_s,"onHeartbeatResponse",__onHeartbeatResponse);
        function __onHeartbeatResponse(isSuccess /* : Boolean */, jsonResponse /* : String */) /* : void */  {
            _s._utils.runProtected(
                function () { 
                    if (isSuccess) {
                        var decodedResponse /* : DictionaryCS(string, object) */ = _s._utils.jsonDecode(jsonResponse);
                        if (decodedResponse.ContainsKey("clid")) {
                            _s._utils.setClientIdFromServer(Lang.ToString((decodedResponse.GetValue("clid"))), false);
                        }
                        var newSendLogs /* : Boolean */ = (decodedResponse.ContainsKey("slg") && Boolean(decodedResponse.GetValue("slg")));
                        if (newSendLogs != _s._sendLogs) {
                            _s.log("Turning " + (newSendLogs ? "on" : "off") +" sending of logs");
                            _s._sendLogs = newSendLogs;
                        }
                        if (decodedResponse.ContainsKey("hbi")) {
                            var heartbeatIntervalSec /* : int */ = slint.Cast(decodedResponse.GetValue("hbi"));
                            if(1000 * heartbeatIntervalSec !=  _s._settings.heartbeatIntervalMs) {
                                 _s.log("Received hbInterval from server " + heartbeatIntervalSec);
                                 _s._settings.heartbeatIntervalMs = 1000 * heartbeatIntervalSec;
                                 if (_s._heartbeatTimer != null) {
                                      _s.resetHeartbeatTimer();
                                 }
                            }
                        }
                        if (decodedResponse.ContainsKey("gw")) {
                            var gatewayUrl /* : String */ =  (decodedResponse.GetValue("gw"));
                            if(gatewayUrl != _s._settings.gatewayUrl) {
                                _s.log("Received gatewayUrl from server "+gatewayUrl);
                                _s._settings.gatewayUrl = gatewayUrl;
                            }
                        }
                    } else {
                        _s.log("Received no response (or a bad response) to a POST request.");
                    }
                }, "onHeartbeatResponse");
        }

        defPrivMeth(_s,"onClientIdLoaded",__onClientIdLoaded);
        function __onClientIdLoaded() /* : void */  {
            // This is fired on loading of the clientId from the local storage 
            _s._utils.clientIdLoadingDone.DeleteHandler(_s._clientIdWaiter);
            _s._clientIdWaiter = null;
            _s.sendHeartbeat();
        }

        defPubMeth(_s,"initialResourceBitrateSelection",__initialResourceBitrateSelection);
        function __initialResourceBitrateSelection() /* : void */  {
            var proxy /* : ConvivaStreamerProxy */ = _s._monitor.streamer;
            //TODO: get a list of candidate resources and bitrates
            var candidateStreams /* : ListCS(StreamInfo) */ = new ListCS/*StreamInfo*/();
            proxy.StartPlay(LivePass.STREAM_SELECTION_SUCC, Lang.ListToRepr/*StreamInfo*/(candidateStreams), ConvivaStreamerProxy.BOOLEAN_FALSE);
        }

        defPubMeth(_s,"midStreamResourceBitrateSelection",__midStreamResourceBitrateSelection);
        function __midStreamResourceBitrateSelection(switchTriggers /* : String */) /* : void */  {
            //TODO: get the target stream
            var candidateStreams /* : ListCS(StreamInfo) */ = new ListCS/*StreamInfo*/();
            var proxy /* : ConvivaStreamerProxy */ = _s._monitor.streamer;
            proxy.DoSwitch(LivePass.STREAM_SELECTION_SUCC, Lang.ListToRepr/*StreamInfo*/(candidateStreams), ConvivaStreamerProxy.BOOLEAN_FALSE);
        }
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(Session,"Session");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/session/SessionFactory.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-



// namespace session
    /** A factory for Session objects. */
    function SessionFactory() {
        var _s = this;
        if(_s != STAT_INIT) _s.lastSessionId /* : int */ = 0;

        if(_s != STAT_INIT) _s._utils /* : Utils */ = null;
        if(_s != STAT_INIT) _s._settings /* : Settings */ = null;
        if(_s != STAT_INIT) _s._nextSessionId /* : int */ = 0; // Should be positive
        if(_s != STAT_INIT) _s._sessionsById /* : DictionaryCS(int, Session) */ = null;

        function _constr() {
            _s._utils = Utils.getInstance ();
            _s._settings = _s._utils.getSettings();
            _s._nextSessionId = 0;
            _s._sessionsById = new DictionaryCS/*<int, Session>*/();
            _s.lastSessionId = 0;
        }

        defPubMeth(_s,"cleanup",__cleanup);
        function __cleanup() /* : void */  {
            _s._utils = null;
            _s._settings = null;
            if (_s._sessionsById != null) {
                var _for_array_1 =  _s._sessionsById.KeyValuePairs;
                for(var _for_idx_2=0; _for_idx_2 < _for_array_1.length; _for_idx_2++) {
                  var sessionPair /* : KeyValuePairCS */ = _for_array_1[_for_idx_2];

                    var sess /* : Session */ = sessionPair.Value;
                    sess.cleanup();
                }
            }
            _s._sessionsById = null;
            _s._nextSessionId = 0;
        }

        // Generate a new session id
        defPubMeth(_s,"newSessionId",__newSessionId);
        function __newSessionId() /* : int */  {
            var sessionId /* : int */ = _s._nextSessionId;
            _s._nextSessionId++;
            return sessionId;
        }

        defPubMeth(_s,"makeSession",__makeSession);
        function __makeSession(streamer /* : Object */, contentInfo /* : ConvivaContentInfo */, sessionId /* : int */) /* : Session */  {
            
            var session /* : Session */ = new Session(streamer, contentInfo);
            _s._sessionsById.SetValue(sessionId, session);
            _s.lastSessionId = sessionId;
            session.start();
            return session;
        }


        defPubMeth(_s,"getSession",__getSession);
        function __getSession(sessionId /* : int */) /* : Session */  {
            return _s._sessionsById.GetValue(sessionId);
        }

        defPubMeth(_s,"cleanupSession",__cleanupSession);
        function __cleanupSession(sessionId /* : int */) /* : void */  {
            var session /* : Session */ = _s._sessionsById.GetValue(sessionId);
            _s._sessionsById.Remove(sessionId);
            session.cleanup();
        }
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(SessionFactory,"SessionFactory");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/utils/Cleanable.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2011, 2012 Conviva Inc. All Rights Reserved.
// Author: George Necula (necula@conviva.com), Henry Milner (henry@conviva.com), Yan Li (yan@conviva.com)



// namespace utils
    /// <summary>
    /// A class to manage cleanable objects. A Cleanable object is a wrapper for a cleanup Action, 
    /// along with management of a collection of Cleanables. 
    /// </summary>
    function Cleanable() {
        var _s = this;
            
        // The actual cleanup action
        if(_s != STAT_INIT) _s._cleanupAction /* : Function */ = null;

        // Each Cleanable is stored in a collection, indexed by its unique id
        if(_s == STAT_INIT) Cleanable._nextCleanupId /* : int */ = 0;
        if(_s != STAT_INIT) _s._id /* : int */ = 0;
        if(_s != STAT_INIT) _s._cleanupCollection /* : DictionaryCS(int, Cleanable) */ = null;

        // Construct a cleanable object, and add it to a collection of Cleanables
        function _constr(cleanupCollection /* : DictionaryCS(int, Cleanable) */) {
            _s._cleanupCollection = cleanupCollection;
            _s._id = Cleanable._nextCleanupId;
            Cleanable._nextCleanupId++;
            _s._cleanupCollection.SetValue(_s._id, _s);
        }

        defPubMeth(_s,"setCleanupAction",__setCleanupAction);
        function __setCleanupAction(cleanupAction /* : Function */) /* : void */  {
            _s._cleanupAction = cleanupAction;
        }


        /// <summary>
        /// Cleanup the object
        /// </summary>
        defPubMeth(_s,"cleanup",__cleanup);
        function __cleanup() /* : void */  {
            if (_s._cleanupAction != null) {
                _s._cleanupAction();
            }
            _s._cleanupCollection.Remove(_s._id);
        }



        /// <summary>
        /// Cleanup an entire collection
        /// </summary>
        /// <param name="collection"></param>
        defStatMeth(_s,Cleanable,"cleanupCollection",__cleanupCollection);
        function __cleanupCollection(collection /* : DictionaryCS(int,Cleanable) */) /* : void */  {
            // Copy first the cleanable objects
            var toClean /* : ListCS(Cleanable) */ = new ListCS/*Cleanable*/ ();
            var _for_array_1 =  collection.Values;
            for(var _for_idx_2=0; _for_idx_2 < _for_array_1.length; _for_idx_2++) {
              var o /* : Cleanable */ = _for_array_1[_for_idx_2];

                toClean.Add(o);
            }
            var _for_array_3 =  toClean.Values;
            for(var _for_idx_4=0; _for_idx_4 < _for_array_3.length; _for_idx_4++) {
              var o /* : Cleanable */ = _for_array_3[_for_idx_4];

                o.cleanup ();
            }
            collection.Clear();
        }

        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(Cleanable,"Cleanable");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/utils/Eventer.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright (c) 2009, Conviva Inc. All Rights Reserved.
// Author: George Necula (necula@conviva.com)

// namespace utils
    // The type of an event dispatcher with event handlers 
    function Eventer() {
        var _s = this;
        if(_s != STAT_INIT) _s._handlers /* : ListCS(Action) */ = undefined;

        function _constr() {
            _s._handlers = new ListCS/*Action*/();
        }

        /// <summary>
        /// Deletes all event handlers
        /// </summary>
        defPubMeth(_s,"Cleanup",__Cleanup);
        function __Cleanup() /* : void */  {
            _s._handlers = new ListCS/*Action*/();
        }

        defPubMeth(_s,"AddHandler",__AddHandler);
        function __AddHandler(handler /* : Function */) /* : void */  {
            _s._handlers.Add(handler);
        }

        defPubMeth(_s,"DeleteHandler",__DeleteHandler);
        function __DeleteHandler(handler /* : Function */) /* : void */  {
            _s._handlers.Remove(handler);
        }

        defPubMeth(_s,"DispatchEvent",__DispatchEvent);
        function __DispatchEvent() /* : void */  {
            var _for_array_1 =  _s._handlers.Values;
            for(var _for_idx_2=0; _for_idx_2 < _for_array_1.length; _for_idx_2++) {
              var h /* : Function */ = _for_array_1[_for_idx_2];

                h();
            }
        }
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(Eventer,"Eventer");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/utils/Measurement.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// namespace utils
    function Measurement() {
        var _s = this;
        if(_s != STAT_INIT) _s.name /* : String */ = null;
        if(_s != STAT_INIT) _s.startTimeMs /* : Number */ = 0;

        function _constr(name /* : String */, startTimeMs /* : Number */) {
            _s.name = name;
            _s.startTimeMs = startTimeMs;
        }
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(Measurement,"Measurement");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/utils/NativeLang.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-


// namespace utils
    /**
     * Utilities for interfacing with native language objects.  This should be
     * used only when speed is critical and Slash data structures are too slow.
     */
    function NativeLang() {
        var _s = this;

        //NOTE(henry): We should have Slash find these calls and translate them
        // directly to native dictionary accesses.  This would save a method
        // call.

        defStatMeth(_s,NativeLang,"setDictValue",__setDictValue);
        function __setDictValue(dict /* : Object */, key /* : Object */, value /* : Object */) /* : void */  {
            dict[key] = value;
        }

        defStatMeth(_s,NativeLang,"makeList",__makeList);
        function __makeList() /* : Object */  {
            return [];
        }

        defStatMeth(_s,NativeLang,"listCount",__listCount);
        function __listCount(lst /* : Object */) /* : int */  {
            return lst.length;
        }

        defStatMeth(_s,NativeLang,"addListValue",__addListValue);
        function __addListValue(list /* : Object */, value /* : Object */) /* : void */  {
            list.push(value);
        }
        
        defStatMeth(_s,NativeLang,"removeListValueAt",__removeListValueAt);
        function __removeListValueAt(list /* : Object */, idx /* : int */) /* : void */  {
            list.splice(idx, 1);
        }
    }
statInit(NativeLang,"NativeLang");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/utils/Settings.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-


// namespace utils
    function Settings() {
        var _s = this;
        // The interval between heartbeats.
        if(_s != STAT_INIT) _s.heartbeatIntervalMs /* : int */ = 20000;
        // The maximum number of events per heartbeat per session.  Some events
        // will be dropped if more than this many are enqueued.
        if(_s != STAT_INIT) _s.maxEventsPerHeartbeat /* : int */ = 10;
        // The API key (corresponding to a customer name) used for heartbeats.
        if(_s != STAT_INIT) _s.apiKey /* : String */ = null;

        if(_s != STAT_INIT) _s.gatewayUrl /* : String */ = "https://cws.conviva.com";
        if(_s != STAT_INIT) _s.gatewayPath /* : String */ = "/0/wsg";
                
        if(_s != STAT_INIT) _s.protocolVersion /* : String */ = "1.5";

        // The next line will be modified by set_versions.pl
        if(_s != STAT_INIT) _s.clientVersion /* : String */ = "2.67.0.71691";

        // The next field will be changed by LivePass upon "init" 
        if(_s != STAT_INIT) _s.clientInstanceId /* : int */ = 0;

        // The maximum length of time we wait when loading from local storage.
        if(_s != STAT_INIT) _s.loadDataTimeoutMs /* : int */ = 10000;
        
        //  Enable logging to device console
        if(_s != STAT_INIT) _s.enableLogging /* : Boolean */ = false;

        if(_s != STAT_INIT) _s.enablePerformanceMeasurements /* : Boolean */ = false;

        // An instance of PlatformApi with methods for access to low-level services
        if(_s != STAT_INIT) _s.platformApi /* : PlatformApi */ = null;

        if(_s != STAT_INIT) _s.pingComponentName /* : String */ = "jscws";
        if(_s != STAT_INIT) _s.pingUrl /* : String */ = "https://livepass.conviva.com/ping.ping";


        // Change the settings based on the dictionary of keys
        defPubMeth(_s,"changeSettings",__changeSettings);
        function __changeSettings(settings /* : Object */) /* : void */  {
            if (settings == null) return;
            var sobj /* : DictionaryCS(string, object) */ = Lang.DictionaryFromRepr/*string, object*/(settings);
            var _for_array_1 =  sobj.Keys;
            for(var _for_idx_2=0; _for_idx_2 < _for_array_1.length; _for_idx_2++) {
              var key /* : String */ = _for_array_1[_for_idx_2];

                var v /* : Object */ = sobj.GetValue(key);
                switch (key) {
                    case "platformApi":
                        _s.platformApi = (v);
                        break;
                    case "gatewayUrl":
                        _s.gatewayUrl = (sobj.GetValue(key));
                        break;
                    case "heartbeatIntervalMs":
                        _s.heartbeatIntervalMs = slint.Cast(sobj.GetValue(key));
                        break;
                    case "maxEventPerHeartbeat":
                        _s.maxEventsPerHeartbeat = slint.Cast(sobj.GetValue(key));
                        break;
                    case "enableLogging":
                        _s.enableLogging = Boolean(sobj.GetValue(key));
                        break;
                    default:
                        throw new Error("Unsupported settings: " + key);
                }
            }
        }
    }
statInit(Settings,"Settings");

// DO NOT EDIT! THIS FILE WAS GENERATED BY Slash from ../../csClient/CwsClient/utils/Utils.cs
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-






// namespace utils
    function Utils() {
        var _s = this;

        // The unique instance of the Utils module
        if(_s == STAT_INIT) Utils._instance /* : Utils */ = null;
        if(_s != STAT_INIT) _s._referenceCount /* : int */ = 0; // We refcount the Utils instance

        // The dependencies of the unique instance of Utils
        if(_s != STAT_INIT) _s._settings /* : Settings */ = null;

        if(_s == STAT_INIT) Utils.DEFAULT_CLIENT_ID /* : String */ = "0"; // This is the default client id if we do not have one

        /* The client ID goes through the following states:              
         * - loading from disk:
         *      clientId == DEFAULT_CLIENT_ID
         *      clientIdLoadingDone != null (Eventer fires when loading from storage, or loading timeout)
         *      _loadClientIdTimeout != null
         *      
         * - loaded from disk:
         *      clientId != DEFAULT_CLIENT_ID
         *      clientIdLoadingDone == null
         *      _loadClientIdTimeout == null
         *      
         * - loading from disk timed out:
         *      clientId == DEFAULT_CLIENT_ID
         *      clientIdLoadingDone == null
         *      _loadClientIdTimeout == null
         *      
         * - set from server response
         *      clientID != DEFAULT_CLIENT_ID
         *      written to local storage (best effort)
         *      clientIdLoadingDone == null
         *      _loadClientIdTimeout == null
         *      
         */
        if(_s != STAT_INIT) _s._loadClientIdTimeout /* : Cleanable */ = null;
        if(_s != STAT_INIT) _s.clientId /* : String */ = Utils.DEFAULT_CLIENT_ID;
        if(_s != STAT_INIT) _s.clientIdLoadingDone /* : Eventer */ = null; // To signal when we are done loading the client id

        //@owner
        if(_s != STAT_INIT) _s._pingUrl /* : String */ = null;
        if(_s != STAT_INIT) _s._isSendingPing /* : Boolean */ = false;
        if(_s != STAT_INIT) _s._measurementStack /* : ListCS(Measurement) */ = null;

        if(_s != STAT_INIT) _s.protocolVersion /* : String */ = null;
        if(_s != STAT_INIT) _s.device /* : String */ = null;
        if(_s != STAT_INIT) _s.os /* : String */ = null;
        if(_s != STAT_INIT) _s.osVersion /* : String */ = null;
        if(_s != STAT_INIT) _s.deviceVersion /* : String */ = null;
        if(_s != STAT_INIT) _s.deviceType /* : String */ = null;
        if(_s != STAT_INIT) _s.platform /* : String */ = null;

        // We keep a list of Cleanable, indexed by some id
        if(_s != STAT_INIT) _s._cleanables /* : DictionaryCS(int, Cleanable) */ = null;
        

        /* We accumulate a circular list of logs in the log buffer */
        if(_s == STAT_INIT) Utils.MAX_SIZE_LOGBUFFER /* : int */ = 32;
        if(_s == STAT_INIT) Utils.logBuffer /* : ListCS(string) */ = new ListCS/*string*/();

        

        function _constr(settings /* : Settings */) {
            _s._settings = settings;
            _s._referenceCount = 1;
            _s._pingUrl = null;
            _s._isSendingPing = false;

            if (_s._settings.enablePerformanceMeasurements) {
                _s._measurementStack = new ListCS/*Measurement*/();
            }
            _s.protocolVersion = _s._settings.protocolVersion;

            _s._cleanables = new DictionaryCS/*<int, Cleanable>*/();

            // See if we need to create the _settings object
            if (_s._settings.platformApi == null) {
            _s._settings.platformApi = new Html5PlatformApi();
            }
            if (_s._settings.platformApi == null) {
                throw new Error("PlatformApi is null");
            }

            _s.device = _s._settings.platformApi.getDevice();
            _s.deviceVersion = _s._settings.platformApi.getDeviceVersion();
            _s.deviceType = _s._settings.platformApi.getDeviceType();
            _s.os = _s._settings.platformApi.getOS();
            _s.osVersion = _s._settings.platformApi.getOSVersion();

            _s.platform = "Js";

        }

        /// <summary>
        /// Call this function once to initialize the Utils module. 
        /// For every instance of CreateUtils, there must be a call to cleanup()
        /// </summary>
        /// <param name="platformApi"></param>
        /// <param name="settings"></param>
        defStatMeth(_s,Utils,"CreateUtils",__CreateUtils);
        function __CreateUtils(settings /* : Object */) /* : Utils */  {
            if (Utils._instance == null) {
                var s /* : Settings */ = new Settings();
                s.changeSettings(settings);
                Utils._instance = new Utils(s);
                Utils._instance._start();
            } else {
                Utils._instance.getSettings().changeSettings(settings);
                Utils._instance._referenceCount ++;
            }
            return Utils._instance;
        }


        /// <summary>
        /// Call this function to get the singleton instance of the Utils module
        /// </summary>
        /// <returns></returns>
        defStatMeth(_s,Utils,"getInstance",__getInstance);
        function __getInstance() /* : Utils */  {
            if (Utils._instance == null) {
                throw new Error("CreateUtils module has not been called");
            }
            return Utils._instance;
        }

        defPubMeth(_s,"getPlatformApi",__getPlatformApi);
        function __getPlatformApi() /* : PlatformApi */  {
            return _s._settings.platformApi;
        }

        defPubMeth(_s,"getSettings",__getSettings);
        function __getSettings() /* : Settings */  {
            return _s._settings;
        }

        defPrivMeth(_s,"_start",___start);
        function ___start() /* : void */  {
            // Called only once to initialize the Utils module
        }


        defPubMeth(_s,"cleanup",__cleanup);
        function __cleanup() /* : void */  {
            _s._referenceCount--;
            if (_s._referenceCount > 0) {
                return;
            }
            _s._settings = null;
            
            if (_s.clientIdLoadingDone != null) {
                _s.clientIdLoadingDone.Cleanup();
                _s.clientIdLoadingDone = null;
            }
            if (_s._cleanables != null) {
                Cleanable.cleanupCollection(_s._cleanables);
                _s._cleanables = null;
            }
        }

        /***
         * Record (or show) a log message. The log message can optionally start with "ERROR: " to signal an error.
         * The log message can also contain "sid=sessionId" when the log message can be associated with the given session.
         */
        defPubMeth(_s,"log",__log);
        function __log(msg /* : String */) /* : void */  {
            var timeMsec /* : Number */ = _s.epochTimeMs();
            var theTime /* : String */ = undefined;
            theTime = (timeMsec / 1000.0).toFixed(3).toString();

            msg = "[" + theTime + "] " + msg;

            if (Utils.logBuffer.Count >= Utils.MAX_SIZE_LOGBUFFER) {
                Utils.logBuffer.RemoveAt(0);
            }
            // We add to the buffer, independent of whether enableLogging is set
            Utils.logBuffer.Add(msg);

            var isError /* : Boolean */ = Lang.StringContains(msg, "ERROR:");

            if (_s._settings.enableLogging) {
                if(isError) {
                    _s._settings.platformApi.consoleErr(msg);
                } else {
                    _s._settings.platformApi.consoleLog(msg);
                }
            }
        }

        defPubMeth(_s,"logSession",__logSession);
        function __logSession(msg /* : String */, sessionId /* : int */) /* : void */  {
            _s.log("sid=" + sessionId + " " + msg);
        }

        defPubMeth(_s,"err",__err);
        function __err(message /* : String */) /* : void */  {
            _s.log("ERROR: " + message);
        }


        /** 
         * Retrieve the log messages for the given session 
         */ 
        defPubMeth(_s,"getLogs",__getLogs);
        function __getLogs(sessionId /* : int */) /* : ListCS(string) */  {
            // Send all logs in the same session
            var res /* : ListCS(string) */ = Utils.logBuffer;
            Utils.logBuffer = new ListCS/*string*/();
            return res;
        }
           


        defPubMeth(_s,"assert",__assert);
        function __assert(condition /* : Boolean */, msg /* : String */) /* : void */  {
            if (!condition) {
                throw new Error("Assertion failure: " + msg);
            }
        }




        /// <summary>
        /// Run a function with try/catch protection
        /// </summary>
        /// <param name="func"></param>
        /// <param name="message"></param>
        defPubMeth(_s,"runProtected",__runProtected);
        function __runProtected(func /* : Function */, message /* : String */) /* : void */  {
            _s.runProtectedSync(func, message);
        }

        /// <summary>
        /// Run a function with try/catch protection. This function is guaranteed to be synchronous
        /// </summary>
        /// <param name="func"></param>
        /// <param name="message"></param>
        defPubMeth(_s,"runProtectedSync",__runProtectedSync);
        function __runProtectedSync(func /* : Function */, message /* : String */) /* : void */  {
            var allowUncaughtExceptions /* : Boolean */ = _s._settings.platformApi.allowUncaughtExceptions();

            var perfMeasurements /* : Boolean */ = _s._settings.enablePerformanceMeasurements;
            if (perfMeasurements) {
                _s.pushMeasurement(message);
            }
            if (allowUncaughtExceptions) {
                func();
            } else {
                try {
                    func();
                } catch(e /* : Error */) {
                    _s.onUncaughtException(message, e);
                }
            }
                
            if (perfMeasurements) {
                _s.popMeasurement();
            }
        }


        defPrivMeth(_s,"onUncaughtException",__onUncaughtException);
        function __onUncaughtException(msg /* : String */, e /* : Error */) /* : void */  {
            try {
                _s.ping("Uncaught exception: " + Lang.ToString(e));
            } catch(eping /* : Error */) {
                _s.err("Caught exception while sending ping: " + Lang.ToString(eping));
            } 
        }

        defPubMeth(_s,"ping",__ping);
        function __ping(msg /* : String */) /* : void */  {
            if (_s._isSendingPing) {
                return;
            }
            _s._isSendingPing = true;
            _s.initPing();
            var pingUrl /* : String */ = _s._pingUrl + "&d=" + _s.urlEncodeString(Lang.ToString(msg));
            _s.err("Ping: " + pingUrl);
            _s.httpRequest(false, pingUrl, null, null, null);
            _s._isSendingPing = false;

        }

        defPrivMeth(_s,"initPing",__initPing);
        function __initPing() /* : void */  {
            if (_s._pingUrl == null) {
                // Prepare the ping URL.
                var pingId /* : String */ = null;
                if (_s.clientId != null) {
                    // Find the first int in clientId, which is period-delimited.
                    var firstPeriod /* : int */ = Lang.StringIndexOf(_s.clientId, ".");
                    if (firstPeriod >= 0) {
                        pingId = Lang.StringSubstring(_s.clientId, 0, firstPeriod);
                    }
                }
                if (pingId == null) {
                    // No client ID, or it was malformed.
                    pingId = Lang.ToString((_s.randInt())); //FIXME: Make sure Slash handles this appropriately.
                }
                //FIXME(henry): Move to Settings?
                var componentName /* : String */ = "jscws";
                _s._pingUrl = _s._settings.pingUrl + "?" + "uuid=" + Lang.ToString(pingId) + "&comp=" + componentName;
            }
        }

        defPrivMeth(_s,"urlEncodeString",__urlEncodeString);
        function __urlEncodeString(rawString /* : String */) /* : String */  {
            return escape(rawString);
        }

        /// <summary>
        /// Return the current time in milliseconds since Unix epoch 
        /// </summary>
        /// <returns></returns>
        defPubMeth(_s,"epochTimeMs",__epochTimeMs);
        function __epochTimeMs() /* : Number */  {
            return _s._settings.platformApi.epochTimeMs();
        }

        /// <summary>
        /// Create a periodic timer
        /// </summary>
        /// <param name="timerAction"></param>
        /// <param name="intervalMs"></param>
        /// <param name="actionName"></param>
        /// <returns></returns>
        defPubMeth(_s,"createTimer",__createTimer);
        function __createTimer(timerAction /* : Function */, intervalMs /* : int */, actionName /* : String */) /* : Cleanable */  {
            var cleanable /* : Cleanable */ = new Cleanable(_s._cleanables);
            cleanable.setCleanupAction(_s._settings.platformApi.createTimer(timerAction, intervalMs, actionName));
            return cleanable;
        }

        /// <summary>
        /// Schedule an action after an interval
        /// </summary>
        /// <param name="action"></param>
        /// <param name="delayMs"></param>
        /// <param name="actionName"></param>
        /// <returns></returns>
        defPubMeth(_s,"scheduleAction",__scheduleAction);
        function __scheduleAction(action /* : Function */, delayMs /* : int */, actionName /* : String */) /* : Cleanable */  {
            var cleanable /* : Cleanable */ = new Cleanable(_s._cleanables);
            var actionHappened /* : Boolean */ = false;
            var wrappedAction /* : Function */ = null;
            wrappedAction = (
                 function () { 
                     if (cleanable != null) {
                         cleanable.cleanup();
                     }
                     action();
                     actionHappened = true;
                 });
            cleanable.setCleanupAction(_s._settings.platformApi.createTimer(wrappedAction, delayMs, actionName));
            // This is necessary because makeTimer() might have already caused
            // wrappedAction() to be called (e.g. it might call the action
            // synchronously if delayMs=0).  In that case, theTimer was null
            // when wrappedAction was called, so theTimer couldn't be cleaned
            // up.
            if (actionHappened) {
                cleanable.cleanup();
            }
            return cleanable;
        }


        defPubMeth(_s,"parseInt",__parseInt);
        function __parseInt(decimalInt /* : String */, defaultResult /* : int */) /* : int */  {
            var result /* : int */ = defaultResult;
            try {
                result = slint.Parse(decimalInt);
            } catch(e /* : Error */) {
            }
            return result;
        }

        defPubMeth(_s,"startFetchClientId",__startFetchClientId);
        function __startFetchClientId() /* : void */  {
            //  Called on LivePass.init
            _s.clientId = Utils.DEFAULT_CLIENT_ID;
            _s.clientIdLoadingDone = new Eventer(); // This means we do not yet have a clientId

            _s._loadClientIdTimeout = _s.scheduleAction(
                 function () { 
                     _s.log("Timeout in reading clientId. Using " + Utils.DEFAULT_CLIENT_ID + ".");
                     _s.ping("Timeout in reading clientId. IGNORE waited " + _s._settings.loadDataTimeoutMs + "ms.");
                     if (_s.clientIdLoadingDone != null) {
                         _s.clientIdLoadingDone.DispatchEvent();
                     }
                     _s._loadClientIdTimeout = null;
                 },
                 _s._settings.loadDataTimeoutMs,
                 "utils.readClientId timeout callback");

            var onLoad /* : Function(Action(bool, string)) */ = undefined;
            onLoad = (
                function (fSuccess /* : Boolean */, loadedData /* : String */) { 
                    _s.runProtected(
                        function () { 
                            if (_s._loadClientIdTimeout != null) {
                                _s._loadClientIdTimeout.cleanup();
                                _s._loadClientIdTimeout = null;
                            }
                            var result /* : DictionaryCS(string, object) */ = undefined;
                            try {
                                result = _s.jsonDecode(loadedData);
                            } catch(e /* : Error */) {
                                result = null;
                            }
                            var loadedClientId /* : String */ = null;
                            if(fSuccess && result != null && result.ContainsKey("clId")) {
                                    loadedClientId = result.GetValue("clId");
                            }
                            if (loadedClientId != null && loadedClientId != Utils.DEFAULT_CLIENT_ID && loadedClientId != "null") {
                                _s.clientId = loadedClientId;
                                _s.log("Setting the client id to " + loadedClientId + " (from local storage)");
                            } else {
                                _s.log("Failed to load the client id from local storage");
                            }
                            if (_s.clientIdLoadingDone != null) {
                                _s.clientIdLoadingDone.DispatchEvent();
                                _s.clientIdLoadingDone = null;
                            }
                        },
                        "utils.fetchClientId onLoad");
            });
             _s._settings.platformApi.loadLocalData(onLoad);
        }

        /* 
        * This function is used to set the client id from a server response
        */ 
        defPubMeth(_s,"setClientIdFromServer",__setClientIdFromServer);
        function __setClientIdFromServer(newClientId /* : String */, fromStorage /* : Boolean */) /* : void */  {

            if (_s._loadClientIdTimeout != null) {
                _s._loadClientIdTimeout.cleanup();
                _s._loadClientIdTimeout = null;
            }
            _s.clientIdLoadingDone = null;
            if (_s.clientId == Utils.DEFAULT_CLIENT_ID) {
                _s.clientId = newClientId;
                _s.log("Setting the client id to " + newClientId + " (from server)");
                _s.writeClientId();
            }
        }

        // Write the current client ID to local storage, so that it can be
        // retrieved later by startFetchClientId() above.
        defPrivMeth(_s,"writeClientId",__writeClientId);
        function __writeClientId() /* : void */  {
            var dataToSave /* : DictionaryCS(string, string) */ = new DictionaryCS/*<string, string>*/();
            dataToSave.SetValue("clId", _s.clientId);
            var onSaved /* : Function(Action(bool)) */ = undefined;
            onSaved = (
                function (fSuccess /* : Boolean */) { 
                    _s.runProtected(
                        function () { 
                            if (!fSuccess) {
                                _s.err("An error occurred while saving the clientId.");
                            }
                        }, "utils.writeClientId onSaved");
            });
            _s.jsonEncode(Lang.StringDictionaryToRepr(dataToSave),
                function (dataStr /* : String */) { 
                    _s._settings.platformApi.saveLocalData(dataStr, onSaved);
                }); 
        }

       
        

        defPubMeth(_s,"randInt",__randInt);
        function __randInt() /* : int */  {
            return Math.floor(Math.random()*64000);
        }

        /// <summary>
        /// Send HTTP request
        /// 
        /// The callback function must be invoked after the operation
        /// </summary>
        /// <param name="isPOST">true if this is a POST request; false for GET</param>
        /// <param name="url">URL of the request</param>
        /// <param name="data">The data to be sent (ignored for GET)</param>
        /// <param name="contentType">The content type (if null, the default will be "application/json")</param>
        /// <param name="callback">Callback function to be invoked. May be null. The callback will receive the following parameters:
        ///               * a boolean representing success of the operation
        ///               * a String representing data in the response
        /// </param>
        defPubMeth(_s,"httpRequest",__httpRequest);
        function __httpRequest(isPOST /* : Boolean */, url /* : String */, data /* : String */, contentType /* : String */, callback /* : Function(Action(bool, string)) */) /* : Cleanable */  {
            var cleanable /* : Cleanable */ = new Cleanable(_s._cleanables);
            cleanable.setCleanupAction(_s._settings.platformApi.httpRequest(isPOST, url, data, contentType, 
                                                      function (isSuccess /* : Boolean */, response /* : String */) { 
                                                           cleanable.cleanup ();
                                                           if(callback != null) {
                                                                callback(isSuccess, response);
                                                               }
                                                      }));
            return cleanable;
        }

        /**
         * @obj must be a JSON encodable data structure in the native language
         * (i.e. not a Slash data structure like a DictionaryCS).  It must have
         * one of the following types:
         *   - dictionary
         *   - array or list
         *   - string
         *   - number
         *   - boolean
         *   - null
         * @callback is called with the JSON result when encoding is done. This
         * asynchronous interface is provided because encoding may take too
         * long for a single handler on some platforms.
         */
        defPubMeth(_s,"jsonEncode",__jsonEncode);
        function __jsonEncode(obj /* : Object */, callback /* : Function(Action(string)) */) /* : void */  {
            _s._settings.platformApi.jsonEncode(obj, callback);
        }


        defPubMeth(_s,"jsonDecode",__jsonDecode);
        function __jsonDecode(json /* : String */) /* : DictionaryCS(string, object) */  {
            return _s._settings.platformApi.jsonDecode(json);
        }


        /** 
         * Start measuring time-to-completion for computation @name.
         * Calls to this must be paired with a corresponding popMeasurement().
         */
        defPubMeth(_s,"pushMeasurement",__pushMeasurement);
        function __pushMeasurement(name /* : String */) /* : void */  {
            if (_s._settings.enablePerformanceMeasurements) {
                var now /* : Number */ = _s.epochTimeMs();
                _s._measurementStack.Add(new Measurement(name, now));
            }
        }

        /** 
         * Finish measuring time-to-completion for the computation
         * at the top of the current measurement stack.
         */
        defPubMeth(_s,"popMeasurement",__popMeasurement);
        function __popMeasurement() /* : void */  {
            if (_s._settings.enablePerformanceMeasurements && _s._measurementStack.Count > 0) {
                var idx /* : int */ = _s._measurementStack.Count - 1;
                var measurement /* : Measurement */ = _s._measurementStack.GetValue(idx);
                _s._measurementStack.RemoveAt(idx);
                var now /* : Number */ = _s.epochTimeMs();
                _s.log(Lang.ToString(measurement.name) + " took " + (now - measurement.startTimeMs) + "ms.");
            }
        }
       //JSRENAME:constant:_s:Utils:DEFAULT_CLIENT_ID
       //JSRENAME:constant:_s:Utils:MAX_SIZE_LOGBUFFER
        if(_s != STAT_INIT) _constr.apply(_s, arguments);
    }
statInit(Utils,"Utils");

// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-
// Copyright Conviva Inc 2010
// Author: George Necula (necula@conviva.com)

function Lang() {
    Lang.StringIndexOf = function (s1, s2) {
        return s1.indexOf(s2);
    };

    Lang.StringStartsWith = function (s1, s2) {
        return (0 == s1.indexOf(s2));
    };

    Lang.StringContains = function (s1, s2) {
        return (0 <= s1.indexOf(s2));
    };

    Lang.StringGetChar = function (s, where) {
        if(where < 0 || where >= s.length) {
            throw new Error("ArgumentOutOfRange");
        }
        return s[where];
    };

    Lang.StringSubstring = function (str, startIndex, count) {
        if(startIndex < 0 || startIndex >= str.length || (count != undefined && (count < 0 || startIndex + count > str.length))) {
            throw new Error("ArgumentOutOfRange");
        }
        if(count == undefined) {
            return str.substr(startIndex);
        } else {
            return str.substr(startIndex, count);
        }
    };

    Lang.StringSplit = function (s1, sep) {
        var res = s1.split(sep);
        return ArrayCS.FromRepr(res);
    };

    Lang.StringEnumerator = function (s) {
        return s.split("");
    };

    Lang.StringCompareTo = function (str1, str2) {
        if(str1 == null) {
            if(str2 == null) return 0;
            return -1;
        } 
        if(str2 == null) return 1;
        // both are non-null
        if(str1 < str2) return -1;
        if(str1 == str2) return 0;
        return 1;
    };

    Lang.StringTrim = function (s) {
        return s.replace(/^\s*/, "").replace(/\s*$/, "");
    };

    Lang.StringReplace = function (str1, strsearch, strreplace) {//NO_RENAME:substr,indexOf,length
        if(strsearch == null || strsearch == "" || strreplace == null) {
            throw new Error("InvalidArgument");
        }
        // We cannot use replace because we want to replace all occurrences
        var searchIdx = str1.indexOf(strsearch);
        if (searchIdx >= 0) {
            var searchLen = strsearch.length;
            // We have at least one occurrence
            return str1.substr(0, searchIdx) + strreplace + Lang.StringReplace(str1.substr(searchIdx + searchLen), strsearch, strreplace); 
        } else {
            return str1;
        }     
    };

    Lang.StringLastIndexOf = function (str1, strsearch) {//NO_RENAME:lastIndexOf
        if(strsearch == null || strsearch == "") {
            throw new Error("InvalidArgument");
        }
        return str1.lastIndexOf(strsearch);
    }

    Lang.ListFromRepr = function (a) {
        return ListCS.FromRepr(a);
    };

    /** Convert from a language-specific representation of an T[] to a T[] **/
    Lang.ArrayFromRepr = function (repr) {
        return ArrayCS.FromRepr(repr);
    };
    
        
        
    /** Convert from a T[] to a language-specific representation **/
    Lang.ArrayToRepr = function (a) {
        if(a == null) return null;
        return a.ToRepr();
    };

    Lang.ListFromRepr = function (repr) {
        return ListCS.FromRepr(repr);
    };

    /** Convert from a List<> to a language-specific representation **/     
    Lang.ListToRepr = function (l) {
        if(l == null) return null;
        return l.ToRepr();
    };

    /** Convert from a language-specific representation of a T[] to the array **/
    Lang.DictionaryFromRepr = function (repr) {
	var tmp = DictionaryCS.FromRepr(repr);
        //return DictionaryCS.FromRepr(repr);
        return tmp;
    };

    /** Convert from a T[] to a language-specific representation. For AS3 this is a Object, to be indexed with strings **/
    Lang.StringDictionaryToRepr = function (dict) {
        if(dict == null) return null;
        if(dict.hasOwnProperty(__id("ToObject"))) {
            return dict.ToObject();
        } else {
            return dict; // already an Object
        }
    };
        
    Lang.DictionaryCopyToObject = function (dict, obj) {
        if(dict == null) return;
        dict.CopyToObject(obj);
    };

    /** 
     * Convert from a string to a language-independent representation.
     * For JS this does nothing, since there is only one kind of string.
     */
    Lang.StringFromRepr = function (s) {
        return s;
    }


    Lang.StringToXml = function (str) {
        try {//NO_RENAME:DOMParser,parseFromString,documentElement,ActiveXOBject,async,loadXML
            if (window.DOMParser) {
                var xmlobject = (new DOMParser()).parseFromString(str, "text/xml");
                return xmlobject.documentElement;
            } else {
                // IE
                var xmlobject = new ActiveXObject("Microsoft.XMLDOM");
                xmlobject.async="false";
                xmlobject.loadXML(str);
                return xmlobject.documentElement;
            }
        } catch(e) {
            return null;
        }
    };

    Lang.XmlToString = function (oXML) {
        try {//NO_RENAME:XMLSerializer,serializeToString,xml
            if (window.XMLSerializer) {
                return (new XMLSerializer()).serializeToString(oXML);
            } else { // IE
                return oXML.xml;
            }
        } catch(e) {
            return null;
        }
    };

    Lang.ToString = function (o) {
        // Perhaps we find the ToString method
        if(typeof(o.ToString) == "function") {
            return o.ToString ();
        } else {
            return o.toString ();
        }
    }

    Lang.StringToLower = function (s) {
        return s.toLowerCase();
    }

    Lang.StringToInt = function (s) {
        return parseInt(s);
    }

    Lang.AsDouble = function (v) {
        if (v instanceof Int64) {
            return v.asNumber;
        } else if(v instanceof UInt64) {
            return v.asNumber;
        } else {
            return Number(v);
        }
    }

    // First, recognize the 
    Lang.doubleRegex = new RegExp("^([+-]?[0-9]*\\.?[0-9]+)((e|E)[+-]?[0-9]+)?$");
    Lang.parseDouble = function (v) {
        // This function needs to parse the same strings that C# double.Parse parses
        // In Javascript parsing an invalid string return NaN
        Lang.parseChecker(v, Lang.doubleRegex, "double");
        return parseFloat(v);
    }

    Lang.parseChecker = function (s, pattern, what) {
        if (!pattern.test(s)) {
            throw new Error("Invalid string for "+what+": "+s);
        }
    }
}
statInit(Lang,"Lang");

// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-
// Copyright Conviva Inc 2010
// Author: George Necula (necula@conviva.com)

function ArrayCS() {
    var _s = this;

    if(_s != STAT_INIT) _s.arr = undefined;
    function _constr(size) {
        if(size == undefined) size = 0;
        _s.arr = new Array(size);
    }

    defStatMeth(_s,ArrayCS,"Create",__Create);
    function __Create() {
        var res = new ArrayCS();
        var l = [ ];
        // it seems that we have to copy the arguments array instead
        // of using it directly
        for (var i = 0; i < arguments.length; i++) {
            l.push(arguments[i]);
        }
        res.arr = l;
        return res;
    }

    defStatMeth(_s,ArrayCS,"FromRepr",__FromRepr);
    function __FromRepr(a) {
        if (a == null) return null;
        var res = new ArrayCS();
        res.arr = a;
        return res;
    }

    defPubMeth(_s,"ToRepr",__ToRepr);
    function __ToRepr() {
        return _s.arr;
    }

    
    defGet(_s,"Length",__Length);
    function __Length() {
        return _s.arr.length;
    }


    defPubMeth(_s,"GetValue",__GetValue);
    function __GetValue(idx) {
        if(idx >= _s.arr.length) {
            throw new Error("Index out of bounds: "+idx+" (length "+_s.arr.length+")");
        } else if(idx < 0) {
            throw new Error("Index out of bounds: "+idx);
        }
        return _s.arr[idx];
    }

    defPubMeth(_s,"SetValue",__SetValue);
    function __SetValue(idx, v) {
        if(idx >= _s.arr.length) {
            throw new Error("Index out of bounds: "+idx+" (length "+_s.arr.length+")");
        } else if(idx < 0) {
            throw new Error("Index out of bounds: "+idx);
        }
        _s.arr[idx] = v;
    }

    defGet(_s,"Values",__Values);
    function __Values() {
        return _s.arr;
    }
    
    if(_s != STAT_INIT) _constr.apply(this, arguments);

}
statInit(ArrayCS,"ArrayCS");

// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-
// Copyright Conviva Inc 2010
// Author: George Necula (necula@conviva.com)

function ListCS() {
    var _s = this;

    if(_s != STAT_INIT) _s.arr = undefined;
    function _constr() {
        // Inherit from ArrayCS
        if (arguments.length > 1) {
            Ping.Send("Error: Instantiate ListCS with too many arguments");
        } else if (arguments.length == 0) {
                ArrayCS.call(_s, 0);
        } else if (arguments[0] instanceof Number) {
                ArrayCS.call(_s, arguments[0]);
        } else  if (arguments[0] instanceof Array) {
                ArrayCS.call(_s, arguments[0].length);
                _s.arr = arguments[0];
        } else   if (arguments[0] instanceof ArrayCS) {
                ArrayCS.call(_s, arguments[0].length);
                _s.arr = arguments[0].arr;
        } else {
            Ping.Send("Error: Instantiate ListCS with inappropriate arguments");
        }
    }

    defStatMeth(_s,ListCS,"Create",__Create);
    function __Create(a) {
        var res = new ListCS();
        for(var i=0;i<arguments.length;i++) {
            res.arr.push(arguments[i]);
        }
        return res;
    }

    defStatMeth(_s,ListCS,"FromRepr",__FromRepr);
    function __FromRepr(a) {
        if (a == null) {
            return a;
        }
        if (a instanceof ListCS) {
            return a;
        }
        var res = new ListCS();
        res.arr = a;
        return res;
    }

    defPubMeth(_s,"ToRepr",__ToRepr);
    function __ToRepr() {
        return _s.arr;
    }

    defGet(_s,"Count",__Count);
    function __Count() {
        return _s.arr.length;
    }
    
    defGet(_s,"Values",__Values);
    function __Values() {
        return _s.arr;
    }
    
    defPubMeth(_s,"Add",__Add);
    function __Add(e) {
        _s.arr.push(e);
    }

    defPubMeth(_s,"Clear",__Clear);
    function __Clear(e) {
        _s.arr.length = 0;
    }

    defPubMeth(_s,"IndexOf",__IndexOf);
    function __IndexOf(e) {
        var startIndex = arguments[1];
        if (startIndex == null) {
            startIndex = 0;
        } else if (startIndex < 0 || startIndex >= _s.arr.length) {
            throw new Error("Starting index out of bound");
        }

        for(var i=startIndex;i<_s.arr.length;i++) {
            if(_s.arr[i] == e) return i;
        }
        return -1;
    }

    defPubMeth(_s,"Contains",__Contains);
    function __Contains(e) {
        return _s.IndexOf(e) >= 0;
    }

    defPubMeth(_s,"Insert",__Insert);
    function __Insert(idx, e) {
        _s.arr.splice(idx, 0, e);
    }

    defPubMeth(_s,"Remove",__Remove);
    function __Remove(e) {
        var idx = _s.IndexOf(e);
        if(idx < 0) return false;
        _s.RemoveAt(idx);
        return true;
    }

    defPubMeth(_s,"RemoveRange",__RemoveRange);
    function __RemoveRange(where, count) {
        _s.arr.splice(where, count);
    }

    defPubMeth(_s,"RemoveAt",__RemoveAt);
    function __RemoveAt(where) {
        _s.arr.splice(where, 1);
    }

    defPubMeth(_s,"GetRange",__GetRange);
    function __GetRange(startidx, len) {
        var res = new ListCS();
        res.arr = _s.arr.slice(startidx, startidx + len); 
        return res;
    }

    defPubMeth(_s,"Sort",__Sort);
    function __Sort() {
        _s.arr.sort.apply(_s.arr, arguments);
    }

    defPubMeth(_s,"ToArray",__ToArray);
    function __ToArray() {
        return ArrayCS.FromRepr(_s.arr.slice());
    }

    if(_s != STAT_INIT) _constr.apply(this, arguments);

}
statInit(ListCS,"ListCS");

// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-
// Copyright Conviva Inc 2010
// Author: George Necula (necula@conviva.com)

function DictionaryCS() {
    var _s = this;

    if(_s != STAT_INIT) _s.obj = undefined;
    function _constr(size) {
        _s.obj = { };
    }

    defStatMeth(_s,DictionaryCS,"FromRepr",__FromRepr);
    function __FromRepr(o) {
        if(o == null) return null;
        if(o instanceof DictionaryCS) {
            return o;
        }
        if(o.hasOwnProperty(__id("ToObject"))) { // It is a DictionaryCS from the module
            o = o.ToObject ();
        }
        var res = new DictionaryCS ();
        // It must be an Object
        slForEachProp(o, function (k) {
            res.obj[k] = o[k];
        });
        return res;    
    }

    defPubMeth(_s,"ToObject",__ToObject);
    function __ToObject() {
        return _s.obj;
    }

    // Create a dictionary from a string of key, values, key ...
    defStatMeth(_s,DictionaryCS,"Create",__Create);
    function __Create() {
        var res = new DictionaryCS ();
        for(var i = 0; i + 1 < arguments.length; i += 2) {
            res.obj[arguments[i]] = arguments[i + 1];
        }
        return res;
    };
    
    /* Copy a dictionary to an object. Fails if called on a dictionary whose keys are not string */
    defPubMeth(_s,"CopyToObject",__CopyToObject);
    function __CopyToObject(obj) {
        slForEachProp(_s.obj, function (k) {
                obj[k] = _s.obj[k];
            });
    }
        
    defPubMeth(_s,"GetValue",__GetValue);
    function __GetValue (key) {
        return _s.obj[key];
    }

    defPubMeth(_s,"SetValue",__SetValue);
    function __SetValue (key, v) {
        _s.obj[key] = v;
    }

        /* Dictionary<>.Clear () */
    defPubMeth(_s,"Clear",__Clear);
    function __Clear( ) {
        slForEachProp(_s.obj, function (p) {
                delete _s.obj[p];
            });
    }
    
    /* Dictionary<>.ContainsKey */
    defPubMeth(_s,"ContainsKey",__ContainsKey);
    function __ContainsKey( key ) {
        return (_s.obj[key] !== undefined);
    }

    defPubMeth(_s,"Contains",__Contains);
    function __Contains( key ) {
        return ContainsKey(key);
    }
    
    /* Dictionary<>.Keys  is the collection of Keys, represented as KeyCollection */
    defGet(_s,"Keys",__Keys);
    function __Keys () {                                    
        var res = new Array ();
        slForEachProp(_s.obj, function (p) {
                res.push(p);
            });
        return res;
    }

    /* Dictionary<>.Values  is the collection of values */
    defGet(_s,"Values",__Values);
    function __Values () {                                    
        var res = new Array ();
        slForEachProp(_s.obj, function (p) {
                res.push(_s.obj[p]);
            });
        return res;
    }

        
    defGet(_s,"KeyValuePairs",__KeyValuePairs);
    function __KeyValuePairs () {                                    
        var res = new Array ();
        slForEachProp(_s.obj, function (p) {
                res.push( new KeyValuePairCS( p, _s.obj[p] ));
            });
        return res;
    }
        
    
    defGet(_s,"Count",__Count);
    function __Count () {
        var res = 0;
        slForEachProp(_s.obj, function (p) {
                res ++;
            });
        return res;       
    }

        
    /** Dictionary<>.Add: add an item */
    defPubMeth(_s,"Add",__SetValue);
                            
        /** Dictionary<>.Remove: delete an item, return if removed */
    defPubMeth(_s,"Remove",__Remove);
    function __Remove(key) {
        if(_s.ContainsKey(key)) {
            delete _s.obj[key];
            return true;
        } else
            return false;               
    }

    if(_s != STAT_INIT) _constr.apply(this, arguments);

}
statInit(DictionaryCS,"DictionaryCS");

// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-
// Copyright Conviva Inc 2010
// Author: George Necula (necula@conviva.com)

function KeyValuePairCS() {
    var _s = this;

    if(_s != STAT_INIT) _s.key = undefined;
    if(_s != STAT_INIT) _s.val = undefined;
    function _constr(key, val) {
        _s.key = key;
        _s.val = val;
    }

    
    defGet(_s,"Key",__Key);
    function __Key() {
        return _s.key;
    }

    defGet(_s,"Value",__Value);
    function __Value() {
        return _s.val;
    }
    if(_s != STAT_INIT) _constr.apply(this, arguments);

}
statInit(KeyValuePairCS,"KeyValuePairCS");

var JSON;JSON||(JSON={});
(function(){function k(a){return a<10?"0"+a:a}function o(a){p.lastIndex=0;return p.test(a)?'"'+a.replace(p,function(a){var c=r[a];return typeof c==="string"?c:"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4)})+'"':'"'+a+'"'}function l(a,j){var c,d,h,m,g=e,f,b=j[a];b&&typeof b==="object"&&typeof b.toJSON==="function"&&(b=b.toJSON(a));typeof i==="function"&&(b=i.call(j,a,b));switch(typeof b){case "string":return o(b);case "number":return isFinite(b)?String(b):"null";case "boolean":case "null":return String(b);case "object":if(!b)return"null";
e+=n;f=[];if(Object.prototype.toString.apply(b)==="[object Array]"){m=b.length;for(c=0;c<m;c+=1)f[c]=l(c,b)||"null";h=f.length===0?"[]":e?"[\n"+e+f.join(",\n"+e)+"\n"+g+"]":"["+f.join(",")+"]";e=g;return h}if(i&&typeof i==="object"){m=i.length;for(c=0;c<m;c+=1)typeof i[c]==="string"&&(d=i[c],(h=l(d,b))&&f.push(o(d)+(e?": ":":")+h))}else for(d in b)Object.prototype.hasOwnProperty.call(b,d)&&(h=l(d,b))&&f.push(o(d)+(e?": ":":")+h);h=f.length===0?"{}":e?"{\n"+e+f.join(",\n"+e)+"\n"+g+"}":"{"+f.join(",")+
"}";e=g;return h}}if(typeof Date.prototype.toJSON!=="function")Date.prototype.toJSON=function(){return isFinite(this.valueOf())?this.getUTCFullYear()+"-"+k(this.getUTCMonth()+1)+"-"+k(this.getUTCDate())+"T"+k(this.getUTCHours())+":"+k(this.getUTCMinutes())+":"+k(this.getUTCSeconds())+"Z":null},String.prototype.toJSON=Number.prototype.toJSON=Boolean.prototype.toJSON=function(){return this.valueOf()};var q=/[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
p=/[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,e,n,r={"\u0008":"\\b","\t":"\\t","\n":"\\n","\u000c":"\\f","\r":"\\r",'"':'\\"',"\\":"\\\\"},i;if(typeof JSON.stringify!=="function")JSON.stringify=function(a,j,c){var d;n=e="";if(typeof c==="number")for(d=0;d<c;d+=1)n+=" ";else typeof c==="string"&&(n=c);if((i=j)&&typeof j!=="function"&&(typeof j!=="object"||typeof j.length!=="number"))throw Error("JSON.stringify");return l("",
{"":a})};if(typeof JSON.parse!=="function")JSON.parse=function(a,e){function c(a,d){var g,f,b=a[d];if(b&&typeof b==="object")for(g in b)Object.prototype.hasOwnProperty.call(b,g)&&(f=c(b,g),f!==void 0?b[g]=f:delete b[g]);return e.call(a,d,b)}var d,a=String(a);q.lastIndex=0;q.test(a)&&(a=a.replace(q,function(a){return"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4)}));if(/^[\],:{}\s]*$/.test(a.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g,"@").replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g,
"]").replace(/(?:^|:|,)(?:\s*\[)+/g,"")))return d=eval("("+a+")"),typeof e==="function"?c({"":d},""):d;throw new SyntaxError("JSON.parse");}})();
// Copyright (c) 2009-2012, Conviva Inc. All Rights Reserved.

// ALBAN: on SEF support
// this class would only use SEF API GetSystemVersion, which I could never get to return anything but -1 during emulator/tv tests
// hence we choose not to support passing a SEF object to initialize this class, sticking with the nnavi object
// HBO-EU app seems to use nnavi plugin for other features regardless
// and this allows us to seamlessly push an update without modifying the integration guide
// note that we sometimes observed erratic behavior opening multiple SEF sessions on the same SEF object
// which is another reason not to change the current behavior for this class
// as this would require either investigation the erratic behavior further, or
// asking the customer to include another SEF object in the page just for to initialize this class

function SamsungApi () {
    var _s = this;
    var _fileName = 'conviva.txt';
    var _fileSystemObj = new FileSystem();
    var _persistentData = null;

    function _constr(nnaviPlugin) {
       _s._nnaviPlugin = nnaviPlugin;
    }
    /// <summary>
    /// Save data on persistent storage
    /// callback must be invoked after the operation.
    /// </summary>
    /// <param name="data">a string representing data to be saved</param>
    /// <param name="callback">Callback function to be invoked. Pass in true if success, false otherwise</param>
    this.saveLocalData = function(data, callback) {
        _persistentData = data;
        //save persistentData to file
        var succ = false;
        if(_fileSystemObj) {
            var fileObj = _fileSystemObj.openCommonFile(_fileName, 'w');
            fileObj.writeLine(_persistentData);
            _fileSystemObj.closeCommonFile(fileObj);
            succ = true;
        }
        if(typeof callback != undefined &&
          callback != null) {
            callback(succ);
        }
    }

    /// <summary>
    /// Retrieve data from persistent storage
    /// callback must be invoked after the operation.
    /// </summary>
    /// <param name="callback">callback function to be invoked. 
    /// Two parameters are required: 
    /// * a boolean representing success of the operation
    /// * a string representing the retrieved data
    /// </param>
    this.loadLocalData = function(callback) {
        if(_persistentData == null) {//read data from file
            _loadData(callback);    
        } else {
            callback(true, _persistentData);
        }
    }
   
    var _loadData = function(cbk) {
        var succ = false;
        if(_fileSystemObj && _fileSystemObj.isValidCommonPath(_fileName)) {
            var fileObj = _fileSystemObj.openCommonFile(_fileName, 'r');
            var d = fileObj.readLine();
            _fileSystemObj.closeCommonFile(fileObj);
            _persistentData = d;
            succ = true;
        } 
        if(typeof cbk != undefined &&
            cbk != null) {
            cbk(succ, _persistentData);
        }
    }

    /// <summary>
    /// Send HTTP request
    /// 
    /// The callback function must be invoked after the operation
    /// </summary>
    /// <param name="isPOST">true if this is a POST request; false for GET</param>
    /// <param name="url">URL of the request</param>
    /// <param name="data">The data to be sent (ignored for GET)</param>
    /// <param name="contentType">The content type (if null, the default will be "application/json")</param>
    /// <param name="callback">Callback function to be invoked
    /// Two parameters are required: 
    /// * a boolean representing success of the operation
    /// * a String representing data in the response
    /// </param>
    /// Returns a function that can be called to cancel and cleanup the request. The function may be called 
    /// multiple times. 
    this.httpRequest = function (isPOST, url, data, contentType, callback) {
        if(isPOST) {
            //samsung browser does not use *same origin policy*
            //simply send out Ajax POST request
            var xhr = new XMLHttpRequest();
            xhr.onreadystatechange = function() {
                try {
                    // PD-9228: we implement application-level redirect to support Touchstone
                    // 2012+ Samsung TVs should support it but don't (tested on 2012)
                    if (xhr.readyState == 4 && xhr.status == 307) {
                        var newUrl = xhr.getResponseHeader("Location");
                        _s.consoleLog("Redirecting to " + newUrl);
                        if (newUrl) {
                            _s.httpRequest(isPOST, newUrl, data, contentType, callback);
                        }
                    } else if (xhr.readyState == 4 && xhr.status == 200) {
                        var strResponse = xhr.responseText;
                        _s.consoleLog(strResponse);
                        callback(true, strResponse);
                    } else if (xhr.readyState == 4) {
                        callback(false, null);
                    }
                } catch(err) {
                    _s.consoleErr(err);
                }
            };
            xhr.open("POST", url, true);
            this.consoleLog(data);
            xhr.send(data);
        } else {
            //samsung browser does not use same origin policy
            //simply send out Ajax GET request
            var xhr = new XMLHttpRequest();
            xhr.open("GET", url, true);
            xhr.send(null);
        }
        return null; // Cannot cancel the request
    }


    /// <summary>
    /// Return the current time in milliseconds since Unix epoch 
    /// </summary>
    /// <returns></returns>
    this.epochTimeMs = function () {
        var d = new Date();
        return d.getTime();
    }

    /// <summary>
    /// Start a periodic timer
    /// 
    /// <param name="timerAction">The Action to run periodicaly</param>
    /// <param name="intervalMs">The interval in ms</param>
    /// <param name="actionName">The name of the action (for debugging)</param>
    /// </summary>
    /// <returns>A function that can be called to cancel/cleanup the timer</returns>
    this.createTimer = function (timerAction, intervalMs, actionName) {
        var wrapperAction = function () {
            Utils.getInstance().runProtected(timerAction, actionName);
        };
        var timerId = setInterval(timerAction, intervalMs);
        return (function () { 
            if(timerId != -1) {
                clearInterval(timerId); 
                timerId = -1;
            }});
    }

    /// <summary>
    /// Encode an object to JSON (asynchronously). This
    /// asynchronous interface is provided because encoding may take too
    /// long for a single handler on some platforms.
    /// </summary>
    /// <param name="obj">must be a JSON encodable data structure in the native language
    /// (i.e. not a Slash data structure like a DictionaryCS).  It must have
    /// one of the following types:
    ///   - dictionary
    ///   - array or list
    ///   - string
    ///   - number
    ///   - boolean
    //   - null</param>
    /// <param name="callback">Called when the encoding is done</param>
    this.jsonEncode = function (obj, callback) {
        var result = JSON.stringify(obj);
        callback(result);
    }

    /// <summary>
    /// Decode a string to a Dictionary. The values in a dictionary will be the same
    /// type as the JSON encodable values documented in jsonEncode
    /// </summary>
    /// <param name="json"></param>
    /// <returns></returns>
    this.jsonDecode = function (json) {
        var parsedJson = json;
        if(typeof(json) == 'string') {
            parsedJson = JSON.parse (json);
        }
        var result = Lang.DictionaryFromRepr(parsedJson);
        return result;
    }

    /// <summary>
    /// Debugging message logging
    /// </summary>
    /// <param name="message"> message to be logged</param>
    this.consoleLog = function(message) {
        if(typeof console != 'undefined' && console.log) {
            console.log(message);
			// ALBAN:
			// apparently console exists on SamsungTVs but console.log() traces don't
			// get sent back to the IDE, adding alert() here as well for QE
			alert(message);
        } else {
            alert(message);
        }
		
    }

    /// <summary>
    /// Error message logging
    /// </summary>
    /// <param name="errMsg">error message to be logged</param>
    this.consoleErr = function(errMsg) {
        _s.consoleLog("Error: " + errMsg);
    }

    /// <summary>
    /// Specify whether the Conviva code should allow uncaught exceptions. 
    /// This can be used on some platforms to check if the debugger is attached. In that
    /// case it may help to allow exceptions to propagate. 
    /// </summary>
    /// <returns></returns>
    this.allowUncaughtExceptions = function () {
        return false;
    }

    /// <summary>
    /// Return a string that names the device. 
    /// Maximum length is 64 bytes.
    /// Return null if not available
    /// </summary>
    this.getDevice = function() {
        return 'samsung';
    }

    /// <summary>
    /// Return a string representing the version number of the device. 
    /// Maximum length is 64 bytes.
    /// Return null if not available
    /// </summary>
    this.getDeviceVersion = function() {
        return '';
    }

    /// <summary>
    /// Return a string representing the type of the device.
    /// Maximum length is 64 bytes. Should be selected from the following list: 
    /// * PC
    /// * Mobile
    /// * Console
    /// * Settop
    /// </summary>
    this.getDeviceType = function() {
        return Conviva.ConvivaContentInfo.DEVICE_TYPE_SET_TOP_BOX;
    }

    /// <summary>
    /// Return a string naming the operating system in which the session is running. 
    /// Maximum length is 64 bytes. Must be in upper case. 
    /// Should be selected from the following list:
    /// * MAC
    /// * WIN
    /// * IOS
    /// * XBOX
    /// * SAMSUNG
    /// * PS3
    /// * AND (Android)
    /// * UNIX (for any *nix other than Android, including Linux).
    /// If none of these options apply, choose an appropriate name.
    /// </summary>
    this.getOS = function() {
        return 'SAMSUNG';
    }

    /// <summary>
    /// A string indicating the full version number of the operating system indicated under "os", if any.
    /// null if not available
    /// </summary>
    this.getOSVersion = function () {
        if(_s._nnaviPlugin != null) {
            return _s._nnaviPlugin.GetSystemVersion(0);
        }
        return null;
    }
    
    this.getPlatformMetadata = function () {
        var res = { 'sch' : 'html5.sam1' }
        if(_s._nnaviPlugin != null) {
            res['sv'] = _s._nnaviPlugin.GetSystemVersion(0);
        }
    }

    /// <summary>
    /// Return the current time in milliseconds since Unix epoch
    /// </summary>
    this.epochTimeMs = function () {
        return (new Date()).getTime();
    }

    if(_s != STAT_INIT) _constr.apply(_s, arguments);
}
registerName(SamsungApi, 'SamsungApi');
// -*- Mode: java; tab-width: 4; indent-tabs-mode: nil -*-

// Copyright Conviva Inc 2012

function SamsungStreamer (videoElement) {
    var _videoElement = videoElement;
    var _streamerType = 'Samsung';

    this.Cleanup = function() {
        _onPauseCbk = null;
        _onResumeCbk = null;
        _onPrerollStartCbk = null;
        _onPrerollCompleteCbk = null;
    }

    this.getStreamerType = function() {
        return _streamerType;
    }
    this.getVideoElement = function() {
        return _videoElement;
    }

    var _onPauseCbk = null;
    var _onResumeCbk = null;
    var _onPrerollStartCbk = null;
    var _onPrerollCompleteCbk = null;
    
    this.setOnPauseCbk = function(cbk) {
        _onPauseCbk = cbk;
    }
    
    this.setOnResumeCbk = function(cbk) {
        _onResumeCbk = cbk;
    }

    this.setOnPrerollStartCbk = function(cbk) {
        _onPrerollStartCbk = cbk;
    }

    this.setOnPrerollCompleteCbk = function(cbk) {
        _onPrerollCompleteCbk = cbk;
    }

    /** Call this function whenever player is paused
    */
    this.onPause = function() {
        if(_onPauseCbk != null) {
            _onPauseCbk();
        }
    }

    /** Call this function whenever player is resumed
    * from pause state
    */
    this.onResume = function() {
        if(_onResumeCbk != null) {
            _onResumeCbk();
        }
    }
    
    /** Call this function when preroll ad starts
    */
    this.onPrerollStart = function() {
        if(_onPrerollStartCbk != null) {
            _onPrerollStartCbk();
        }
    }
    
    /** Call this function when preroll ad
    * completes 
    */
    this.onPrerollComplete = function() {
        if(_onPrerollCompleteCbk != null) {
            _onPrerollCompleteCbk();
        }
    }
}
registerName(SamsungStreamer, 'SamsungStreamer');
// Copyright (c) 2009-2012, Conviva Inc. All Rights Reserved.

// ALBAN: on SEF support
// - we detect a SEF videoElement in the constructor by checking its "classid" attribute, then set _s._isSef
// - in startMonitoring we set up a different event listener than the normal ones when in SEF mode
// - _callSamsungApi wrapper takes care of calling Samsung APIs regardless of the current protocol, based on the value of _s._isSef
// - in endMonitoring we clean up our new event listener as well when in SEF mode

// ALBAN: on overwriting listeners
// whichever code overwrites the listeners last will be triggered first
// meaning since Conviva code is often initialized last, we're slowing down everything else
// and if we fail to execute the previous listener properly, we potentially break everything else
// also, even if we attempt to restore listeners on cleanup, we have no guarantee that we won't bypass some other module, unless we really were the last module to overwrite the listeners
// ie A hooks its listener, we come in, save A's, hook in our own, B comes in with its own hook later on, if we exit at this point, we will restore A's and thus bypass B's

var streamerType = 'Samsung';
var sId = 0;

function SamsungStreamerProxy() {
    var _s = this;

    /** StreamerProxy between Monitor and samsung Streamer
    * @param samsungStreamer [SamsungStreamer] instance of samsungStreamer
    * @param videoElement[DeviceAPI object] plugin player
    */
    function _constr(samsungStreamer, videoElement) {
        ConvivaStreamerProxy.call(_s);
        _s._streamerState = new StreamerStateManager();
        _s._realStreamer = samsungStreamer;
        _s._videoElement = videoElement;
        _s._utils = Utils.getInstance();
        _s._id = sId;
        sId++;

		 // GetCurrentBitrates/GetAvailableBitrates won't work before Play() is called (hopefully StartPlayback() works as well)
		 // So we will never be able to detect whether we can get a bitrate here, unless we move the session creation code after Play(), which we never want to do
		 // Choices:
		 // - keep a hardcoded value of 7 if we can confirm with HBO-EU that they'll be using HLS or smt in production, so we're confident we'll get bitrate, send 0 as bitrate if we get an invalid one somehow
		 // - keep a hardcoded value of 3, and update the bitrate later if we get it, hopefully the backend still processes it
		_s.capabilities = 7; // VIDEO + QUALITY + BITRATES

        // ALBAN: transitioning to STOPPED state immediately
        // not really anything to initialize that we can expect to have available upon starting the session
        _s._currState = PlayerStates.UNINITIALIZED;/*String*/
        _stateChanged(PlayerStates.STOPPED);

        _s._bitrate = 0;
        _s._durationSec = 0;
        _s._isPlayingPreroll = false;

        /* Monitor player state */
        _s._pollingTimer /*Timer*/ = null;
        _s._playheadTimeLastUpdatedAt = 0;
        
        // checking for classid="clsid:SAMSUNG-INFOLINK-SEF"
        // not sure if values like "clsid:SOME-OTHER-ID,SAMSUNG-INFOLINK-SEF" are possible
        // checking for "SAMSUNG-INFOLINK-SEF" feels safer
        _s._isSef = (_s._videoElement.getAttribute("classid").indexOf("SAMSUNG-INFOLINK-SEF") != -1);
        if(_s._isSef) { _s._videoElement.Open("SamsungStreamerProxy","0.1","nocreds"); }
    }
    
    var _callSamsungApi = function () {
        var funcName = arguments[0];
        var args = [];
        for (var i = 1; i < arguments.length; i++) {
            args.push(arguments[i]);
        }
        if (_s._isSef) {
            args.unshift(funcName);
            return _s._videoElement.Execute.apply(_s._videoElement,args);
        } else {
            return _s._videoElement[funcName].apply(_s._videoElement,args);
        }
    }
    
    defPubMeth(_s,"GetStreamerStateManager",__GetStreamerStateManager);
    function __GetStreamerStateManager() /* : StreamerStateManager*/  {
        return _s._streamerState;
    }

    defPubMeth(_s,"Cleanup",__Cleanup);
    function __Cleanup() {
        _s._utils.log('SamsungStreamerProxy.Cleanup() starts');
        if(_s._realStreamer != null) {
            _s._realStreamer.Cleanup();
            _s._realStreamer = null;
        }

        _endMonitoring();

        if(_s._isSef) { _s._videoElement.Close(); }
        
        _s._videoElement = null;
        _s._utils.log('SamsungStreamerProxy.Cleanup() ends');
    }

    defPubMeth(_s,"GetCapabilities",__GetCapabilities);
    function __GetCapabilities() {
        return _s.capabilities;
    }

    // ALBAN: new intermediary function to declare errors
    // maps specific error code strings to the provided parameter received from the player
    // then calls the usual _declareError function with the adjusted error code
    var _declareDetailedError = function(errCode, type) {
        if (type) {
            if (errCode == SamsungStreamerProxy.RenderError) {
                if (isNaN(type) || type < 0 || type >= SamsungStreamerProxy.RenderErrorCodes.length) { type = 0; }
                errCode += ' - ' + SamsungStreamerProxy.RenderErrorCodes[type];
            }
        }
        _declareError(errCode);
    }

    var _declareError = function(errCode) {
        _s._streamerState.OnError(_convertMediaError(errCode));
        // ALBAN: switch to ERROR state upon declaring an error
        _stateChanged(PlayerStates.ERROR);
    }

    var _convertMediaError = function(errCode) {
        _s._streamerState.Log("SamsungStreamerProxy._convertMediaError: code " + errCode);
        return StreamerError.makeUnscopedError(errCode, StreamerError.SEVERITY_FATAL);
    }

    var _metaDataChanged = function(encodedFps) {
        var o = {framerate: encodedFps};
        _s._streamerState.OnMetadata(o);
    }
    
    var _durationChanged = function(durationSec) {
        var o = {duration: durationSec};
        _s._streamerState.OnMetadata(o);
    }

    var _bitrateChanged = function(btr) {
        if(btr != _s._bitrate) {
            _s._bitrate = btr;
            var streamInfo = new StreamInfo(btr, ConvivaContentInfo.CDN_NAME_OTHER, null);
            _s._streamerState.SetStream(streamInfo);
        }
    }

    var _stateChanged = function(state) {
        if(_s._isPlayingPreroll) {//pause monitoring
            return;
        }
        if(state != _s._currState) {
            _s._prevState = _s._currState;
            _s._currState = state;
            _s._streamerState.SetPlayingState(state);
			// ALBAN:
			// moved this so it only logs on actual state changes
			_s._utils.log('SamsungStreamerProxy.stateChanged: ' + state);
        } 
    }

    /** The position of the play head. To be overriden */
    defPubMeth(_s,"GetPlayheadTimeMs",__GetPlayheadTimeMs);
    function __GetPlayheadTimeMs() /* : uint */  {
        return _s._playheadTimeLastUpdatedAt;
    }

    /* Monitor Player State and bitrate */

    /** If video is playing, OnCurrentPlayTime is fired
    * within less than 1 sec. So we can determine
    * the current state is playing if the PlayheadTime is
    * updated within 1.5 sec. FF or RW will also
    * fire this event, but FF and RW are not considered
    * as independent states, so we can ignore them
    */
    var _pollPlayerState = function() {
        if(_s._currState == PlayerStates.BUFFERING) {
            //buffering: state is already buffering. we'll wait for buffering complete event
            return;
        } else {
            var d = new Date();
            if(d.getTime() - _s._playheadTimeLastUpdatedAt < 1500) {//playing || rewind || fastforward
                var offset = _updatedPlayHeadTime - _lastPlayHeadTime;
                // ALBAN: including 0, don't want to infer playing state from PHT not moving
                if(offset <= 0 || offset > 2000) {//rewind || fastforward
                    return;
                }
                _stateChanged(PlayerStates.PLAYING);
            } else {//pause state
                // ALBAN: ideally we don't want to infer paused state at the start
                // two 0 PHT samples just means we didn't start yet.
                // since QE says STOPPED > BUFFERING > STOPPED > PAUSE > PLAYING chain isn't a problem on initial start
                // we leave it as is for now
                _stateChanged(PlayerStates.PAUSED);
            }
        }
    }

    var _pollBitrate = function() {
        var btr = Math.round(_callSamsungApi("GetCurrentBitrates")/1000);
        if(btr >= 0) {
            _bitrateChanged(btr)
        }
    }

    var _pollContentLen = function() {
        if(_s._durationSec > 0) {//we already have a valid duration
            return;
        } else if(_callSamsungApi("GetDuration") > 0) {//valid duration in ms
            _s._durationSec = Math.round(_callSamsungApi("GetDuration")/1000);
            _durationChanged(_s._durationSec);
        }
    }

    var _pollPlayer = function () {
        _pollPlayerState();
        _pollBitrate();
        _pollContentLen();
    }

    var _initPolling = function() {
        if(_s._pollingTimer != null) {
            _s._pollingTimer.cleanup();
        }
        //poll player every 1 sec for 
        //updated player state
        _s._pollingTimer = _s._utils.createTimer(_pollPlayer, 1000, '_pollingPlayer');
    }
    
    /** clear polling interval and restart it
    * after the delay (MS)
    * @param delay [int] Delay in milliseconds
    */
    var _restartPolling = function(delay) {
        if(_s._pollingTimer != null) {
            _s._pollingTimer.cleanup();
        }
        _s._pollingTimer = _s._utils.createTimer(_initPolling, delay|0, '_initPolling');
    }

    var _setStreamerCbks = function() {
        if(_s._realStreamer == null) {
            return;
        }
        _s._realStreamer.setOnPauseCbk(function() {
            _stateChanged(PlayerStates.PAUSED);
            //reset the monitor timer to avoid setting state
            //to Playing mistakenly. Such mistake can be made
            //if we poll the player state immediately
            //after the streamer reports the Pause state and the playheadtime
            //is updated within 1.5 sec. So,
            //we need to reset the polling timer to postpone checking
            //playheadtime
            _restartPolling(1500);
        });

        _s._realStreamer.setOnResumeCbk(function() {
            _stateChanged(PlayerStates.PLAYING);
        });
        _s._realStreamer.setOnPrerollStartCbk(function() {
            _s._isPlayingPreroll = true;
        });
        _s._realStreamer.setOnPrerollCompleteCbk(function() {
            _s._isPlayingPreroll = false;
        });
    }
    
    /**To listen to an event from DeviceAPI, we need to 
    * overwrite the event listener.
    */
    var _origErrorHandlers = {}; /* event's name [String] -> original handler's Name [String] */
    var _overwriteSamsungErrorEventsHandler = function(evtName/* String */, convivaHandler/*Closure*/, convivaHandlerParam) {
        if(SamsungStreamerProxy.MonitorErrorEvents == null) {
            SamsungStreamerProxy.MonitorErrorEvents = { };
        }
        if(_s._videoElement[evtName] != null) {
            _s._utils.log('SamsungStreamerProxy._overwriteSamsungErrorEventsHandler: store orig handler for event ' + evtName);
            _origErrorHandlers[evtName] = _s._videoElement[evtName];
        }
        // ALBAN: renamed 'param' to 'param1' for clarity
        // maybe some day we will need to use param2 as well
        SamsungStreamerProxy.MonitorErrorEvents[evtName+_s._id] = function(param1) {
            _s._utils.runProtected(function() {
                // ALBAN: now passing a second parameter to the conviva handler
                // Due to javascript not really caring how many arguments we provide,
                // legacy handlers that don't expect this parameter won't be affected
                convivaHandler(convivaHandlerParam, param1);
                if(_origErrorHandlers.hasOwnProperty(evtName) &&
                    _origErrorHandlers[evtName] != null) {
                    _s._utils.log(_origErrorHandlers[evtName] + '(' + param1 + ')');
                    eval(_origErrorHandlers[evtName] + '(' + param1 + ')');
                }
            }, "Conviva.MonitorErrorEvents."+(evtName+_s._id));
        };
        _s._videoElement[evtName] /* :String */ = 'Conviva.MonitorErrorEvents.'+(evtName+_s._id);
    }

    var _resetMonitorErrorEvents = function() {
        for (var evtName in _origErrorHandlers) {
            _s._utils.log('restore: ' + evtName + ' ' + _origErrorHandlers[evtName]);
            if(_origErrorHandlers.hasOwnProperty(evtName)) {
                _s._videoElement[evtName] = _origErrorHandlers[evtName];
            }
        }
        _origErrorHandlers = {};
        _s._utils.log('SamsungStreamerProxy._resetMonitorErrorEvents() ends');
    }
    
    var _initMonitorErrorEvents = function() {
        _overwriteSamsungErrorEventsHandler(
            'OnRenderError',
            // ALBAN: redirecting here to include RenderError parameter data
            _declareDetailedError,
            SamsungStreamerProxy.RenderError
        );
        _overwriteSamsungErrorEventsHandler(
            'OnNetworkDisconnected',
            _declareError,
            SamsungStreamerProxy.NetworkDisconnected 
        );
        _overwriteSamsungErrorEventsHandler(
            'OnConnectionFailed',
            _declareError,
            SamsungStreamerProxy.ConnectionFailed
        );
        _overwriteSamsungErrorEventsHandler(
            'OnAuthenticationFailed',
            _declareError,
            SamsungStreamerProxy.AuthenticationFailed
        );
        _overwriteSamsungErrorEventsHandler(
            'OnStreamNotFound',
            _declareError,
            SamsungStreamerProxy.StreamNotFound
        );
        _overwriteSamsungErrorEventsHandler(
            'OnBufferingStart',
            function() { // ALBAN: removed unused parameter
                _s._utils.log('SamsungStreamerProxy.OnBufferingStart');
                _stateChanged(PlayerStates.BUFFERING);
            },
            null
        );
        _overwriteSamsungErrorEventsHandler(
            'OnBufferingComplete',
            function() { // ALBAN: removed unused parameter
                _s._utils.log('SamsungStreamerProxy.OnBufferingComplete');
                _stateChanged(_s._prevState);
            },
            null
        );
    };

    var _origOnCurrentPlayTime /*String*/= null;
    var _lastPlayHeadTime = 0;
    var _updatedPlayHeadTime = 0;
    
    var OnCurrentPlayTime = function(time) {
        var d = new Date();
        _s._playheadTimeLastUpdatedAt = d.getTime();
        if(time != _updatedPlayHeadTime) {
            _lastPlayHeadTime = _updatedPlayHeadTime;
            _updatedPlayHeadTime = time;
        }
        if(_origOnCurrentPlayTime != null) {
            eval(_origOnCurrentPlayTime+'('+time+')');
        }
    };
    
    var _initMonitorPlayheadTime = function() {
        _s._playheadTimeLastUpdatedAt = 0;
        _lastPlayHeadTime = 0;
        _updatedPlayHeadTime = 0;
        _origOnCurrentPlayTime = null;
        if(SamsungStreamerProxy.MonitorPlayheadTime == null) {
            SamsungStreamerProxy.MonitorPlayheadTime = {};
        }
        if(_s._videoElement.OnCurrentPlayTime != null) {
            _origOnCurrentPlayTime = _s._videoElement.OnCurrentPlayTime;
        }
        SamsungStreamerProxy.MonitorPlayheadTime['OnCurrentPlayTime' + _s._id] = OnCurrentPlayTime;
        _s._videoElement.OnCurrentPlayTime = 'Conviva.MonitorPlayheadTime.OnCurrentPlayTime'+_s._id;
    };
    
    var _resetMonitorPlayheadTime = function() {
        if(_origOnCurrentPlayTime != null) {
            _s._videoElement.OnCurrentPlayTime = _origOnCurrentPlayTime;
            _origOnCurrentPlayTime = null;
        }
        _s._utils.log('SamsungStreamerProxy._resetMonitorPlayheadTime() ends');
    };
    
    /** ALBAN: SEF events handling
    * SEF seems to never trigger listeners specific to a single event
    * So we hook into all events and react to the ones we need
    */
    
    var _origOnEvent /*String*/= null;
    
    var _initOnEventHandler = function() {
        _origOnEvent = null;
        if(SamsungStreamerProxy.MonitorAllEvents == null) {
            SamsungStreamerProxy.MonitorAllEvents = {};
        }
        if(_s._videoElement.OnEvent != null) {
            _origOnEvent = _s._videoElement.OnEvent;
        }
        SamsungStreamerProxy.MonitorAllEvents['OnEvent' + _s._id] = OnEvent;
        _s._videoElement.OnEvent = 'Conviva.MonitorAllEvents.OnEvent'+_s._id;
        _s._utils.log('SamsungStreamerProxy._initOnEventHandler() ends');
    };
    
    var OnEvent = function (eventId, param1, param2) {
        try {
            switch (eventId) {
                // ALBAN: adding logs and reformatting this part
                case 1:
                    // OnConnectionFailed (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnConnectionFailed');
                    _declareError(SamsungStreamerProxy.ConnectionFailed);
                    break;
                case 2:
                    // OnAuthenticationFailed (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnAuthenticationFailed');
                    _declareError(SamsungStreamerProxy.AuthenticationFailed);
                    break;
                case 3:
                    // OnStreamNotFound (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnStreamNotFound');
                    _declareError(SamsungStreamerProxy.StreamNotFound);
                    break;
                case 4:
                    // OnNetworkDisconnected (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnNetworkDisconnected');
                    _declareError(SamsungStreamerProxy.NetworkDisconnected);
                    break;
                case 6:
                    // OnRenderError (source: dev testing)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnRenderError');
                    _declareDetailedError(SamsungStreamerProxy.RenderError, param1);
                    break;
                case 8:
                    // OnRenderingComplete (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnRenderingComplete');
                    _stateChanged(PlayerStates.STOPPED);
                    break;
                case 9: break; // OnStreamInfoReady (source: HBO-EU app)
                case 11:
                    // OnBufferingStart (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnBufferingStart');
                    _stateChanged(PlayerStates.BUFFERING);
                    break;
                case 12:
                    // OnBufferingComplete (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnBufferingComplete');
                    _stateChanged(_s._prevState);
                    break;
                case 13: break; // OnBufferingProgress (source: HBO-EU app)
                case 14:
                    // OnCurrentPlayTime (source: HBO-EU app)
                    _s._utils.log('SamsungStreamerProxy.OnEvent:OnCurrentPlayTime');
                    OnCurrentPlayTime(param1);
                    break;
                default:
                    _s._utils.log('SamsungStreamerProxy.OnEvent: unhandled player event ' + eventId + ' / ' + param1 + ' / ' + param2);
                    break;
            }
        }
        catch (e) {
            _s._utils.log('SamsungStreamerProxy.onEvent error: ' + e.message);
        }
        if(_origOnEvent != null) {
            try {
                var evalstring = _origOnEvent + '(' + eventId;
                if (param1) { evalstring += ',' + param1; }
                if (param2) { evalstring += ',' + param2; }
                evalstring += ')';
                eval(evalstring);
            } catch (e) {
                _s._utils.log('SamsungStreamerProxy.onEvent error: failed to execute previous handler: ' + e.message);
            }
        }
    };

    var _resetOnEventHandler = function() {
        if(_s._videoElement != null && _origOnEvent != null) {
            _s._videoElement.OnEvent = _origOnEvent;
            _origOnEvent = null;
        }
        _s._utils.log('SamsungStreamerProxy._resetOnEventHandler() ends');
    };

    this.startMonitoring = function() {
        /*set all cbks of samsungStreamer*/
        // ALBAN: QE requested some logs to be notified of monitoring
        _s._utils.log('SamsungStreamerProxy.startMonitoring() starts');
        _setStreamerCbks();
        if (_s._isSef) {
            _s._utils.log('SamsungStreamerProxy.SEF monitoring mode');
            _initOnEventHandler();
        } else {
            _s._utils.log('SamsungStreamerProxy.NNAVI monitoring mode');
            _initMonitorPlayheadTime();
            _initMonitorErrorEvents();
        }
        _initPolling();
        _s._utils.log('SamsungStreamerProxy.startMonitoring() ends');
    };

    var _endMonitoring = function() {
        _s._utils.log('SamsungStreamerProxy.endMonitoring() starts');
        //stop _pollingTimer
        if(_s._pollingTimer != null) {
            _s._pollingTimer.cleanup();
        }
        _s._pollingTimer = null;
        if (_s._isSef) {
            _resetOnEventHandler();
        } else {
            _resetMonitorErrorEvents();
            _resetMonitorPlayheadTime();
        }
        _s._utils.log('SamsungStreamerProxy.endMonitoring() ends');
    };
    /* End of Monitor player */

    if(_s != STAT_INIT) _constr.apply(_s, arguments);
}
statInit(SamsungStreamerProxy,"SamsungStreamerProxy");

SamsungStreamerProxy.createSamsungStreamerProxy = function(samsungStreamer) {
    if(samsungStreamer == null || 
        typeof samSungStreamer == undefined ||
        !samsungStreamer.hasOwnProperty('getStreamerType') ||
        samsungStreamer.getStreamerType() != 'Samsung' ||
        !samsungStreamer.hasOwnProperty('getVideoElement') ||
        samsungStreamer.getVideoElement() == null) {
        return null;//error: streamer obj must be defined and have a valid videoElement
    } else {
        var samsungStreamerProx = new SamsungStreamerProxy(samsungStreamer, samsungStreamer.getVideoElement());
        samsungStreamerProx.startMonitoring();
        return samsungStreamerProx;
    }
}

SamsungStreamerProxy.MonitorErrorEvents = { };
SamsungStreamerProxy.MonitorPlayheadTime = { };
SamsungStreamerProxy.MonitorAllEvents = { };

// ALBAN: from Samsung DeviceAPI / SEF documentation
// added "Unknown" for ease of use and to handle cases where we get an unexpected parameter
SamsungStreamerProxy.RenderErrorCodes /* Array */ = ["Unknown", "Unsupported container", "Unsupported video codec", "Unsupported audio codec", "Unsupported video resolution"]

SamsungStreamerProxy.RenderError /* String */ = 'RenderError';
SamsungStreamerProxy.NetworkDisconnected /* String */ = 'NetworkDisconnected';
SamsungStreamerProxy.ConnectionFailed /* String */ = 'ConnectionFailed';
SamsungStreamerProxy.AuthenticationFailed /* String */ = 'AuthenticationFailed';
SamsungStreamerProxy.StreamNotFound /* String */ = 'StreamNotFound';
registerName(SamsungStreamerProxy.MonitorErrorEvents, 'MonitorErrorEvents');
registerName(SamsungStreamerProxy.MonitorPlayheadTime, 'MonitorPlayheadTime');
registerName(SamsungStreamerProxy.MonitorAllEvents, 'MonitorAllEvents');
Conviva.LivePass = ConvivaPrivateLoader.LivePass; Conviva.ConvivaContentInfo = ConvivaPrivateLoader.ConvivaContentInfo; Conviva.StreamerError = ConvivaPrivateLoader.StreamerError; Conviva.StreamerStateManager = ConvivaPrivateLoader.StreamerStateManager; Conviva.ConvivaStreamerProxy = ConvivaPrivateLoader.ConvivaStreamerProxy; Conviva.Settings = ConvivaPrivateLoader.Settings; Conviva.StreamInfo = ConvivaPrivateLoader.StreamInfo; Conviva.SamsungStreamer = ConvivaPrivateLoader.SamsungStreamer; Conviva.MonitorErrorEvents = ConvivaPrivateLoader.MonitorErrorEvents; Conviva.MonitorPlayheadTime = ConvivaPrivateLoader.MonitorPlayheadTime; Conviva.MonitorAllEvents = ConvivaPrivateLoader.MonitorAllEvents; Conviva.SamsungApi = ConvivaPrivateLoader.SamsungApi;
};
